/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Base/Utils.cpp,v $
 * $Date: 2003/05/13 18:41:56 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "PICompat.h"
#include "Utils.h"

/*____________________________________________________________________________*\
 *
 Global declarations:
\*____________________________________________________________________________*/

/* ---
	ltoa type function do not use more than 64 bytes
--- */

enum { BUF_SIZE=63 };
#define ALLOC_SZBUF(x)	char *szBuf=PI_NEW( char[(x)+1] );
#define FREE_SZBUF	PI_DELETE( [] szBuf );

#if defined(__BORLANDC__) || defined(_MSC_VER)

#if defined(_MSC_VER)
#	define strncmpi _strnicmp	// MSVC4.2
#endif

/* --- WINDOWS -- */

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
bool pi_ltoa( long l, int iBase, PIString &sResult )
{
	ALLOC_SZBUF(BUF_SIZE);
	*szBuf = '\0';
	bool bRet=ltoa(l, szBuf, iBase)!=0;
	if ( bRet ) sResult=szBuf;
	FREE_SZBUF;
	return bRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
bool pi_ultoa( unsigned long ul, int iBase, PIString &sResult )
{
	ALLOC_SZBUF(BUF_SIZE);
	*szBuf = '\0';
	bool bRet=ultoa(ul, szBuf, iBase)!=0;
	if ( bRet ) sResult=szBuf;
	FREE_SZBUF;
	return bRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int pi_stricmp( const char *pS1, const char *pS2 )
{
	return stricmp( pS1, pS2 );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int pi_strncmpi( const char *pS1, const char *pS2, int iLen )
{
	return strncmpi( pS1, pS2, iLen );
}

#else

/* --- UNIX --- */

/* --- 
prototype strcasecmp, strncasecmp in case are not prototyped
in header files
--- */
extern int strcasecmp( const char *, const char * );
extern int strncasecmp( const char *, const char *, size_t );

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
bool pi_ltoa( long l, int iBase, PIString &sResult )
{
	ALLOC_SZBUF(BUF_SIZE);
	switch( iBase )
		{
		case 8:
			sprintf( szBuf, "%lo", l ); break;
		case 10:
			sprintf( szBuf, "%ld", l ); break;
		case 16:
			sprintf( szBuf, "%lx", l ); break;
		default:
			FREE_SZBUF;
			return false;
		};

	sResult=szBuf;
	FREE_SZBUF;
	return true;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
bool pi_ultoa( unsigned long ul, int iBase, PIString &sResult )
{
	ALLOC_SZBUF(BUF_SIZE);
	switch( iBase )
		{
		case 8:
			sprintf( szBuf, "%lo", ul ); break;
		case 10:
			sprintf( szBuf, "%lu", ul ); break;
		case 16:
			sprintf( szBuf, "%lx", ul ); break;
		default:
			FREE_SZBUF;
			return false;
		};

	sResult=szBuf;
	FREE_SZBUF;
	return true;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int pi_stricmp( const char *pS1, const char *pS2 )
{
	return strcasecmp( pS1, pS2 );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int pi_strncmpi( const char *pS1, const char *pS2, int iLen )
{
	return strncasecmp( pS1, pS2, iLen );
}

#endif	/* UNIX */

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
bool pi_ldtoa( long double ld, PIString &sResult )
{
	ALLOC_SZBUF(BUF_SIZE);
	sprintf( szBuf, "%LG", ld );
	sResult=szBuf;
	FREE_SZBUF;
	return true;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Handles NULL by returning an empty string, however iStrLen must
	be 0 is pS is NULL.
\*____________________________________________________________________________*/
char *DuplicateString( const char *pS, int iStrLen )
{
	char *pNewS=PI_NEW( char[ iStrLen+1 ] );
	if ( iStrLen )
		{ strncpy( pNewS, pS, iStrLen ); };
	pNewS[iStrLen]='\0';
	return pNewS;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
char *DuplicateString( const char *pS )
{
	return DuplicateString( pS, pS ? strlen( pS ) : 0 );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int pi_min( int i, int j )
{
	if ( i<j ) return i;
	return j;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int pi_max( int i, int j )
{
	if ( j<i ) return i;
	return j;
}

