/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/EnhPi3/NonIP.cpp,v $
 * $Date: 2003/05/13 18:41:58 $
 *
 Description:
    Hastily thrown together configuration page for non-IP virtual hosts.

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <assert.h>

#include "DblList.h"
#include "PIString.h"

#include "resrc1.h"
#include "Dialogs.h"
#include "Config.h"
#include "PIUtil.h"

/*____________________________________________________________________________*\
 *
 Description:
	Definitions and global values
\*____________________________________________________________________________*/
static HWND hWndList = 0;
static HWND hWndHostname = 0;
static HWND hWndDocumentRoot = 0;
static HWND hWndAdd = 0;
static HWND hWndReplace = 0;
static HWND hWndDelete = 0;
static HWND hWndAdministrator = 0;
static DblList lHostnames;
static DblList lDocumentRoots;
static DblList lAdministrators;

#define SECTION     "VirtualHosts"
#define VARIABLE    "NonIP"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static int Internal_verifyFields( HWND hDlg, char *pszBuf, int iBufSize )
{
    int iToLen;

    /*
    ** Get the 'To' field
    */
    SendMessage( hWndDocumentRoot, WM_GETTEXT, iBufSize, (LPARAM)pszBuf );

    /*
    ** 'DocumentRoot' field must end with '/'
    */
    assert( *pszBuf );
    iToLen = strlen(pszBuf);
    if ( pszBuf[iToLen-1]!='/' && pszBuf[iToLen-1]!='\\' )
        {
        MessageBox( hDlg, "'DocumentRoot' field must end with a forward-slash ('/') \
or a backslash ('\\').",
            "Error in 'DocumentRoot' Field", MB_OK | MB_ICONEXCLAMATION );

        ::SetFocus( hWndDocumentRoot );
        ::SendMessage( hWndDocumentRoot, EM_SETSEL, (WPARAM)(INT)0, (LPARAM)(INT)-1 );

        goto verify_fail;
        };

    /*
    ** Good
    */
    return 1;

verify_fail:
    return 0;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_addLine( const char *pLine )
{
	assert( pLine );
	assert( lHostnames.Size()==lDocumentRoots.Size() );
	assert( lDocumentRoots.Size()==lAdministrators.Size() );

	int i, j;
    i = 0;
	for( ; pLine[i] && isspace(pLine[i]); i++ );
    j = i;
	for( ; pLine[i] && !isspace(pLine[i]); i++ );
	
	/* --- scan to tab --- */
	lHostnames.Append( (DblList::type)PI_NEW( PIString( &(pLine[j]), i-j)));

	/* --- scan over whitespace to start of second field --- */
	for( ; pLine[i] && isspace(pLine[i]); i++ );
    j = i;
	for( ; pLine[i] && !isspace(pLine[i]); i++ );
	lDocumentRoots.Append( (DblList::type)PI_NEW( PIString( &(pLine[j]), i-j)));	

	/* --- scan over whitespace to start of third field --- */
	for( ; pLine[i] && isspace(pLine[i]); i++ );
    j = i;
	for( ; pLine[i] && !isspace(pLine[i]); i++ );
	lAdministrators.Append( (DblList::type)PI_NEW( PIString( &(pLine[j]), i-j)));	

	assert( lHostnames.Size()==lDocumentRoots.Size() );
	assert( lDocumentRoots.Size()==lAdministrators.Size() );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_clearArrays()
{
	DblListIterator x( lHostnames );
	DblListIterator y( lDocumentRoots );
	DblListIterator z( lAdministrators );
	for( ; !x.BadIndex(); x++, y++, z++ )
		{
		assert( !x.BadIndex() && !y.BadIndex() && !z.BadIndex() );
		PI_DELETE( (PIString *)x.Current() );
		PI_DELETE( (PIString *)y.Current() );
		PI_DELETE( (PIString *)z.Current() );
		};
	lHostnames.Clear();
	lDocumentRoots.Clear();
	lAdministrators.Clear();
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_reloadArrays()
{
	enum { BUF_SIZE=2047 };
	char szBuf[BUF_SIZE+1];

	Internal_clearArrays();

	/*
	** Get number of items
	*/
	int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );

	/* 
	** Insert from listbox into internal arrays
	*/
	int i;
	for( i=0; i<iCount; i++ )
		{
		SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
		Internal_addLine( szBuf );
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_updateButtons( WPARAM wParam, LPARAM lParam,
    HWND hWndEdit1, HWND hWndEdit2, const char *pData )
{
	if ( GET_WM_COMMAND_CMD(wParam, lParam)!=EN_CHANGE )
		{ return; };

	/*
	** If the file extension changed then check if this is an existing
	** file extension
	*/
	int iExistingIndex = -1;
	assert( pData );
	int iChangeToIndex = -1;
	int iIndex = 0;
	if ( *pData )
		{
		for( DblListIterator i( lHostnames ); !i.BadIndex(); i++, iIndex++ )
			{
			PIString *pString = (PIString *)i.Current();
			assert( pString );
			if ( !PIUtil_strncmpi( (*pString), pData, strlen( pData )))
				{
				iChangeToIndex = iIndex;
				/*
				** If this is a full index, select it
				*/
				if ( !stricmp( (*pString), pData ) )
					{ iExistingIndex = iIndex; };
				break;
				};
			};
		};

	/*
	** Remove listbox selection or scroll to correct line
	** as appropriate 
	*/
	SendMessage( hWndList, LB_SETCURSEL, iChangeToIndex, 0 );

	/*
	** Set buttons
	*/
	if ( ::SendMessage( hWndEdit1, EM_LINELENGTH, 0, 0 )==0 ||
	    ::SendMessage( hWndAdministrator, EM_LINELENGTH, 0, 0 )==0 )
		{
		if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
		if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
		}
	else if ( SendMessage( hWndEdit2, EM_LINELENGTH, 0, 0 )!=0 )
		{
		if ( iExistingIndex>-1 )	/* --- disable add, enable replace --- */
			{
			if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
			if ( !IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, TRUE );
			}
		else						/* --- disable replace, enable add --- */
			{
			if ( !IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, TRUE );
			if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
			};
		};

	/*
	** If there is a selected item in the listbox then
	** enable delete, otherwise disable delete	
	*/
	if ( SendMessage( hWndList, LB_GETCURSEL, 0, 0 )==LB_ERR )
		{ EnableWindow( hWndDelete, FALSE ); }
	else
        { EnableWindow( hWndDelete, TRUE ); };
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
	LPARAM lParam)
{
	enum { BUF_SIZE=1023 };
	char szBuf[BUF_SIZE+1];
	int aTabstop[3];

	switch (uMsg)
		{
		case WM_INITDIALOG:
			AD_hourglassOn();
			{
			const char *pTmp;
			int iTmp;
			hWndList = GetDlgItem( hDlg, IDC_LIST );
			hWndHostname = GetDlgItem( hDlg, IDC_HOSTNAME );
			hWndDocumentRoot= GetDlgItem( hDlg, IDC_DOCUMENTROOT );
			hWndAdd = GetDlgItem( hDlg, IDC_ADD );
			hWndReplace = GetDlgItem( hDlg, IDC_REPLACE );
			hWndDelete= GetDlgItem( hDlg, IDC_DELETE );
			hWndAdministrator= GetDlgItem( hDlg, IDC_ADMIN );

			assert( hWndList && hWndHostname && hWndDocumentRoot );
			assert( hWndAdd && hWndReplace && hWndDelete && hWndAdministrator);

			EnableWindow( hWndAdd, FALSE );
			EnableWindow( hWndReplace, FALSE );
			EnableWindow( hWndDelete, FALSE );

			/*
			** Limit edit field lengths, really just a sanity thing
			*/
			SendMessage( hWndHostname, EM_SETLIMITTEXT, 256, 0 );
			SendMessage( hWndDocumentRoot, EM_SETLIMITTEXT, 256, 0 );
			SendMessage( hWndAdministrator, EM_SETLIMITTEXT, 256, 0 );

			/*
			** Setup tabstops
			*/
			aTabstop[0] = 0;
			aTabstop[1] = 75;
			aTabstop[2] = 150;
			SendMessage( hWndList, LB_SETTABSTOPS, 3, (LPARAM)&aTabstop );

			/*
			** Load all data into the listbox
			*/
			iTmp = 0;
			for(;; iTmp++ )
				{
				pTmp = AD_lookupValueEx( __pConfig, SECTION, 
					VARIABLE, iTmp );
				if ( !pTmp )
					{ break; };

				SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)pTmp );
				};
			Internal_reloadArrays();
			}
			AD_hourglassOff();
			break;

		case WM_DESTROY:
			{
			Internal_clearArrays();
			};
			break;

		case WM_COMMAND:
			switch (GET_WM_COMMAND_ID(wParam, lParam))
				{
				case IDC_ADD:
                    {
                    /*
                    ** Verify fields are valid
                    */
                    if ( !Internal_verifyFields( hDlg, szBuf, BUF_SIZE ) )
                        { break; };

					int i, j;
					*szBuf = '\0';
					SendMessage( hWndHostname, WM_GETTEXT, BUF_SIZE,
                        (LPARAM)szBuf );
					for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++ );
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lHostnames.Append( (DblList::type)PI_NEW(PIString(szBuf )));
					szBuf[i++] = '\t';

					SendMessage( hWndDocumentRoot, WM_GETTEXT, BUF_SIZE-i,
                        (LPARAM)&(szBuf[i]) );
					for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lDocumentRoots.Append( (DblList::type)PI_NEW(
                        PIString( &(szBuf[j]) ) ) );
					szBuf[i++] = '\t';
					SendMessage( hWndAdministrator, WM_GETTEXT, BUF_SIZE-i,
                        (LPARAM)&(szBuf[i]) );
					for(j=i; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++);
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lAdministrators.Append( (DblList::type)PI_NEW(
                        PIString( &(szBuf[j]) ) ) );

					SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );

					/*
					** Clear edit fields, focus to list
					*/
					SendMessage( hWndHostname, WM_SETTEXT, 0, (LPARAM)"" );
					SendMessage( hWndDocumentRoot, WM_SETTEXT, 0, (LPARAM)"" );
					SendMessage( hWndAdministrator, WM_SETTEXT, 0, (LPARAM)"" );
					Internal_reloadArrays();
					int iIndex = SendMessage( hWndList, LB_FINDSTRINGEXACT, 0, (LPARAM)szBuf );
					if ( iIndex!=LB_ERR ) SendMessage( hWndList, LB_SETTOPINDEX, iIndex, 0 );
					SetFocus( hWndList );
					AD_Changed( hDlg );
					};
					break;
	
				case IDC_REPLACE:
                    {
                    /*
                    ** Verify fields are valid
                    */
                    if ( !Internal_verifyFields( hDlg, szBuf, BUF_SIZE ) )
                        { break; };

					/*
					** Get selected item index 
					*/
					int iCurSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );

					/*
					** Get text to add and add it to internal lists only
					*/
					int i, j;
					*szBuf = '\0';
					SendMessage( hWndHostname, WM_GETTEXT, BUF_SIZE,
                        (LPARAM)szBuf );
					for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++ );
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lHostnames.Append( (DblList::type)PI_NEW(PIString(szBuf )));
					szBuf[i++] = '\t';

					SendMessage( hWndDocumentRoot, WM_GETTEXT, BUF_SIZE-i,
                        (LPARAM)&(szBuf[i]) );
					for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lDocumentRoots.Append( (DblList::type)PI_NEW(
                        PIString( &(szBuf[j]) ) ) );
					szBuf[i++] = '\t';
					SendMessage( hWndAdministrator, WM_GETTEXT, BUF_SIZE-i,
                        (LPARAM)&(szBuf[i]) );
					for(j=i; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++);
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lAdministrators.Append( (DblList::type)PI_NEW(
                        PIString( &(szBuf[j]) ) ) );

					/*
					** Delete the item from the list and internal lists
					*/
					SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
					PI_DELETE( (PIString *)lHostnames.Detach( iCurSel) );
					PI_DELETE( (PIString *)lDocumentRoots.Detach( iCurSel) );
					PI_DELETE( (PIString *)lAdministrators.Detach( iCurSel) );
					
					/*
					** Add new item to listbox
					*/
					SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );

					/*
					** Clear edit fields, focus to list
					*/
					SendMessage( hWndHostname, WM_SETTEXT, 0, (LPARAM)"" );
					SendMessage( hWndDocumentRoot, WM_SETTEXT, 0, (LPARAM)"" );
					SendMessage( hWndAdministrator, WM_SETTEXT, 0, (LPARAM)"" );
					Internal_reloadArrays();
					int iIndex = SendMessage( hWndList, LB_FINDSTRINGEXACT, 0,
                        (LPARAM)szBuf );
					if ( iIndex!=LB_ERR ) SendMessage( hWndList,
                        LB_SETTOPINDEX, iIndex, 0 );
					SetFocus( hWndList );
					AD_Changed( hDlg );
					};
					break;

				case IDC_DELETE:
					{
					int iCurSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
					SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
					PI_DELETE( (PIString *)lHostnames.Detach( iCurSel) );
					PI_DELETE( (PIString *)lDocumentRoots.Detach( iCurSel) );
					PI_DELETE( (PIString *)lAdministrators.Detach( iCurSel) );
			
					/*
					** Clear edit fields, focus to list
					*/
					SendMessage( hWndHostname, WM_SETTEXT, 0, (LPARAM)"" );
					SendMessage( hWndDocumentRoot, WM_SETTEXT, 0, (LPARAM)"" );
					SendMessage( hWndAdministrator, WM_SETTEXT, 0, (LPARAM)"" );
					SetFocus( hWndList );
					AD_Changed( hDlg );
					};
					break;

				case IDC_HOSTNAME:
					*szBuf = '\0';
					SendMessage( hWndHostname, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf ); 
					Internal_updateButtons( wParam, lParam, hWndHostname, 
						hWndDocumentRoot, szBuf );
					break;

				case IDC_DOCUMENTROOT:
					*szBuf = '\0';
					SendMessage( hWndHostname, WM_GETTEXT, BUF_SIZE,(LPARAM)szBuf );
					Internal_updateButtons( wParam, lParam, hWndDocumentRoot, 
						hWndHostname, szBuf );
                    break;

				case IDC_ADMIN:
					*szBuf = '\0';
					SendMessage( hWndHostname, WM_GETTEXT, BUF_SIZE,(LPARAM)szBuf );
					Internal_updateButtons( wParam, lParam, hWndHostname, 
						hWndDocumentRoot, szBuf );
					break;

				case IDC_LIST:
					if ( GET_WM_COMMAND_CMD(wParam, lParam)==LBN_SELCHANGE )
						{
						int iSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
						if ( iSel!=LB_ERR )
							{
							assert( iSel>=0 && iSel<lHostnames.Size() );
							EnableWindow( hWndDelete, TRUE );
							SendMessage( hWndHostname, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)lHostnames[iSel]) );
							SendMessage( hWndDocumentRoot, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)lDocumentRoots[iSel]) );
							SendMessage( hWndAdministrator, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)lAdministrators[iSel]) );
							};
						};
					break;

				default:;		
				}
			break;

		case WM_NOTIFY:
			switch( ((NMHDR *)lParam)->code )
				{
				case PSN_QUERYCANCEL:
					return AD_cancel( hDlg );

				case PSN_KILLACTIVE:
					{
					int i;
					int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );

					/*
					** Save all date
					*/
					/* --- remove old ones first --- */
					AD_deleteValue( __pConfig, SECTION, VARIABLE );
					for( i=0; i<iCount; i++ )
						{
						*szBuf = '\0';
						szBuf[BUF_SIZE] = '\0';
						SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
						AD_addValue( __pConfig, SECTION, VARIABLE, szBuf );
						};
					SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
					break;
					}

				default:
					return (FALSE);
				};
			break;

		default:
			return (FALSE);
		};

	return (TRUE);
}

/*____________________________________________________________________________*\
 *
 Description:
	Definitions and global values
\*____________________________________________________________________________*/
DLGPROC fnNonIP = (DLGPROC)DialogProc;

