/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/EnhPi3/PMain.cpp,v $
 * $Date: 2003/05/13 18:41:58 $
 *
 Description:
	This file is a bit of a mess while I sort out what sort of main window
	it should have.

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <assert.h>
#include <stdio.h>

#include "PIString.h"
#include "MiscUtil.h"

#include "resrc1.h"
#include "Common.h"

/*____________________________________________________________________________*\
 *
 Description:
	Values for substitution
\*____________________________________________________________________________*/
#define AD_WNDCLASS	"Ad class"

/*____________________________________________________________________________*\
 *
 Description:
	Definitions and global values
\*____________________________________________________________________________*/
static HWND __hPropSheet = 0;
PTConfig __pConfig = 0;
static const char **__ppArgv = 0;
static const char *__pSemaphoreName = 0;
static HWND __hStatusWindow = 0;
static int __iChanged = 0;

#if defined(NDEBUG)
#	define AD_MUSTBETRUE(x)	x
#else
#	define AD_MUSTBETRUE(x)	assert(x)
#endif

#define TITLE		"Pi3Web Server Admin"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Returns 1 for true, 0 for false and -1 for failure.
\*____________________________________________________________________________*/
static int Internal_isServiceInstalled()
{
	const char *pServiceName = AD_lookupValue( __pConfig, "General", 
			"ServiceName" );
	return CTRL_isServiceInstalled( pServiceName );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Returns 1 for true and -1 for failure.
\*____________________________________________________________________________*/
static int Internal_installService()
{
	/*
	** Get command line of server
	*/
	enum { BUF_SIZE=1023 };
	char szBuf[BUF_SIZE+1];

	const char *pServiceName = AD_lookupValue( __pConfig, "General", 
			"ServiceName" );

	/*
	** Get commandline
	*/
	sprintf( szBuf, "%s\\bin\\%s", AD_SERVICE_EXECUTABLE,
		AD_lookupValue( __pConfig, "General", "InstallDir" ) );
	return CTRL_installService( pServiceName, szBuf );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Returns 1 for true and -1 for failure.
\*____________________________________________________________________________*/
static int Internal_unInstallService()
{
	const char *pServiceName = AD_lookupValue( __pConfig, "General", 
			"ServiceName" );
	return CTRL_unInstallService( pServiceName );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Start server with appropriate messages. Starts the server as either
	a system service or an application. Assumes the server is not already
	running.

	Returns 1 on success, and -1 on failure.
\*____________________________________________________________________________*/
static int Internal_startServer()
{
	/*
	** Get command line of server
	*/
	enum { BUF_SIZE=1023 };
	char szBuf[BUF_SIZE+1];

	int iRet;

	/*
	** see what the run mode of the server is 
	*/
	const char *pType = AD_lookupValue( __pConfig, "General", "RunMode" );
	if ( pType && strcmp( pType, "Desktop" ) )
		{
		/*
		** Service
		*/
		
		/*
		** Firstly, make sure the service is installed
		*/
		int iInstalled = Internal_isServiceInstalled();
		if ( iInstalled==-1 )
			{ return -1; };
		if ( !iInstalled )
			{
			if ( Internal_installService()!=1 )
				{ return -1; };
			};

		const char *pServiceName = AD_lookupValue( __pConfig, "General", 
			"ServiceName" );
		iRet = CTRL_startService( pServiceName );
		}
	else
		{
		/*
		** Desktop
		*/

		/*
		** Get commandline and semaphore name
		*/
		sprintf( szBuf, "%s /START %s", __ppArgv[0], __ppArgv[1] );
		iRet = CTRL_startPi3( szBuf, __pSemaphoreName );
		};

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Returns 1 on success, and -1 on failure.
\*____________________________________________________________________________*/
static int Internal_stopServer()
{
	int iRet;

	/*
	** see what the run mode of the server is 
	*/
	const char *pType = AD_lookupValue( __pConfig, "General", "RunMode" );
	if ( pType && strcmp( pType, "Desktop" ) )
		{
		/*
		** Service
		*/
		const char *pServiceName = AD_lookupValue( __pConfig, "General", 
			"ServiceName" );
		iRet = CTRL_stopService( pServiceName );
		}
	else
		{
		/*
		** Desktop
		*/

		/*
		** Get semaphore name
		*/
		iRet = CTRL_stopPi3( __pSemaphoreName );
		};

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Returns 1 for true, 0 for false and -1 for failure.
\*____________________________________________________________________________*/
static int Internal_isServerRunning()
{
	int iRet;

	/*
	** see what the run mode of the server is 
	*/
	const char *pType = AD_lookupValue( __pConfig, "General", "RunMode" );
	if ( pType && strcmp( pType, "Desktop" ) )
		{
		/*
		** Service
		*/
		const char *pServiceName = AD_lookupValue( __pConfig, "General", 
			"ServiceName" );
		iRet = CTRL_isServiceRunning( pServiceName );
		}
	else
		{
		/*
		** Desktop
		*/

		/*
		** Get semaphore name
		*/
		iRet = CTRL_isPi3Running( __pSemaphoreName );
		};

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Returns 1 for true and -1 for failure.
\*____________________________________________________________________________*/
static int Internal_applyChanges()
{
	/*
	** Is this a service or no?
	*/
	const char *pType = AD_lookupValue( __pConfig, "General", "RunMode" );
	int iService = ( pType && strcmp( pType, "Desktop" ) );

	/*
	** Is the server current running or no?
	*/
	int iRunning = Internal_isServerRunning();
	if ( iRunning==-1 )
		{ return -1; };

	/*
	** Stop the server if it is running
	*/
	if ( iRunning )
		{
		if ( Internal_stopServer()==-1 )
			{ return -1; };
		};

	/*
	** Write out configuration values
	*/
	AD_writeConfig( __pConfig, __ppArgv[1] );

	/*
	** If the server was running then start ip up again
	*/
	if ( iRunning ) 
		{
		if ( Internal_startServer()!=1 )
			{ return -1; };
		};

	return 1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void AD_Changed( HWND __hDlg ) {
	__iChanged = 1;
	PropSheet_Changed( GetParent( __hDlg ), __hDlg );
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void AD_UnChanged( HWND __hDlg ) {
	__iChanged = 0;
	PropSheet_UnChanged( GetParent( __hDlg ), __hDlg );
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int AD_IsChanged() { return __iChanged; };

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL AD_apply( HWND hDlg )
{
	if (AD_IsChanged())
		{
		AD_hourglassOn();
		int iSaved = Internal_applyChanges();
		AD_hourglassOff();

		if ( iSaved!=1 )
			{
			::MessageBox( GetParent( hDlg ), "Error saving configuration \
or restarting server.", "Error", MB_OK | MB_ICONSTOP );
			SetWindowLong( hDlg, DWL_MSGRESULT, PSNRET_INVALID );
			}
		else
			{
			SetWindowLong( hDlg, DWL_MSGRESULT, PSNRET_NOERROR );
			AD_UnChanged( hDlg );
			}
		}
	return (TRUE);
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL AD_cancel( HWND hDlg )
{
	if (AD_IsChanged())
		{
		int mbRes = MessageBox( GetParent( hDlg ), "Discard changes?",
			"Warning", MB_YESNOCANCEL | MB_ICONWARNING );
		switch (mbRes)
			{
			case IDYES:
				SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
				break;
			case IDNO:
				NMHDR nmhdr;
				memset((void *)&nmhdr, sizeof(nmhdr), 0);
				nmhdr.hwndFrom = hDlg;
				nmhdr.code = PSN_KILLACTIVE;
				/* ---
				Force to write back changes of property sheet page
				before safe configuration
				--- */
				SendMessage( hDlg, WM_NOTIFY, 0, (LPARAM)&nmhdr );
				AD_apply( hDlg );
				SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
				break;
			case IDCANCEL:
			default:
				SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );
			}
		}
	return (TRUE);
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Windowing loop with modeless dialog and tray icon.
\*____________________________________________________________________________*/
static int Internal_WindowsLoop( HINSTANCE hInstance )
{
	int i;
	int iSheet;

	/*
	** Register URL class
	*/
	AD_registerURLClass( hInstance );

	/*
	** Attempt to load configuration file
	*/
	assert( __ppArgv );
	__pConfig = AD_loadConfig( __ppArgv[1] );
	if ( !__pConfig)
		{
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
		sprintf( szBuf, "Unable to load configuration file '%s'",
			__ppArgv[1] );
		::MessageBox( NULL, szBuf, "Error", MB_OK );
		goto error;
		};

	/*
	** Set configuration values
	*/
	AD_replaceValue( __pConfig, "Startup", "SemaphoreName", __pSemaphoreName );
	AD_replaceValue( __pConfig, "Startup", "ConfigName", __ppArgv[1] );

	/*
	** Setup property sheet data structures
	*/
	PROPSHEETPAGE psp;
	PROPSHEETHEADER psh;
	HPROPSHEETPAGE hpsp[NUM_PAGES];

	memset( &psp, 0, sizeof( PROPSHEETPAGE ) );
	psp.dwSize = sizeof( PROPSHEETPAGE );
	psp.dwFlags = PSP_DEFAULT;
	psp.hInstance = hInstance;

	iSheet = 0;

	/* Status page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_STATUS );
	psp.pfnDlgProc = fnStatus;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* General page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_GENERAL );
	psp.pfnDlgProc = fnGeneral;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* Performance page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_PERFORMANCE );
	psp.pfnDlgProc = fnPerformance;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* Mapping page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_MAPPING );
	psp.pfnDlgProc = fnMapping;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* Mime types page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_MIMETYPES );
	psp.pfnDlgProc = fnMimeTypes;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* Users page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_USERS );
	psp.pfnDlgProc = fnUsers;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );
#if 0
	/* Documents page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_DOCUMENTS );
	psp.pfnDlgProc = fnDocuments;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );
#endif
	/* Logging page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_LOGGING );
	psp.pfnDlgProc = fnLogging;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* Directory Indexing */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_DIRINDEX );
	psp.pfnDlgProc = fnDirIndex;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* HTTP page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_HTTP );
	psp.pfnDlgProc = fnHttp;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* SSL page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_SSL );
	psp.pfnDlgProc = fnSSL;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* Supplementary IO Objects */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_IO );
	psp.pfnDlgProc = fnIO;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* Non-IP Virtual hosts page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_NONIP );
	psp.pfnDlgProc = fnNonIP;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/* IP Virtual hosts page */
	psp.pszTemplate = MAKEINTRESOURCE( IDD_IP );
	psp.pfnDlgProc = fnIP;
	hpsp[iSheet++] = CreatePropertySheetPage( &psp );

	/*
	** Validate all the property sheets were correctly setup
	*/
	for( i=0; i<iSheet; i++ )
		{
		if ( !(hpsp[i]) )
			{
			::MessageBox( NULL, "Internal error generating configuration",
				"Pi3Web Properties", MB_OK );
			goto error;
			};
		};

	/* setup property structure */
	memset( &psh, 0, sizeof( PROPSHEETHEADER ) );
	psh.dwSize = sizeof( PROPSHEETHEADER );
	psh.dwFlags = PSH_MODELESS | PSH_DEFAULT | PSH_USEICONID;
	psh.hInstance = hInstance;
	psh.hwndParent = NULL;
	psh.pszIcon = MAKEINTRESOURCE(IDI_PI3ADMIN);
	psh.nPages = iSheet;
	psh.phpage = hpsp;
	psh.pszCaption = TITLE;

	/* Create modeless property sheet */
	assert( iSheet<=NUM_PAGES );
	__hPropSheet = (HWND)PropertySheet( &psh );

	/* ---
	Wait for end of window
	--- */
	MSG tMsg;
	while( GetMessage( &tMsg, NULL, 0, 0 ) )
		{
		if ( !(PropSheet_IsDialogMessage( __hPropSheet, &tMsg )) )
			{ 
			TranslateMessage( &tMsg );
			DispatchMessage( &tMsg ); 
			}
		else
			{
			if (!PropSheet_GetCurrentPageHwnd(__hPropSheet))
				{
				::DestroyWindow( __hPropSheet );
				::ExitProcess( 0 );
				}
			}
		};

	/*
	** Unregister URL class
	*/
	AD_unregisterURLClass( hInstance );
	AD_destroyConfig( __pConfig );

	return 0;

error:
	AD_unregisterURLClass( hInstance );
	if ( __pConfig ) { AD_destroyConfig( __pConfig ); };

	return -1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static HCURSOR __hHourglass = 0;
EXTERN_C void AD_hourglassInit( void )
{
	__hHourglass = LoadCursor( 0, MAKEINTRESOURCE( IDC_WAIT ) );
	assert( __hHourglass );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
EXTERN_C void AD_hourglassDestroy( void )
{
	FreeResource( __hHourglass );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static HCURSOR __hSave = 0;
EXTERN_C void AD_hourglassOn( void )
{
	__hSave = SetCursor( __hHourglass );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
EXTERN_C void AD_hourglassOff( void )
{
	AD_MUSTBETRUE( SetCursor( __hSave ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Windows entry point for configuration
\*____________________________________________________________________________*/
int AD_configMain( HINSTANCE hInstance, int iArgc,
	const char *ppArgv[] )
{
	int iRet;

	/*
	** Check that argument 1 exists and is an existing configuration
	** file
	*/
	if ( iArgc<=1 )
		{
		/*
		** No configuration file specified
		*/
		::MessageBox( NULL, "No configuration file specified", "Error in \
parameters", MB_OK );
		return -1;
		};

	__ppArgv = ppArgv;
	assert( __ppArgv );

	/*
	** Semaphore name for application is the full
	** path to the configuration file in upper case
	*/
	PIString sSemaphoreName;
	CTRL_semaphoreNameFromConfigPath( __ppArgv[1], sSemaphoreName );
	__pSemaphoreName = sSemaphoreName;

	AD_hourglassInit();
	iRet = Internal_WindowsLoop( hInstance );
	AD_hourglassDestroy();

	return iRet;
}

