/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/HTTP/MIMEType.cpp,v $
 * $Date: 2003/05/13 18:42:03 $
 *
 Description:
	Handler to set the MIME type.
\*____________________________________________________________________________*/
//$SourceTop:$

#include "HandBase.h"
#include "HTTPCore.h"
#include "PIStrStr.h"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	MIMETypeByExtension

Description:
	Set the MIME Type of the resource identified by the current physical
	path by using the function HTTPCore_getMIMETypeFromExtension().
	<P>
	This looks up the MIME type using the file extension component of the
	physical path in the list of MIME types loaded into the server.

<!--
no options

Options:
<H5>Overview</H5>
<TABLE BORDER=1>
<TH>Option
<TH>Default
<TH>Values
<TH>Short Description
<TH>Example(s)

<TR>
<TD>RequirePermissions
<TD>-
<TD>Combination of 'X', 'R', 'X'
<TD>Permissions required on file
<TD>RequirePermissions="X"

</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>
<H5>
	Require Permissions
</H5>
These permissions will be compared against the permissions on the filesystem
resource to determine if this handler will indicate a match by returning
PIAPI_COMPLETED.
This value is obtained by concatenating any of the following flags together.
<CENTER>
<TABLE BORDER=1>
<TH>Option
<TH>Meaning
<TR>
<TD>R
<TD>Resource must be readable
<TR>
<TD>W
<TD>Resource must be writable
<TR>
<TD>X
<TD>Resource must be executable
</TABLE>
</CENTER>
Permission flags are not case sensitive.

-->

Phase:
	CHECKTYPE

Returns:
	PIAPI_CONTINUE in all cases.

Example:
	<PRE>
	&lt;Object&gt;
		Name MIMETypeByExtension
		Class MIMETypeByExtensionClass
	&lt;/Object&gt;

	&lt;Object&gt;
		...
		CheckType MIMETypeByExtension
		...
	&lt;/Object&gt;
	</PRE>
/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class MIMETypeByExtension : public HandlerBaseHTTP
{
private:
	/* ---
	Configuration data
	--- */

protected:
	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );
		PIOStrStream os;
		os << pWhere << "MIMETypeByExtension: ";
/*	No parameters */
/*		if ( !PIUtil_stricmp( KEY_CONF_DEFAULTMIME, pVariable ) ) */
		if ( 0 )
			{
			}
		else
			{
			os << "Undefined parameter '" << pVariable << "'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	MIMETypeByExtension( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	HandlerBaseHTTP( pObject )
		{
		ReadParameters( iArgc, ppArgv );
		};

	~MIMETypeByExtension()
		{
		};

	int Handle( int iPhase, PIHTTP &tPIHTTP, PIIOBuffer & )
		{
		if ( iPhase!=PH_CHECKTYPE )
			{ return PIAPI_ERROR; };

		const char *pPath = (const char *)PIDB_lookup( tPIHTTP.pResponseDB,
			PIDBTYPE_STRING, KEY_INT_PATH, 0 );
		PIFInfo *pFInfo = HTTPCore_getCachedFile( pPath );
		const char *pExtension = PIFInfo_getExtension( pFInfo );
		if ( pExtension )
			{
			const char *pMime=HTTPCore_getMIMETypeFromExtension( pExtension );
			PIDB_replace( tPIHTTP.pResponseDB, PIDBTYPE_RFC822,
				KEY_HTTP_CONTENTTYPE, (void *)pMime, 0 );
			HTTPCore_releaseCachedFile( pFInfo );
			return PIAPI_CONTINUE;
			};
		HTTPCore_releaseCachedFile( pFInfo );
		return PIAPI_CONTINUE;
		};
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int MIMETypeByExtension_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return HandlerBaseHTTP_constructor( pObj, PI_NEW( MIMETypeByExtension( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name MIMETypeByExtensionClass
		Type LogicExtension
		Library HTTP
		OnClassLoad HandlerBaseHTTP_onClassLoad
		Constructor MIMETypeByExtension_constructor
		CopyConstructor HandlerBaseHTTP_copyConstructor
		Destructor HandlerBaseHTTP_destructor
		Execute HandlerBaseHTTP_execute
	</Class>

	<Object>
		Name MIMETypeByExtension
		Class MIMETypeByExtensionClass
	</Object>

/*___+++CNF_END+++___*/
#endif

