/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/HTTP/SendFile.cpp,v $
 * $Date: 2003/06/07 19:37:15 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>
#include <ctype.h>
#include <fstream.h>

#include "HandBase.h"
#include "HTTPCore.h"
#include "HTTPUtil.h"
#include "PIStrStr.h"
#include "StrToken.h"
#include "PIString.h"
#include "DeQuote.h"
#include "PiAPI.h"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_HEADERPATTERN		"HeaderPattern"
#define KEY_CONF_FOOTERPATTERN		"FooterPattern"
#define KEY_CONF_METHODS			"Methods"
#define KEY_CONF_FILEKEY			"FileKey"
#define KEY_CONF_NOTEKEY			"NoteKey"
#define KEY_CONF_UPLOADPATH			"UploadPath"
#define KEY_CONF_LIMIT				"Limit"
#define KEY_CONF_DESCFILE			"DescriptionFile"
#define KEY_CONF_NOTE				"Annotation"

#define NOTE_EXT ".txt"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	SendFile

Description:
	Send a file including headers to a remote client. If this is a 'HEAD'
	request only the HTTP headers will be sent. This handler also handles
	'PUT', 'DELETE' and 'POST' (HTTP form upload) requests.

<H5>Overview</H5>
<TABLE BORDER=1>
<TH>Option
<TH>Default
<TH>Values
<TH>Short Description
<TH>Example(s)

<TR>
<TD>HeaderPattern
<TD>-
<TD>A Pi3Expression
<TD>Output first
<TD>HeaderPattern="<HTML><TITLE>The Title</TITLE>"

<TR>
<TD>FooterPattern
<TD>-
<TD>A Pi3Expression
<TD>Output last
<TD>FooterPattern="</HTML>"

<TR>
<TD>Methods
<TD>GET | HEAD
<TD>GET | HEAD | PUT | DELETE | POST | OPTIONS | TRACE
<TD>Supported HTTP methods
<TD>Methods GET | HEAD | PUT | DELETE

<TR>
<TD>FileKey
<TD>-
<TD>A string
<TD>Key for the filename part of multipart HTTP message
<TD>FileKey "upfile"

<TR>
<TD>NoteKey
<TD>-
<TD>A string
<TD>Key for the annotation part of a multipart HTTP message
<TD>NoteKey "notes"

<TR>
<TD>UploadPath
<TD>-
<TD>A valid path in URL format.
<TD>Override the default path of a form upload
<TD>UploadPath "/uploads/"

<TR>
<TD>Limit
<TD>-
<TD>A number, the default is 0.
<TD>Limitation of the upload filesize
<TD>Limit 2.097.152

<TR>
<TD>DescriptionFile
<TD>-
<TD>A filename without a path
<TD>File with annotations about the uploaded file
<TD>DescriptionFile ".desc"

<TR>
<TD>Annotation
<TD>-
<TD>A Pi3Expression
<TD>Expression is written to 'filename_ext.txt' during upload
<TD>Annotation "From:\t$A$MHost:\t$h$MTime:\t$t$MUrl:\t\"$r\"$M"

</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>

<H5>HeaderPattern</H5>
Specifies a pattern of data that will be pre-appended to the file.

<H5>FooterPattern</H5>
Specifies a pattern of data that will be appended to the file.

<H5>Methods</H5>
A chain of option fields describing the HTTP methods supported by this handler. 
Supported methods are GET | HEAD | PUT | DELETE | POST | OPTIONS.
The default is GET | HEAD if this option is omitted.

<P>The action caused by the HTTP methods are
<TABLE BORDER=1>
<TR>
<TH>Method
<TH>Action
<TR>
<TD>GET
<TD>To send the requested file to the browser.
<TR>
<TD>HEAD
<TD>To send only the HTTP header for the requested file to the browser.
<TR>
<TD>PUT
<TD>HTTP/1.1 request, to upload the requested file from a web browser to the server.
If the file already exists it must be a writeable and a regular file.
<TR>
<TD>DELETE
<TD>HTTP/1.1 request, to delete the requested file. File must exist and be a regular
file or directory. Deletion works not recursive in directories and directory must be
empty.
<TR>
<TD>POST
<TD>To upload a file using HTTP form upload with multipart message. An upload form
looks like
<P><XMP>
   <HTML>
   <BODY>
      <FORM method="POST" enctype="multipart/form-data" action="/errors/success.htm">
      <P>File  <INPUT type=file name=upfile>
      <P>Notes <INPUT type=text name=notes>
      <P>      <INPUT type=submit value=Upload>
      </FORM>
   </BODY>
   </HTML>
</XMP>
You can use PathInfo to redirect file to an upload directory action="/errors/success.htm".
The file in the action URL should always expire. Your browser will force this with a meta
tag in the HTML-Header
<P><XMP>
   <META http-equiv="expires" content="0">
</XMP>
<TR>
<TD>OPTIONS
<TD>HTTP/1.1 request, to retrieve the supported methods for the respective URL or
the server itself.
<TR>
<TD>TRACE
<TD>HTTP/1.1 request, which works like an echo handler. this could be used to test
the client
</TABLE>

<H5>FileKey</H5>
The key for the filename part during HTTP form upload of a multipart HTTP message.
This value is usually the same as the parameter 'name' of the filename input field
within the upload form.

<H5>NoteKey</H5>
The key for the annotation part during HTTP form upload of a multipart HTTP message.
This value is usually the same as the parameter 'name' of the annotations input field
within an upload form.

<H5>UploadPath</H5>
A valid path in URL format. This value overrides the default path of a form upload.
The default path is the PathInfo of the HTTP request URL.

<H5>Limit</H5>
This option is used to limitate the maximum number of bytes for a single file upload.

<H5>DescriptionFile</H5>
Allows to write annotations about uploaded file into a descriptive file. This file has
usually the same name as the file used by the HTML-directory index for descriptions.

<H5>Annotation</H5>
The parsed Pi3Expression is written to a file 'filename_ext.txt' during upload.
The annotations are listed in the directory index of the upload directory visible for
other users.

Phase:
	HANDLE

Returns:
	PIAPI_COMPLETED if the file was sent. INT_REDIRECT is returned in a POST
	request if the file was uploaded.
	If this handler was invoked for a phase other than HANDLE then PIAPI_ERROR
	is returned.

Note:

Example:
	<PRE>
	&lt;Object&gt;
		Name SendFile
		Class SendFileClass
	&lt;/Object&gt;

	&lt;Object&gt;
		...
		Handle SendFile
		...
	&lt;/Object&gt;

	</PRE>
/*___+++HTMLDOC_END+++___*/
#endif

int fnMultipartCb( int iKind, char *szName, char *szData, long lData, PIHTTP *pPIHTTP, void *pUser );

#define FLG_GET		0x00000002
#define FLG_POST	0x00000004
#define FLG_HEAD	0x00000008
#define FLG_PUT		0x00000010
#define FLG_DELETE	0x00000020
#define FLG_OPTIONS	0x00000040
#define FLG_TRACE	0x00000080
#define FLG_CONNECT	0x00000100


	/*___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ *
		
				Map these flags to names

	 *___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ */
struct {
	const char *pName;
	int iFlag;
} aMethodMap[] =
	{
	{ MD_NAME_GET,				FLG_GET },
	{ MD_NAME_POST,				FLG_POST },
	{ MD_NAME_HEAD,				FLG_HEAD },
	{ MD_NAME_PUT,				FLG_PUT },
	{ MD_NAME_DELETE,			FLG_DELETE },
	{ MD_NAME_OPTIONS,			FLG_OPTIONS },
	{ MD_NAME_TRACE,			FLG_TRACE },
	{ MD_NAME_CONNECT,			FLG_CONNECT },

	/* --- leave this last always --- */
	{ 0, 0 }
	};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class DescriptionMap
{
public:
	PIString sFileName;
	PIString sDescription;

	DescriptionMap( const char *pLine )
		{
		assert( pLine );
		int iLen = strlen( pLine );
		int i=0;
		for( i=0; i<iLen && pLine[i]!='|'; i++ );
		PIString sTmp( pLine, i );
		sFileName = sTmp;
		if ( i<iLen ) { i++; };
		pLine= &( pLine[i] );
		sDescription = pLine;
		};
	DescriptionMap( PIString sFile, char *szBuf, int lBuf )
		{
			sFileName = sFile;
			PIString sTmp( szBuf, lBuf );
			sDescription = sTmp;
		};

};


/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SendFile : public HandlerBaseHTTP
{
friend int fnMultipartCb( int iKind, char *szName, char *szData, long lData, PIHTTP *pPIHTTP, void *pUser );

private:
	/* ---
	Configuration data
	--- */
	Pi3Expression *pHeaderPattern;	/* pattern for top */
	Pi3Expression *pFooterPattern;	/* pattern for footer */
	int iMethods;					/* flags for supported HTTP-Methods */
	PIString sFileKey;
	PIString sNoteKey;
	Pi3Expression *pAnnotation;
	PIString sDescFile;
	PIString sPathInfo;
	PIString sUploadPath;
	PIString sUploadFile;
	int iLimit;
	PIPLATFORM_FD tFD;
	const char *pFKMethod;

protected:
	/*___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ *

				Load an expression or condition

	 *___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ */
	int LoadExpression( const char *pName, Pi3Expression **ppExpression,
		FnPi3Write *pFunctions, const char *pValue, PIOStrStream &os )	
		{
		assert( ppExpression );
		Pi3Expression *pExpression = *ppExpression;
		if ( pExpression )
			{
			os << "'" << pName << "' may only be specified once." << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};
		Pi3String *pError = Pi3String_new( 0 );
		pExpression = Pi3Expression_new( pValue, pFunctions, pError );
		if ( !pExpression )
			{
			os << "Error parsing expression: " <<
				Pi3String_getPtr( pError ) << ends;
			CONFIG_ERR( Object(), os.str() );
			Pi3Expression_delete( pExpression );
			Pi3String_delete( pError );
			return 0;
			};
		Pi3String_delete( pError );
		*ppExpression = pExpression;
		return 1;
		};

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );
		PIOStrStream os;
		os << pWhere << "SendFile: ";
		if ( !PIUtil_stricmp( KEY_CONF_HEADERPATTERN, pVariable ))
			{
			if ( !LoadExpression( KEY_CONF_HEADERPATTERN, &pHeaderPattern,
				0, pValue, os ) )
				{ return 0; };
			assert( pHeaderPattern );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_FOOTERPATTERN, pVariable ))
			{
			if ( !LoadExpression( KEY_CONF_FOOTERPATTERN, &pFooterPattern, 0, pValue, os ))
				{ return 0; };
			assert( pFooterPattern );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_FILEKEY, pVariable ))
			{
			sFileKey = (const char *)DeQuote( pValue );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_NOTEKEY, pVariable ))
			{
			sNoteKey = (const char *)DeQuote( pValue );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_NOTE, pVariable ))
			{
			if ( !LoadExpression( KEY_CONF_NOTE, &pAnnotation, 0, pValue, os ))
				{ return 0; };
			assert( pAnnotation );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_DESCFILE, pVariable ))
			{
			sDescFile = (const char *)DeQuote( pValue );
			PIFInfo *pFInfo = HTTPCore_getCachedFile( sDescFile );
			if ( !pFInfo || strcspn( PIFInfo_getPath( pFInfo ), "/\\" ) < strlen( sDescFile ))
				{
				HTTPCore_releaseCachedFile( pFInfo );
				os << "Error during read parameter '" << pVariable << "' or \
the filename '" << sDescFile << "' contains a path." << ends;
				CONFIG_ERR( Object(), os.str() );
				return 0;
				};
			HTTPCore_releaseCachedFile( pFInfo );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_UPLOADPATH, pVariable ))
			{
			sPathInfo = (const char *)DeQuote( pValue );			
			}
		else if ( !PIUtil_stricmp( KEY_CONF_LIMIT, pVariable ))
			{
			iLimit = atol( (const char *)pValue );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_METHODS, pVariable ))
			{
			/* ---
			Split the options up by the bar ('|') character and then
			strip trailing and leading whitespace from option before
			attempting to match it.
			--- */
			StringTokenizer tTokens( pValue, "|" );
			iMethods = 0;
			for(int i = 0; i < tTokens.NumTokens(); i++)
				{
				const char *pToken = tTokens.GetToken( i );

				/* --- skip leading whitespace --- */
				for( ; *pToken && (isspace(*pToken)); pToken++ );

				/* ---
				include only to first whitespace trailing whitespace
				--- */
				int j=0;
				for( ; pToken[j] && !(isspace(pToken[j])); j++ );
				/* ---
				j now contains the length of the first word in pToken
				--- */

				/* ---
				cycle through the list of available flags
				comparing them with this one
				--- */
				int i=0;
				for( ; aMethodMap[i].pName; i++ )
					{
					if ( !PIUtil_strncmpi( aMethodMap[i].pName, pToken, j ) )
						{ break; };
					};
				if ( !aMethodMap[i].pName )
					{
					/* --- flag not found --- */
					PIString sTmp( pToken, j );
					os << "Unknown HTTP method flag '" << sTmp << "'." << ends;
					CONFIG_ERR( Object(), os.str() );
					return 0;
					};
				iMethods |= aMethodMap[i].iFlag;
				};	/* --- loop over tokens seperated by '|' --- */
			}
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};


	/* ---
	Create a translated path
	--- */
	int GetPathTranslated( PIHTTP &tPIHTTP, PIString &sPath )
		{
		/* --- make sub request context --- */
		PIHTTP *pChildHTTP = PIHTTP_newChild( &tPIHTTP );

		/* --- set path --- */
		PIDB_replace( pChildHTTP->pResponseDB, PIDBTYPE_STRING,
			KEY_INT_PATH, (void *)(const char *)sPath, 0 );

		/* --- dispatch the sub request across the mapping phase --- */
		int iRet = PIAPI_ERROR;
		iRet = HTTPCore_dispatch( pChildHTTP, PH_MAPPING, PH_MAPPING );
			
		/* --- copy the childs path to this path translated --- */
		if ( iRet == PIAPI_COMPLETED && (( pChildHTTP->iStatus == 0 )
			|| ( pChildHTTP->iStatus == ST_OK )))
			{
			sPath = (const char *)PIDB_lookup( pChildHTTP->pResponseDB, PIDBTYPE_STRING,
				KEY_INT_PATH, 0 );
			}
		else
			{
			PIHTTP_delete( pChildHTTP );
			return PIAPI_ERROR;
			};

		PIHTTP_delete( pChildHTTP );
		return PIAPI_COMPLETED;
		};


	/* ---
	Reset the mapped path before doing internal direct
	--- */
	int ResetPath( PIHTTP &tPIHTTP )
	{
		int iRet = PIAPI_COMPLETED;

		const char *pURI = (const char *)PIDB_lookup( tPIHTTP.pRequestDB, 
					PIDBTYPE_STRING, KEY_HTTP_URI, 0 );
		if ( pURI ) 
			{
			char *pTmp = new char[strlen(pURI)+1];
			strcpy( pTmp, pURI ); 

			/* ---
			Then contract it to avoid side effects of '.' and '..' in
			the URL
			--- */
			if ( HTTPUtil_contractPath( pTmp )>0 )
				{
				/* --- force a 403 (forbidden) error --- */
				tPIHTTP.iStatus = ST_FORBIDDEN;
				iRet = INT_REDIRECT;
				}
			else
				{
				PIDB_replace( tPIHTTP.pResponseDB, PIDBTYPE_STRING,
					KEY_INT_PATH, pTmp, 0 );
				};

			delete [] pTmp;
			}
		else
			{
			HTTPCore_logError( &tPIHTTP, "Could not set request path\n" );
			iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		return iRet;
	}

	/* ---
	Load a description file, if one exists and contains descriptions return 1
	else 0
	--- */
	int LoadDescriptionFile( PIDB *pDescriptionDB, PIString sPath ) const
		{
		assert( pDescriptionDB );
		assert( sPath );

		/* --- get the file --- */
		PIFInfo *pFInfo = HTTPCore_getCachedFile( sPath );
		if ( !pFInfo ) return 0;

		if ( PIFInfo_exists( pFInfo ) && PIFInfo_isRegular( pFInfo ))
			{	
			enum { BUF_SIZE=1023 };
			char szBuf[BUF_SIZE+1];
			*szBuf = '\0';
			ifstream ifs( PIFInfo_getPath( pFInfo ) );
			while( ifs && !ifs.eof() ) 	
				{
				ifs.getline( szBuf, BUF_SIZE );
				if ( strlen( szBuf ))
					{
					DescriptionMap *pDesc = PI_NEW( DescriptionMap( szBuf ));
					PIDB_add( pDescriptionDB, PIDBTYPE_OPAQUE, pDesc->sFileName,
						(void *)pDesc, 0 );
					};
				};
			};
		HTTPCore_releaseCachedFile( pFInfo );
		return 1;
		};


	/* ---
	Save a description file, success return 1 else 0
	--- */
	int SaveDescriptionFile( PIDB *pDescriptionDB, PIString sPath ) const
		{
		assert( pDescriptionDB );
		assert( sPath );

		/* --- get the file --- */
		PIFInfo *pFInfo = HTTPCore_getCachedFile( sPath );
		if ( !pFInfo ) return 0;
		if ( PIFInfo_exists( pFInfo ) && !PIFInfo_isRegular( pFInfo )) return 0;
	
		ofstream ofs( PIFInfo_getPath( pFInfo ) );

		PIDBIterator *pIter = PIDB_getIterator( pDescriptionDB,	PIDBTYPE_OPAQUE, 0, 0 );
		if ( pIter )
			{
			for( ; pIter && PIDBIterator_atValidElement( pIter );
				PIDBIterator_next( pIter ))
				{
				DescriptionMap *pDesc = (DescriptionMap *)PIDBIterator_current( pIter, 0 );
				assert( pDesc );
				ofs << pDesc->sFileName << "|" << pDesc->sDescription << "\n";

				PI_DELETE( pDesc );
				};
			};
		PIDBIterator_delete( pIter );
		HTTPCore_releaseCachedFile( pFInfo );
		return 1;
		};


	/* ---
	Implementation of HTTP methods GET and HEAD (send headers and file)
	--- */
	int Get_Head( PIHTTP &tPIHTTP, int iMethod )
		{
		PIDB *pR = tPIHTTP.pResponseDB;

		/* ---
		Get file object
		--- */
		const char *pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,
			KEY_INT_PATH, 0 );
		PIFInfo *pFInfo = HTTPCore_getCachedFile( pPath ); 
		if ( !pFInfo ) return PIAPI_ERROR;

		/* ---	
		This file must exist and be a regular file 
		--- */
		if ( !PIFInfo_exists( pFInfo ) ||
			!PIFInfo_isRegular( pFInfo ) )
			{
			HTTPCore_logError( &tPIHTTP, "SendFile: File with \
path '%s' does not exist or is not a regular file.", pPath );
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
		};

		/* --- 	
		Get the header
		--- */
		enum { PATTERN_BUF_SIZE=1024 };
		char szHeader[PATTERN_BUF_SIZE];
		char *pHeader = 0;
		int iHeaderLen = 0;
		if ( pHeaderPattern )
			{
			pHeader = ( iMethod==MD_GET ) ? szHeader : 0;
			iHeaderLen = Pi3Expression_write( pHeaderPattern, &tPIHTTP, 0,
					pHeader, PATTERN_BUF_SIZE );
			if ( iHeaderLen>PATTERN_BUF_SIZE && pHeader )
				{
				pHeader = PI_NEW( char[iHeaderLen] );
				Pi3Expression_write( pHeaderPattern, &tPIHTTP, 0, pHeader,
					iHeaderLen );
				};
			};

		/* --- 	
		Get the footer
		--- */
		char szFooter[PATTERN_BUF_SIZE];
		char *pFooter = 0;
		int iFooterLen = 0;
		if ( pFooterPattern )
			{
			pFooter = ( iMethod==MD_GET ) ? szFooter : 0;
			iFooterLen = Pi3Expression_write( pFooterPattern, &tPIHTTP, 0,
					pFooter, PATTERN_BUF_SIZE );
			if ( iFooterLen>PATTERN_BUF_SIZE && pFooter )
				{
				pFooter = PI_NEW( char[iFooterLen] );
				Pi3Expression_write( pFooterPattern, &tPIHTTP, 0, pFooter,
					iFooterLen );
				};
			};

		/* --- transfer-encoding --- */
		int iFlags = (!PIHTTP_isChild( &tPIHTTP) && (int)PIDB_lookup( tPIHTTP.pRequestDB,
			PIDBTYPE_OPAQUE, KEY_HTTP_PROTOCOL, 0 ) == 3 ) ? PIIOBUF_CHUNKED : PIIOBUF_NONE;
		if ( iFlags & PIIOBUF_CHUNKED )
			{
			PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_TRANSFERENCODING,
				(void *)KEY_HTTP_CHUNKED, 0 );
			}
		else
			{
			/* --- content-length --- */
			enum { BUF_SIZE=63 };
			char szBuf[BUF_SIZE+1];
			unsigned long ulLen =  PIFInfo_getSize( pFInfo );
			ulLen += iHeaderLen;
			ulLen += iFooterLen;
			sprintf( szBuf, "%lu", ulLen );
			PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, szBuf, 0 );	
			};

		/* --- last modified --- */
		Pi3String *pTmp = Pi3String_new( 0 );
		time_t tT = PIFInfo_getLastModified( pFInfo );
		PIPlatform_beforeUnsafeBlock();
		struct tm *pTms = gmtime( &tT );
		HTTPUtil_rFC822Time( pTms, pTmp );
		PIPlatform_afterUnsafeBlock();
		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_LASTMODIFIED,
			(void *)Pi3String_getPtr( pTmp ), 0 );
		Pi3String_delete( pTmp );

		/* --- If current status is 0 then change to 200 OK --- */
		if ( !tPIHTTP.iStatus ) tPIHTTP.iStatus = ST_OK;

		/* --- start response --- */
		if (	HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
				HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) )
			{
			if ( pHeader && pHeader!=szHeader ) { PI_DELETE( [] pHeader ); };
			if ( pFooter && pFooter!=szFooter ) { PI_DELETE( [] pFooter ); };

			HTTPCore_releaseCachedFile( pFInfo );	
			return PIAPI_ERROR;
			};

		int iRet = 1;
		if ( iFlags & PIIOBUF_CHUNKED ) PIIOBuffer_flush( tPIHTTP.pBuffer );
		if ( iMethod == MD_GET )
			{
			if ( pHeader && iHeaderLen )
				{
				iRet = ( PIIOBuffer_write( tPIHTTP.pBuffer, pHeader,
					iHeaderLen, iFlags)>0 );
				};

			if ( iRet )
				{
				iRet = !HTTPUtil_sendFile( tPIHTTP.pBuffer, pFInfo,
					iFlags, 0 );
				};

			if ( iRet && pFooter && iFooterLen )
				{
				iRet = ( PIIOBuffer_write( tPIHTTP.pBuffer, pFooter,
					iFooterLen, iFlags)>0 );
				};
			};

		if ( pHeader && pHeader!=szHeader ) { PI_DELETE( [] pHeader ); };
		if ( pFooter && pFooter!=szFooter ) { PI_DELETE( [] pFooter ); };

		HTTPCore_releaseCachedFile( pFInfo );	

		/* ---
		If iRet is zero, an error occurred sending the file.
		** NOTE: Maybe do something with it.
		--- */
		return iRet ? PIAPI_COMPLETED : PIAPI_ERROR;
		};

		
	/* ---
	Implementation of HTTP/1.1 method DELETE (unlink file)
	--- */
	int Delete( PIHTTP &tPIHTTP )
		{
		PIDB *pR = tPIHTTP.pResponseDB;

		/* ---
		Get file object
		--- */
		const char *pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,
			KEY_INT_PATH, 0 );
		PIFInfo *pFInfo = HTTPCore_getCachedFile( pPath ); 
		if ( !pFInfo ) return PIAPI_ERROR;

		/* ---	
		This file must exist and be a regular file 
		--- */
		if ( !PIFInfo_exists( pFInfo ) || !PIFInfo_isRegular( pFInfo ))
			{
			HTTPCore_logError( &tPIHTTP, "SendFile: File with \
path '%s' does not exist or is not a regular file.", pPath );
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
		};

		/* ---	
		This file must be a writeable file 
		--- */
		if ( !PIFInfo_isWritable( pFInfo ))
			{
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_FORBIDDEN );
			};

		if ( PIFile_unlink( PIFInfo_getPath( pFInfo )))
			{
			HTTPCore_logError( &tPIHTTP, "SendFile: Could not unlink file with path '%s'.", pPath );
			HTTPCore_releaseCachedFile( pFInfo );
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		/* --- If current status is 0 then change to 200 OK --- */
		if ( !tPIHTTP.iStatus )	tPIHTTP.iStatus = ST_OK;

		/* --- content-length --- */
		char szBuf[2];
		unsigned long ulLen =  0;
		sprintf( szBuf, "%lu", ulLen );
		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, szBuf, 0 );	

		/* --- start response --- */
		if (	HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
				HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) )
			{
			HTTPCore_releaseCachedFile( pFInfo );	
			return PIAPI_ERROR;
			};

		HTTPCore_releaseCachedFile( pFInfo );	

		/* ---
		If iRet is zero, an error occurred sending the file.
		** NOTE: Maybe do something with it.
		--- */
		return PIAPI_COMPLETED;
		};


	/* ---
	Implementation of HTTP/1.1 method PUT (File upload)
	--- */
	int Put( PIHTTP &tPIHTTP, PIIOBuffer &tB )
		{
		PIDB *pR = tPIHTTP.pResponseDB;

		/* ---
		Check Content-Length for Limit
		--- */
		char *szContentLength = (char *)PIDB_lookup( tPIHTTP.pRequestDB,
			PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, 0 );
		// No Content-Length or a limit is defined and Content-Length is over limit
		if ( !szContentLength ) return PIAPI_ERROR;
		if ( iLimit && ( iLimit < atol( szContentLength )))
			{
			int iRead = 0;
			int iBufRead = 0;

			/* --- loop until no more bytes left --- */
			do
			{
				/* --- no more data or client timeout ? --- */
				if ( PIIOBuffer_pollBeforeRead( &tB ) < 1 ) break;

				/* --- read the data --- */
				PIIOBuffer_read( &tB, &iBufRead );
				iRead += iBufRead;
			}
			while ( iRead < atol( szContentLength ));

			HTTPCore_logError( &tPIHTTP, "SendFile: File is too large for upload.");
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		/* ---
		Get file object
		--- */
		const char *pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,
			KEY_INT_PATH, 0 );
		PIFInfo *pFInfo = HTTPCore_getCachedFile( pPath ); 
		if ( !pFInfo ) return PIAPI_ERROR;

		/* ---	
		If this file exist it must be a regular file 
		--- */
		if ( PIFInfo_exists( pFInfo ) && !PIFInfo_isRegular( pFInfo ))
			{
			HTTPCore_logError( &tPIHTTP, "SendFile: File with \
path '%s' is not a regular file.", pPath );
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
		};

		/* ---	
		If this file exist it must be a writeable file 
		--- */
		if ( PIFInfo_exists( pFInfo ) && !PIFInfo_isWritable( pFInfo ))
			{
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_FORBIDDEN );
			};

		if ( HTTPUtil_recvFile( &tPIHTTP, &tB, pFInfo ))
			{
			HTTPCore_logError( &tPIHTTP, "SendFile: Could not write file with path '%s'. \
The error code was %d",	PIFInfo_getPath( pFInfo ), PIPlatform_getLastError());
			HTTPCore_releaseCachedFile( pFInfo );
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		/* --- Change status to 201 CREATED or 204 NOCONTENT if already exists --- */
		tPIHTTP.iStatus = PIFInfo_exists( pFInfo ) ? ST_NOCONTENT : ST_CREATED;

		HTTPCore_releaseCachedFile( pFInfo );

		/* --- content-length --- */
		char szBuf[2];
		unsigned long ulLen =  0;
		sprintf( szBuf, "%lu", ulLen );
		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, szBuf, 0 );	

		/* Determine and send Location header */
		pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,	KEY_INT_PATHINFO, 0 );
		if (!pPath)
			{
			pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,
				KEY_INT_SCRIPTNAME, 0 );
			};

		if (pPath)
			{
			PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_LOCATION, (void *)pPath, 0 );
			};

		/* --- start response --- */
		if (	HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
				HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) )
			{
			return PIAPI_ERROR;
			};

		return PIAPI_COMPLETED;
		};


	/* ---
	Implementation of HTTP method POST (form upload as multipart message)
	--- */
	int Post( PIHTTP &tPIHTTP, PIIOBuffer &tB )
		{
		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pQ = tPIHTTP.pRequestDB;

		/* ---
		Check Content-Type for multipart MIME message
		--- */
		const char *szContentType = (const char *)PIDB_lookup( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE, 0 );
		if ( strcmp( szContentType, "multipart/form-data;" ) < 0 )
			{
			/* --- The message is not for this handler! --- */
			return PIAPI_CONTINUE;
			}

		/* ---
		Check Content-Length for Limit
		--- */
		const char *szContentLength = (const char *)PIDB_lookup( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, 0 );
		// No Content-Length or a limit is defined and Content-Length is over limit
		if ( !szContentLength ) return PIAPI_ERROR;
		if ( iLimit && ( iLimit < atol( szContentLength )))
			{
			int iRead = 0;
			int iBufRead = 0;

			/* --- loop until no more bytes left --- */
			do
				{
				/* --- no more data or client timeout ? --- */
				if ( PIIOBuffer_pollBeforeRead( &tB ) < 1 ) break;

				/* --- read the data --- */
				PIIOBuffer_read( &tB, &iBufRead );
				iRead += iBufRead;
				}
			while ( iRead < atol( szContentLength ));

			HTTPCore_logError( &tPIHTTP, "SendFile: File is too large for upload.");
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		/* ---
		(Re)set the upload path
		--- */ 
		if ( sPathInfo.Len() > 0 )
			{
			sUploadPath = sPathInfo;
			}
		else
			{
			sUploadPath = (const char *)PIDB_lookup( tPIHTTP.pResponseDB, PIDBTYPE_STRING,
				KEY_INT_PATHINFO, 0 );
			};
		if ( sUploadPath.Len() == 0 ) sUploadPath = "/";
		GetPathTranslated( tPIHTTP, sUploadPath );

		if ( HTTPUtil_recvMultipartMsg( &tPIHTTP, &tB, &fnMultipartCb, this ))
			{
			HTTPCore_logError( &tPIHTTP, "SendFile: Error during file upload. \
The system error code was %d.", PIPlatform_getLastError());
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		/* --- If current status is 0 then change to 200 OK --- */
		if ( !tPIHTTP.iStatus )	tPIHTTP.iStatus = ST_OK;

		/* --- content-length --- */
		char szBuf[2];
		unsigned long ulLen =  0;
		sprintf( szBuf, "%lu", ulLen );
		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, szBuf, 0 );	

		// response - internal redirect to the upload URL again
		int iRet = ResetPath( tPIHTTP );
		return (iRet != PIAPI_COMPLETED) ? iRet : HTTPUtil_doHTTPError(
			&tPIHTTP, ST_OK );
		};


	/* ---
	Implementation of HTTP/1.1 method OPTIONS
	--- */
	int Options( PIHTTP &tPIHTTP, PIIOBuffer &tB )
		{
		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pQ = tPIHTTP.pRequestDB;

		const char *pURI = (const char *)PIDB_lookup( pQ, PIDBTYPE_STRING,
			KEY_HTTP_URI, 0 );

		enum { BUF_SIZE=256 };
		char szBuf[BUF_SIZE+1];
		char *pPos = szBuf;

		if (!PIUtil_stricmp(pURI, "*")) {
			for (int i = MD_UNKNOWN+1; i < MD_LAST; i++) {
				if (iMethods & 1<<i) {
					if (pPos > szBuf) {
						strcpy(pPos, ", ");
						pPos += 2;
					};
					strcpy(pPos, aMethodMap[i-1].pName);
					pPos += strlen(aMethodMap[i-1].pName);
				};
			};
			PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_PUBLIC, szBuf, 0 );			
		} else {
			/* --- make sub request context --- */
			PIHTTP *pChildHTTP = PIHTTP_newChild( &tPIHTTP );

			pURI = (const char *)PIDB_lookup( pQ, PIDBTYPE_STRING,
				KEY_HTTP_URI, 0 );

			/* --- set path --- */
			PIDB_replace( pChildHTTP->pRequestDB, PIDBTYPE_STRING,
				KEY_HTTP_URI, (void *)pURI, 0 );

			for (int i = MD_UNKNOWN+1; i < MD_LAST; i++) {

				// We don't test method OPTIONS in sub request ...
				if ( MD_OPTIONS != i )
					{
					pChildHTTP->iStatus = 0;

					/* --- set method --- */
					PIDB_replace( pChildHTTP->pRequestDB, PIDBTYPE_STRING,
						KEY_HTTP_METHOD, (void *)aMethodMap[i-1].pName, 0 );

					pURI = (const char *)PIDB_lookup( pQ, PIDBTYPE_STRING,
						KEY_HTTP_URI, 0 );

					PIDB_replace( pChildHTTP->pResponseDB, PIDBTYPE_STRING,
						KEY_INT_PATH, (void *)pURI, 0 );

					/* --- dispatch the sub request across the mapping phase --- */
					int iRet = HTTPCore_dispatch( pChildHTTP, PH_MAPPING, PH_CHECKTYPE );
			
					/* --- copy the method to the response if child status OK --- */
					if ( (iRet == PIAPI_COMPLETED) && ( pChildHTTP->iStatus != ST_NOTIMPLEMENTED )
						&& ( pChildHTTP->iStatus != ST_METHODNOTALLOWED ))
						{
							if (pPos > szBuf) {
								strcpy(pPos, ", ");
								pPos += 2;
							};
							strcpy(pPos, aMethodMap[i-1].pName);
							pPos += strlen(aMethodMap[i-1].pName);
						};
					}
				else
				// ... 'cause we know, OPTIONS is allowed for the main request
				{
					if (pPos > szBuf) {
						strcpy(pPos, ", ");
						pPos += 2;
					};
					strcpy(pPos, MD_NAME_OPTIONS);
					pPos += strlen(MD_NAME_OPTIONS);
				};

			};
			PIHTTP_delete( pChildHTTP );

			PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_ALLOW, szBuf, 0 );	
		};

		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, (void *)"0", 0 );
		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE, 0, 0 );

		/* --- start response --- */
		if (	HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
				HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) )
			{
			return PIAPI_ERROR;
			};

		return PIAPI_COMPLETED;
		};


	/* ---
	Implementation of HTTP/1.1 method TRACE
	--- */
	int Trace( PIHTTP &tPIHTTP, PIIOBuffer &tB )
		{
		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pQ = tPIHTTP.pRequestDB;

		const char *pReq = (const char *)PIDB_lookup( pQ, PIDBTYPE_STRING,
			KEY_HTTP_CLF, 0 );
		unsigned long ulLen = strlen(pReq) + 2;

		int i = 0;																
		/*
		** loop over all RFC822 headers in request block to calculate Content Length
		*/				
		PIDBIterator *pIter = PIDB_getIterator( pQ, PIDBTYPE_RFC822, 0, 0 );	
		if ( !pIter )															
			{
				return PIAPI_ERROR;
			};	
	
		for(; PIDBIterator_atValidElement( pIter ); PIDBIterator_next( pIter ) )
			{																
			const char *pKey;													
			const char *pValue = (const char *)PIDBIterator_current( pIter,	&pKey );
			assert( pKey && pValue );
			ulLen += strlen(pKey) + strlen(pValue) + 4;
			};
		PIDBIterator_delete( pIter );

		/* --- content-length --- */
		char szBuf[2];
		sprintf( szBuf, "%lu", ulLen );
		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, szBuf, 0 );
		PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE, (void *)TYPE_MSG_HTTP, 0 );

		/* --- start response --- */
		if (	HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
				HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) )
			{
			return PIAPI_ERROR;
			};

		if ( PIIOBuffer_write( tPIHTTP.pBuffer, pReq, strlen(pReq), PIIOBUF_NONE) == 0 )
			{
			return PIAPI_ERROR;
			};
		if ( PIIOBuffer_write( tPIHTTP.pBuffer, "\r\n", 2, PIIOBUF_NONE) == 0 )
			{
			return PIAPI_ERROR;
			};

		pIter = PIDB_getIterator( pQ, PIDBTYPE_RFC822, 0, 0 );
		for(; PIDBIterator_atValidElement( pIter ); PIDBIterator_next( pIter ) )
			{																
			const char *pKey;													
			const char *pValue = (const char *)PIDBIterator_current( pIter,	&pKey );
			assert( pKey && pValue );
			if ( PIIOBuffer_write( tPIHTTP.pBuffer, pKey, strlen(pKey), PIIOBUF_NONE) == 0 )
				{
				PIDBIterator_delete( pIter );
				return PIAPI_ERROR;
				};
			if ( PIIOBuffer_write( tPIHTTP.pBuffer, ": ", 2, PIIOBUF_NONE) == 0 )
				{
				PIDBIterator_delete( pIter );
				return PIAPI_ERROR;
				};
			if ( PIIOBuffer_write( tPIHTTP.pBuffer, pValue, strlen(pValue), PIIOBUF_NONE) == 0 )
				{
				PIDBIterator_delete( pIter );
				return PIAPI_ERROR;
				};
			if ( PIIOBuffer_write( tPIHTTP.pBuffer, "\r\n", 2, PIIOBUF_NONE) == 0 )
				{
				PIDBIterator_delete( pIter );
				return PIAPI_ERROR;
				};
			};
		PIDBIterator_delete( pIter );

		return PIAPI_COMPLETED;
		};

public:
	SendFile( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	HandlerBaseHTTP( pObject ),
		pHeaderPattern( 0 ),
		pFooterPattern( 0 ),
		iMethods( FLG_GET | FLG_HEAD ),
		sFileKey( "upfile" ),
		sNoteKey( "notes" ),
		pAnnotation( 0 ),
		sDescFile( 0 ),
		sPathInfo( 0 ),
		sUploadPath( 0 ),
		sUploadFile( 0 ),
		iLimit( 0 ),
		tFD( PIPLATFORM_FD_INVALID ),
		pFKMethod( PIDB_getFastKey( KEY_HTTP_METHOD, PIDBTYPE_STRING ))
		{
		ReadParameters( iArgc, ppArgv );
		};

	~SendFile()
		{
		Pi3Expression_delete( pHeaderPattern );
		Pi3Expression_delete( pFooterPattern );
		};

	int Handle( int iPhase, PIHTTP &tPIHTTP, PIIOBuffer &tB )
		{
		if ( iPhase!=PH_HANDLE ) { return PIAPI_ERROR; };

		/* ---
		Lookup method 
		--- */
		int iMethod = (int)PIDB_lookup( tPIHTTP.pRequestDB, PIDBTYPE_OPAQUE,
			pFKMethod, PIDBFLAG_FASTKEY );

		/* ---
		Check for supported methods
		--- */
		if ( iMethods )
			{
			int iMethFlag = 1;
			iMethFlag <<= iMethod;
			/* ---
			The method is not handled by this object
			--- */
			if ( !(iMethFlag & iMethods) )
				{
				/* ---
				The object has been created with default methods
				--- */
				if ( (iMethods==(FLG_GET | FLG_HEAD)) &&
					(!( iMethFlag==FLG_GET || iMethFlag==FLG_HEAD )))
					{ return PIAPI_CONTINUE; };

				/* ---
				The object has been created with method mask
				--- */
				return HTTPUtil_doHTTPError( &tPIHTTP, ST_NOTIMPLEMENTED );
				};
			}

		switch( iMethod )
			{
			case MD_GET:
				return Get_Head( tPIHTTP, iMethod );
		
			case MD_HEAD:
				return Get_Head( tPIHTTP, iMethod );

			case MD_PUT:
				return Put( tPIHTTP, tB );

			case MD_POST:
				return Post( tPIHTTP, tB );

			case MD_DELETE:
				return Delete( tPIHTTP );

			case MD_OPTIONS:
				return Options( tPIHTTP, tB );

			case MD_TRACE:
				return Trace( tPIHTTP, tB );

			default:
				assert( 0 );
				/* ---
				Something is real going wrong
				--- */
				return PIAPI_ERROR;
			};

		return PIAPI_ERROR;
		};
};

#define pfkUpfileHandle "UPFILE_HANDLE"

int fnMultipartCb( int iKind, char *szName, char *szData, long lData, PIHTTP *pPIHTTP, void *pUser )
{
	// The file part of the message
	if ( !PIUtil_stricmp( szName, ((SendFile *)pUser)->sFileKey ))
		{
		PIString sPath;

		if ( iKind & MP_PART_FIRST )
			{
			// Extract filename
			char *pFileName = strstr( szData, "filename=" );
			if ( !pFileName ) return PIAPI_ERROR;
			pFileName += strlen("filename=") + 1;
			char *pStrEnd = strchr( pFileName, '"' );
			if ( pStrEnd )
			    {
			    *pStrEnd = 0;
			    }
			else
			    {
			    return PIAPI_ERROR;
			    };
			// memset( strchr( pFileName, '"' ), 0, 1);

			PIFInfo *pFInfo = HTTPCore_getCachedFile( pFileName );
			if ( !pFInfo ) return PIAPI_ERROR;
			strcpy( pFileName, PIFInfo_getName( pFInfo ));
			((SendFile *)pUser)->sUploadFile = pFileName;
			HTTPCore_releaseCachedFile( pFInfo );

			/* ---
			Get file path 
			--- */
			sPath = ((SendFile *)pUser)->sUploadPath;
			sPath.Concatenate( ((SendFile *)pUser)->sUploadFile );
		
			/* ---
			Get the file object
			--- */
			pFInfo = HTTPCore_getCachedFile( sPath ); 
			if ( !pFInfo ) return PIAPI_ERROR;

			/* ---	
			If this file exist it must be a regular file 
			--- */
			if ( PIFInfo_exists( pFInfo ) && !PIFInfo_isRegular( pFInfo ))
				{
				HTTPCore_logError( pPIHTTP, "SendFile: File with \
path '%s' is not a regular file.", (const char *)sPath );
				HTTPCore_releaseCachedFile( pFInfo );	
				return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
			};

			/* ---	
			If this file exist it must be a writeable file 
			--- */
			if ( PIFInfo_exists( pFInfo ) && !PIFInfo_isWritable( pFInfo ))
				{
				HTTPCore_releaseCachedFile( pFInfo );	
				return HTTPUtil_doHTTPError( pPIHTTP, ST_FORBIDDEN );
				};

			HTTPCore_releaseCachedFile( pFInfo );

			// jump over Content-Disposition
			char *szStart = (char *)memchr( szData, 10, lData );
			if ( !szStart ) return PIAPI_ERROR;

			// jump over Content-Type
			szStart = (char *)memchr( szStart + 1, 10, lData );
			if ( !szStart ) return PIAPI_ERROR;

			// jump over CR/LF
			szStart = (char *)memchr( szStart + 1, 10, lData ) + 1;
			if ( !szStart ) return PIAPI_ERROR;

			// Open file from origin name
			((SendFile *)pUser)->tFD = PIFile_open( sPath, "w" );
			if ( ((SendFile *)pUser)->tFD == PIPLATFORM_FD_INVALID ) return PIAPI_ERROR;

			if ( PIFile_write( ((SendFile *)pUser)->tFD, lData - (szStart - szData), szStart ))
				{
				PIFile_close( ((SendFile *)pUser)->tFD );
				((SendFile *)pUser)->tFD = PIPLATFORM_FD_INVALID;
				return PIAPI_ERROR;
				};
			};

		if ( iKind & MP_PART_NEXT )
			{
			// write the data to the file
			if ( PIFile_write( ((SendFile *)pUser)->tFD, lData, szData ))
				{
				PIFile_close( ((SendFile *)pUser)->tFD );
				((SendFile *)pUser)->tFD = PIPLATFORM_FD_INVALID;
				return PIAPI_ERROR;
				};
			};

		if ( iKind & MP_PART_LAST )
			{
			if ( iKind == MP_PART_LAST )
				{
				// write the data to the file
				if ( PIFile_write( ((SendFile *)pUser)->tFD, lData, szData ))
					{
					PIFile_close( ((SendFile *)pUser)->tFD );
					((SendFile *)pUser)->tFD = PIPLATFORM_FD_INVALID;
					return PIAPI_ERROR;
					};
				};

			// close the file
			if ( PIFile_close( ((SendFile *)pUser)->tFD )) return PIAPI_ERROR;
			((SendFile *)pUser)->tFD = PIPLATFORM_FD_INVALID;

			/* ---
			Write the annotation
			--- */
			if ( !((SendFile *)pUser)->pAnnotation ) return PIAPI_COMPLETED;

			enum { BUF_SIZE=1024 };
			char szBuf[BUF_SIZE];
			strcpy( szBuf, ((SendFile *)pUser)->sUploadPath );
			strcat( szBuf, ((SendFile *)pUser)->sUploadFile );
			char *pStrEnd = strchr( szBuf, '.' );
			if ( pStrEnd )
			    {
			    *pStrEnd = '_';
			    }
			else
			    {
			    return PIAPI_ERROR;
			    };
			
			// if ( strchr( szBuf, '.' )) memset( strchr( szBuf, '.' ), '_', 1 );
			sPath = szBuf;
			sPath.Concatenate( NOTE_EXT );

			char *pBuf = szBuf;
			int iLen = Pi3Expression_write( ((SendFile *)pUser)->pAnnotation, pPIHTTP, 0, pBuf, BUF_SIZE );
			if ( iLen > BUF_SIZE )
				{
				pBuf = PI_NEW( char[iLen] );
				Pi3Expression_write( ((SendFile *)pUser)->pAnnotation, pPIHTTP, 0, pBuf, iLen );
				};
		
			// Open file from origin name
			((SendFile *)pUser)->tFD = PIFile_open( sPath, "w" );
			if ( ((SendFile *)pUser)->tFD == PIPLATFORM_FD_INVALID ) return PIAPI_ERROR;

			/* --- write the message --- */
			int iRet = PIFile_writeAtomic( ((SendFile *)pUser)->tFD, iLen, pBuf );
			if ( pBuf != szBuf )
				{
				PI_DELETE( [] pBuf );
				};
			if ( iRet )
				{
				/* --- an error occurred --- */
				HTTPCore_logError( pPIHTTP, "SendFile: \
PIFile_writeAtomic() failed for file '%s', error code '%d'.", (const char *)sPath,
				PIPlatform_getLastError() );
				PIFile_close( ((SendFile *)pUser)->tFD );
				((SendFile *)pUser)->tFD = PIPLATFORM_FD_INVALID;
				return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
				};

			// close the file
			if ( PIFile_close( ((SendFile *)pUser)->tFD )) return PIAPI_ERROR;
			((SendFile *)pUser)->tFD = PIPLATFORM_FD_INVALID;
			};

		return PIAPI_COMPLETED;
		};

	// The note part of the message
	if ( !PIUtil_stricmp( szName, ((SendFile *)pUser)->sNoteKey ))
		{
		if ( !((SendFile *)pUser)->sDescFile ) return PIAPI_COMPLETED;
		
		// The file name must exist!
		if ( !((SendFile *)pUser)->sUploadFile ) return PIAPI_ERROR;

		/* ---
		Get path information of upload file
		--- */
		PIFInfo *pFInfo = HTTPCore_getCachedFile( ((SendFile *)pUser)->sUploadPath );
		if ( !pFInfo ) return PIAPI_ERROR;

		PIString sPath = ((SendFile *)pUser)->sUploadPath;
		sPath.Concatenate( ((SendFile *)pUser)->sDescFile );
		HTTPCore_releaseCachedFile( pFInfo );

		// jump over Content-Disposition
		char *szStart = (char *)memchr( szData, 10, lData );
		if ( !szStart ) return PIAPI_ERROR;

		// jump over CR/LF
		szStart = (char *)memchr( szStart + 1, 10, lData ) + 1;
		if ( !szStart ) return PIAPI_ERROR;

		PIDB *pDescriptionDB = PIDB_new( 0, "File Descriptions" );
		if ( !((SendFile *)pUser)->LoadDescriptionFile( pDescriptionDB, sPath )) 
			{
			PIDB_delete( pDescriptionDB );
			return PIAPI_ERROR;
			};

		DescriptionMap *pDesc = (DescriptionMap *)PIDB_lookup( pDescriptionDB,
			PIDBTYPE_OPAQUE, ((SendFile *)pUser)->sUploadFile, 0 );

		if ( !pDesc )
			{
			pDesc = PI_NEW( DescriptionMap( ((SendFile *)pUser)->sUploadFile,
				szStart, lData - (szStart - szData)));
			PIDB_add( pDescriptionDB, PIDBTYPE_OPAQUE, pDesc->sFileName,
				(void *)pDesc, 0 );
			}
		else
			{
			PIString sTmp( szStart, lData - (szStart - szData));
			pDesc->sDescription = sTmp;
			};

		if ( !((SendFile *)pUser)->SaveDescriptionFile( pDescriptionDB, sPath )) 
			{
			PIDB_delete( pDescriptionDB );
			return PIAPI_ERROR;
			};

		PIDB_delete( pDescriptionDB );

		return PIAPI_COMPLETED;
		};

	return PIAPI_ERROR;
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int SendFile_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return HandlerBaseHTTP_constructor( pObj, PI_NEW( SendFile( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name SendFileClass
		Type LogicExtension
		Library HTTP
		OnClassLoad HandlerBaseHTTP_onClassLoad
		Constructor SendFile_constructor
		CopyConstructor HandlerBaseHTTP_copyConstructor
		Destructor HandlerBaseHTTP_destructor
		Execute HandlerBaseHTTP_execute
	</Class>

	<Object>
		Name SendFile
		Class SendFileClass
	</Object>

/*___+++CNF_END+++___*/
#endif

