/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/HTTP/WinCGI.cpp,v $
 * $Date: 2003/06/09 13:29:54 $
 *
 Description:
	WinCGI Handler

    Rewritten so this module handles windows WinCGI only. CGI is now in
    CGI.cpp

\*____________________________________________________________________________*/
//$SourceTop:$

#include <ctype.h>
#include <stdio.h>

#include "PIStrStr.h"
#include "HTTPCore.h"
#include "HTTPUtil.h"
#include "HTTPUtil.h"
#include "PIHTTP.h"
#include "Pi2API.h"
#include "StrToken.h"
#include "DeQuote.h"
#include "HandBase.h"
#include "Pi3Expr.h"

#if WIN32
#	include <windows.h>
#	include <io.h>
	typedef HANDLE FILEDESC;
	typedef HANDLE PROCDESC;
#	define INVALID_DESC	(INVALID_HANDLE_VALUE)
	typedef void * ENVIRONMENT;
	inline bool CloseFileDesc( FILEDESC &tFileDesc )
				{
#	if !defined(NDEBUG)
				int iRet = ::CloseHandle( tFileDesc )==0 ? false : true;
				tFileDesc = INVALID_DESC;
				return iRet;
#	else
				return ::CloseHandle( tFileDesc )==0 ? false : true;
#	endif
				};
#elif POSIX
#	include <unistd.h>
#	include <errno.h>
#	include <signal.h>
	typedef int PROCDESC;
	typedef int FILEDESC;
#	define INVALID_DESC	(-1)
	typedef void **ENVIRONMENT;
	inline bool CloseFileDesc( FILEDESC &tFileDesc )
				{
#	if !defined(NDEBUG)
				int iRet = ::close( tFileDesc )==-1 ? false : true;
				tFileDesc = INVALID_DESC;
				return iRet;
#	else
				return ::close( tFileDesc )==-1 ? false : true;
#	endif
				};
#	define _MAX_PATH	PATH_MAX
#else
#	error Unsupported operating system
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_FILEIOOBJECT					"FileIOObject"
#define KEY_CONF_DEFAULTCOMMANDLINE				"DefaultCommandLine"
#define KEY_CONF_COMMANDLINEBYEXT				"CommandLineByExt"
#define KEY_CONF_EXTRAHEADERS					"ExtraHeaders"
#define VALUE_NO								"No"
#define VALUE_YES								"Yes"
#define KEY_CONF_EXTRAHEADERSPREFIX				"ExtraHeadersPrefix"
#define KEY_CONF_EXTRAHEADERSIGNORE				"ExtraHeadersIgnore"
#define KEY_CONF_SENDCRLF						"SendCRLF"
#define KEY_CONF_EXTERNALPATH					"ExternalPath"
#define KEY_CONF_DATAFILE						"DataFile"
#define KEY_CONF_STDINFILE						"StdinFile"
#define KEY_CONF_STDOUTFILE						"StdoutFile"
#define KEY_CONF_DATABLOCK						"DataBlock"
#define KEY_CONF_FLAG							"Flag"
#define KEY_CONF_KILLAFTER						"KillAfter"
#define VALUE_16BIT								"16-Bit"

/* --- really verbose development-time debugging --- */
#define VERBOSE_DEBUG	0

#if VERBOSE_DEBUG
#	define D { cerr << __FILE__ << ", " << dec << __LINE__ << ": " << endl; }
#else
#	define D
#endif

/*
** Flags 
*/
#define FLG_16BIT								0x0001

/*____________________________________________________________________________*\
 *
 Description:
	Documentation.
\*____________________________________________________________________________*/
#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name WinCGIClass
		Type LogicExtension
		Library HTTP
		OnClassLoad HandlerBaseHTTP_onClassLoad
		Constructor WinCGI_constructor
		CopyConstructor HandlerBaseHTTP_copyConstructor
		Destructor HandlerBaseHTTP_destructor
		Execute HandlerBaseHTTP_execute
	</Class>

	<Object>
		Name WinCGI
		Class WinCGIClass
	</Object>

/*___+++CNF_END+++___*/
#endif
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	WinCGI

Description:
	This HTTP Handler handles WinCGI requests.

Options:
<H5>Overview</H5>
<TABLE BORDER=1>
<TH>Option
<TH>Default
<TH>Values
<TH>Short Description
<TH>Example(s)

<TR>
<TD>FileIOObject
<TD>+
<TD>&lt;objectname&gt;
<TD>A Pi3 IO object
<TD>FileIOObject "CGIFileIO"

<TR>
<TD>CommandLineByExt
<TD>-
<TD>&lt;key&gt;=&quot;&lt;Pi3Expression&gt;&quot;
<TD>A key-value pair with an extension and a Pi3Expression
<TD>CommandLineByExt .cgi=&quot;perl %p%q&quot;

<TR>
<TD>DefaultCommandLine
<TD>-
<TD>&lt;Pi3Expression&gt;
<TD>A Pi3Expression, which expands to a command line
<TD>DefaultCommandLine &quot;%p %d&quot;

<TR>
<TD>ExtraHeaders
<TD>Yes
<TD>Yes|No
<TD>Indicates if extra headers are considered
<TD>ExtraHeaders Yes

<TR>
<TD>ExtraHeadersPrefix
<TD>-
<TD>&lt;A string&gt;
<TD>Used as prefix of each extra header
<TD>ExtraHeadersPrefix &quot;HTTP_&quot;

<TR>
<TD>ExtraHeadersIgnore
<TD>-
<TD>&lt;Space delimited strings&gt;
<TD>List of unconsidered extra headers
<TD>ExtraHeadersIgnore &quot;Content-Type Content-Length&quot;

<TR>
<TD>ExternalPath
<TD>-
<TD>&lt;Pi3Expression&gt;
<TD>Path to external CGI form data files
<TD>ExternalPath &quot;./CGITemp/&quot;

<TR>
<TD>DataFile
<TD>-
<TD>&lt;Pi3Expression&gt;
<TD>Path to CGI data file
<TD>DataFile &quot;./CGITemp/%u.ini&quot;

<TR>
<TD>StdinFile
<TD>-
<TD>&lt;Pi3Expression&gt;
<TD>Path to CGI input file
<TD>StdinFile &quot;./CGITemp/%u.in&quot;

<TR>
<TD>StdoutFile
<TD>-
<TD>&lt;Pi3Expression&gt;
<TD>Path to CGI output file
<TD>StdoutFile &quot;./CGITemp/%u.out&quot;

<TR>
<TD>DataBlock
<TD>-
<TD>&lt;Pi3Expression&gt;
<TD>Block of text written to datafile
<TD>DataBlock &quot;[CGI]\nRequest Protocol...&quot;

<TR>
<TD>KillAfter
<TD>-1
<TD>&lt;A number&gt;
<TD>Time to wait before killing the process.
<TD>KillAfter &quot;-1&quot;

<TR>
<TD>SendCRLF
<TD>No
<TD>Yes|No
<TD>Adds additional "CRLF" to the CGI input.
<TD>SendCRLF Yes

<TR>
<TD>Flag
<TD>-
<TD>"16-Bit", etc.
<TD>A flag to effect CGI
<TD>Flag &quot;16-Bit&quot;

</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>
<H5>FileIOObject</H5>
Specifies the IO object to be used to communicate with CGI programs.
A prototype object is created on initialization and a copy is generated
using PIObject_copy() for each CGI program.

<H5>CommandLineByExt</H5>
This configuration key could be given multiple times to declare command
line mappings for file extensions of CGI programs. The value is treated
as Pi3Expression.

<H5>DefaultCommandLine</H5>
This configuration key is used to declare the default command line,
which is used, if no mapping matches. The value is treated as
Pi3Expression.

<H5>ExtraHeaders</H5>
This configuration key defines, if extra headers are considered by
the processing of this handler.

<H5>ExtraHeadersPrefix</H5>
This configuration value will be the trailing string of each extra
request header.

<H5>ExtraHeadersIgnore</H5>
The headers in this list are not treated as extra headers.

<H5>ExternalPath</H5>
A valid path name, which is used to form the full path to temporary
files used to pass external form variables to the CGI program.

<H5>DataFile</H5>
A Pi3Expression which is evaluated to form the full path to a file used to
pass variable-value arguments to the CGI program.
<P>
This mechanism is typically used to support CGI variants other than
standard CGI, namely WinCGI and DOSCGI.
<P>
Specification of 'DataFile', 'StdinFile' or 'StdoutFile' cause the
standard CGI streams environment to be disabled.

<H5>StdinFile</H5>
A Pi3Expression which is evaluated to form the full path to a file used to
pass input from the browser to the CGI program.
<P>
This mechanism is typically used to support CGI variants other than
standard CGI, namely WinCGI and DOSCGI.
<P>
Specification of 'DataFile', 'StdinFile' or 'StdoutFile' cause the
standard CGI streams environment to be disabled.

<H5>StdoutFile</H5>
A Pi3Expression which is evaluated to form the full path to a file used to
pass the standard output from the CGI program. Some forms of CGI, notably
WinCGI use files to pass information between the CGI program and HTTP server.
The parameter %u used in the expression will evaluate to a sequence of numbers
and letters unique to this HTTP process and request.
<P>
This mechanism is typically used to support CGI variants other than
standard CGI, namely WinCGI and DOSCGI.
<P>
Specification of 'DataFile', 'StdinFile' or 'StdoutFile' cause the
standard CGI streams environment to be disabled.

<H5>DataBlock</H5>
A Pi3Expression which is evaluated to form the contents of the data file
(.INI file) passed to the CGI program.
<P>
This mechanism is typically used to support CGI variants other than
standard CGI, namely WinCGI and DOSCGI.

<H5>KillAfter</H5>
Specifies the amount of time the server will wait for a CGI response
before forceably killing the process. -1 specifies an infinite timeout.
This value is measured in seconds.

<H5>SendCRLF</H5>
If this option is set to &quot;Yes&quot;, an additional linefeed (CRLF),
which is not contained in the Content-Length, is added to the CGI input
by the handler. This is required by some CGI programs.

<H5>Flag</H5>
Specifies a flag which changes CGI behaviour. This directive can be
repeated multiple times to add different flags.<P>
Flags are:-<BR>
<I>16-Bit</I> <B>Windows NT only</B>, expect CGI programs to be 16-Bit
Windows or MS-DOS images, change semantics appropriately.

Parameters:
The Pi3Expressions in the configuration variables DataFile, StdinFile,
StdoutFile and DataBlock may contain besides the standard shortcuts
used in Pi3Expressions the following, context specific parameters.

<CENTER>
<TABLE BORDER=1>
<TH>Parameter<TH>Evaluates to<TR>
<TD>%d<TD>Full pathname to the data file<TR>
<TD>%i<TD>Full pathname to the content file<TR>
<TD>%o<TD>Full pathname of the output file<TR>
<TD>%p<TD>Full pathname to WinCGI program<TR>
<TD>%u<TD>Unique ID in the context of the current process (PID)<TR>
<TD>%q<TD>Value of an &quot;isIndex&quot; query<TR>
<TD>%h<TD>List of extra request headers in Windows ini-format<TR>
<TD>%a<TD>List of accepts headers in Windows ini-format<TR>
<TD>%l<TD>List of decoded form literals in Windows ini-format<TR>
<TD>%e<TD>List of decoded external form literals in Windows ini-format<TR>
<TD>%g<TD>List of encoded huge form data in Windows ini-format<TR>
<TD>%f<TD>List of uploaded multipart/form-data files in Windows ini-format<TR>
</TABLE>
</CENTER>

Example:
	<PRE>

	&lt;Object&gt;
		Name MyWinCGI
		Class WinCGIClass
		FileIOObject "CGIFileIO"
		CommandLineByExt .cgi="perl %p %d"
		DefaultCommandLine "%p %d"
		ExtraHeaders Yes
		ExtraHeadersPrefix "HTTP_"
		ExtraHeadersIgnore "Content-Type Content-Length"
		ExternalPath "cgi-temp/"
		DataFile "cgi-temp/$P_%u.dat"
		StdinFile "cgi-temp/$P_%u.in"
		StdoutFile "cgi-temp/$P_%u.out"
		DataBlock "\[CGI]$M\
	Request Protocol=$H$M\
	Request Method=$m$M\
	$M\
	[Accept]$M\
	%a$M\
	[System]$M\
	Output File=%o$M\
	Content File=%i$M\
	$M\
	[Extra Headers]$M\
	%h$M\
	[Form Literal]$M\
	%l$M\
	[Form External]$M\
	%e$M\
	[Form File]$M\
	%f$M\
	[Form Huge]$M\
	%g$M"
	&lt;/Object&gt;
	</PRE>
/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
inline void CloseIfValidDesc( FILEDESC &tFD )
{
	if (tFD!=INVALID_DESC)
		{ ::CloseFileDesc( tFD ); };
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
inline void CloseProcessDesc( PROCDESC &tProcess )
{
#if WIN32
	CloseIfValidDesc( tProcess );
#else
	(void)tProcess;
#endif
}

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class WinCGI : HandlerBaseHTTP
{
private:
	/* --- forbid copy constructor --- */
	WinCGI( const WinCGI &t )
	: HandlerBaseHTTP( t )
		{ assert( 0 ); };

	/* ---
	Configuration data
	--- */
	PIObject *pPipeIO;			/* the pipe IO object */
	Pi3Expression *pDefaultCommandLine;
								/* default command line */
	DblList	lCommandLineByExt;	/* mappings of file extension to command line */
	int iExtraHeaders;			/* whether or not to send extra headers */
	int iSendCRLF;				/* send CRLF to CGI program after data */
	enum { DEFAULT_ENVIRONMENT_SIZE=4094 };
	PIString sExtraHeadersPrefix;
								/* prefix for extra header information */
	DblList lIgnoreHeaders;		/* extra headers to ignore */
	FnPi3Write aFunctions[256];	/* callback functions */
	PIString sExternalPath;     /* path for external form files */
	Pi3Expression *pDataFile;	/* datafile */
	Pi3Expression *pStdinFile;	/* file that CGI program reads input from */
	Pi3Expression *pStdoutFile;	/* file that CGI program writes output to */
	Pi3Expression *pDataBlock;	/* block of text written to datafile */
	int iFlags;					/* flags */
	int iKillAfter;				/* kill the CGI process after this time */

	/* ---
	FastKeys
	--- */
	const char *pFKContentType;			/* RFC822: Content-Type */
	const char *pFKFileInfo;			/* OPAQUE: Pointer to fileinfo object */
	const char *pFKPath;				/* STRING: Translated path */
	const char *pFKURI;					/* RFC822: URI from request header */

	/* ---
	Class with context data for expressions
	--- */
	class CGIContextData
	{
	private:
		static int iUniqueIndex;

	public:
		PIString sUniqueIndex;
		PIString sDataFile;
		PIString sInputFile;
		PIString sOutputFile;
		PIString sExePath;
		PIString sIsIndexValue;
		PIString sExtraHeaders;
		PIString sAcceptHeader;
		PIString sFormLiteral;
		PIString sFormExternal;
		PIString sFormHuge;
		PIString sFormFile;

	public:
		CGIContextData()
			{
			enum { BUF_SIZE=63 };
			char szBuf[BUF_SIZE+1];
			PIPlatform_beforeUnsafeBlock();
			int iIndex = iUniqueIndex++;
			PIPlatform_afterUnsafeBlock();
			sprintf( szBuf, "%lX", (long int)iIndex ); 
			sUniqueIndex = szBuf;
			};
	};

	/* ---
	Class for fileextension to commandline
	--- */
	class FileExtToCommandLineMap
		{
		public:
			PIString sFileExtension;
			Pi3Expression *pExpr;
			int iOK;

			FileExtToCommandLineMap( const char *pPattern,
				FnPi3Write *paFunctions, PIString &sError )
				:	pExpr( 0 ),
					iOK( 0 )
				{
				StringTokenizer tTokens( pPattern, "=", 0 );
				if ( tTokens.NumTokens()!=2 )
					{ return; };

				DeQuote tTmp( tTokens.GetToken( 0 ) );
				const char *pExt = tTmp;
				sFileExtension = (*pExt=='.') ? &(pExt[1]) : pExt;

				DeQuote tTmp2( tTokens.GetToken( 1 ) );
				const PIString &sTmp = (const PIString &)tTmp2;

				Pi3String *pError = Pi3String_new( 0 );
				pExpr = Pi3Expression_new( sTmp, paFunctions, pError );
				if ( !pExpr )
					{
					sError = Pi3String_getPtr( pError );
					Pi3String_delete( pError );
					return;
					};

				Pi3String_delete( pError );

				iOK = 1;
				};

			~FileExtToCommandLineMap()
				{
				Pi3Expression_delete( pExpr );
				};

			inline int IsOK()	{ return iOK; };
		};

private:
	int LoadExpression( const char *pName, Pi3Expression **ppExpression,
		const char *pValue, PIOStrStream &os )
		{
		assert( pName );
		assert( pValue );
		assert( ppExpression );
		Pi3Expression *pExpr = *ppExpression;
		if ( pExpr )
			{
			os << "'" << pName << "' may only be specified once." << ends;
			CONFIG_ERR( Object(), os.str() ); 
			return 0;
			};

		Pi3String *pError = Pi3String_new( 0 );
		pExpr = Pi3Expression_new( pValue, aFunctions, pError );
		if ( !pExpr )
			{
			os << Pi3String_getPtr( pError ) << ends;
			Pi3String_delete( pError );
			CONFIG_ERR( Object(), os.str() ); 
			return 0;
			};
		Pi3String_delete( pError );
		*ppExpression = pExpr;
		return 1;
		};

protected:
	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );
		if ( !pVariable || !pValue )
			{ return 0; };
		PIOStrStream os;			/* in case of error */
		os << pWhere << "WinCGI: ";
		if ( !PIUtil_stricmp( KEY_CONF_DEFAULTCOMMANDLINE, pVariable ) )
			{
			if ( pDefaultCommandLine )
				{
				os << "'DefaultCommandLine' may only be specified once."
					<< ends;
				CONFIG_ERR( Object(), os.str() ); 
				return 0;
				};
			Pi3String *pError = Pi3String_new( 0 );
			Pi3Expression *pExpr = Pi3Expression_new( pValue, aFunctions,
				pError );
			if ( !pExpr )
				{
				os << Pi3String_getPtr( pError ) << ends;
				Pi3String_delete( pError );
				CONFIG_ERR( Object(), os.str() ); 
				return 0;
				};
			Pi3String_delete( pError );
			pDefaultCommandLine = pExpr;
			}
		else if ( !PIUtil_stricmp( KEY_CONF_COMMANDLINEBYEXT, pVariable ) )
			{
			PIString sError;
			FileExtToCommandLineMap *pMap =
				PI_NEW( FileExtToCommandLineMap( pValue, aFunctions, sError ) );
			if ( !pMap || !pMap->IsOK() )
				{
				PI_DELETE( pMap );
				CONFIG_ERR( Object(), os.str() ); 
				return 0;
				};
			lCommandLineByExt.Append( (DblList::type)pMap );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_EXTERNALPATH, pVariable ) )
			{
			sExternalPath=(const char *)DeQuote( pValue );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_DATAFILE, pVariable ) )
			{
			if ( !LoadExpression( KEY_CONF_DATAFILE, &pDataFile,
				pValue, os ) )
				{ return 0; };
			}
		else if ( !PIUtil_stricmp( KEY_CONF_STDINFILE, pVariable ) )
			{
			if ( !LoadExpression( KEY_CONF_STDINFILE, &pStdinFile,
				pValue, os ) )
				{ return 0; };
			}
		else if ( !PIUtil_stricmp( KEY_CONF_STDOUTFILE, pVariable ) )
			{
			if ( !LoadExpression( KEY_CONF_STDOUTFILE, &pStdoutFile,
				pValue, os ) )
				{ return 0; };
			}
		else if ( !PIUtil_stricmp( KEY_CONF_DATABLOCK, pVariable ) )
			{
			if ( !LoadExpression( KEY_CONF_DATABLOCK, &pDataBlock,
				pValue, os ) )
				{ return 0; };
			}
		else if ( !PIUtil_stricmp( KEY_CONF_EXTRAHEADERS, pVariable ) )
			{
			if ( !PIUtil_stricmp( VALUE_NO, pValue ) )
				{ iExtraHeaders = 0; };
			}
		else if ( !PIUtil_stricmp( KEY_CONF_SENDCRLF, pVariable ) )
			{
			if ( !PIUtil_stricmp( VALUE_YES, pValue ) )
				{ iSendCRLF = 1; };
			}
		else if ( !PIUtil_stricmp( KEY_CONF_EXTRAHEADERSPREFIX, pVariable ) )
			{ sExtraHeadersPrefix = (const char *)DeQuote( pValue ); }
		else if ( !PIUtil_stricmp( KEY_CONF_FLAG, pVariable ) )
			{
			if ( !PIUtil_stricmp( pValue, VALUE_16BIT ) )
				{ iFlags |= FLG_16BIT; }
			else
				{
				os << "Unknown flag '" << pValue << "'" << ends;
				CONFIG_ERR( Object(), os.str() ); 
				return 0;
				};
			}
		else if ( !PIUtil_stricmp( KEY_CONF_EXTRAHEADERSIGNORE, pVariable ) )
			{
			StringTokenizer tTokens( (const char *)DeQuote(pValue), " " );
			for( int i=0; i<tTokens.NumTokens(); i++)
				{
				lIgnoreHeaders.Append( (DblList::type)
					PI_NEW( PIString( tTokens.GetToken( i ) ) ) );
				};
			}
		else if ( !PIUtil_stricmp( KEY_CONF_FILEIOOBJECT, pVariable ) )
			{ /* --- read from CGI constructor --- */ }
		else if ( !PIUtil_stricmp( KEY_CONF_KILLAFTER, pVariable ) )
			{ iKillAfter = atoi( pValue ); }
		else
			{
			os << "Unknown parameter '" << pVariable << "'" << ends;
			CONFIG_ERR( Object(), os.str() ); 
			return 0;
			};
		return 1;
		};

	/* --- callback functions for values --- */
/* --- macros to simplify parameter functions --- */
#define O PIDBTYPE_OPAQUE
#define S PIDBTYPE_STRING
#define F PIDBTYPE_RFC822
#define C(x,y) PIDB_lookup( pPIHTTP->pConnectionDB, x, y, 0 )
#define Q(x,y) PIDB_lookup( pPIHTTP->pRequestDB, x, y, 0 )
#define R(x,y) PIDB_lookup( pPIHTTP->pResponseDB, x, y, 0 )
#define V(x,y) PIDB_lookup( pPIHTTP->pHostDB, x, y, 0 )
#define SC_STR_FUNCTION(fn, value)\
static int (fn)( PIHTTP *pPIHTTP, void *, char *pszBuffer, int iLength ) \
{ \
    (void)pPIHTTP; \
    assert( pszBuffer ); \
    const char *pString = (const char *)(value); \
    if ( !pString ) { return 0; }; \
    int iValueLen = strlen( pString ); \
    int iLen = iValueLen < iLength ? iValueLen : iLength;   \
    if ( iValueLen>0 ) \
        { strncpy( pszBuffer, pString, iLen ); }; \
    return iValueLen; \
}

#   define SC_CXT_FUNCTION(fn, value, valuelen )\
static int (fn)( PIHTTP *pPIHTTP, void *pData, char *pszBuffer, int iLength ) \
{ \
    (void)pPIHTTP; \
	CGIContextData &tData = *( (CGIContextData *)pData ); \
    assert( pszBuffer ); \
	int iValueLen = (valuelen); \
	int iLen = iValueLen < iLength ? iValueLen : iLength; \
	if ( iLen>0 ) \
		{ strncpy( pszBuffer, value, iLen ); }; \
	return iValueLen; \
}

/* ---
some of these are just repeat of stuff from shortcuts
--- */
SC_CXT_FUNCTION( f_d, tData.sDataFile, tData.sDataFile.Len() );
SC_CXT_FUNCTION( f_i, tData.sInputFile, tData.sInputFile.Len() );
SC_CXT_FUNCTION( f_o, tData.sOutputFile, tData.sOutputFile.Len() );
SC_CXT_FUNCTION( f_p, tData.sExePath, tData.sExePath.Len() );
SC_CXT_FUNCTION( f_u, tData.sUniqueIndex, tData.sUniqueIndex.Len() );
SC_CXT_FUNCTION( f_q, tData.sIsIndexValue, tData.sIsIndexValue.Len() );
SC_CXT_FUNCTION( f_h, tData.sExtraHeaders, tData.sExtraHeaders.Len() );
SC_CXT_FUNCTION( f_a, tData.sAcceptHeader, tData.sAcceptHeader.Len() );
SC_CXT_FUNCTION( f_l, tData.sFormLiteral, tData.sFormLiteral.Len() );
SC_CXT_FUNCTION( f_e, tData.sFormExternal, tData.sFormExternal.Len() );
SC_CXT_FUNCTION( f_g, tData.sFormHuge, tData.sFormHuge.Len() );
SC_CXT_FUNCTION( f_f, tData.sFormFile, tData.sFormFile.Len() );

#undef S
#undef F
#undef C
#undef Q
#undef R

public:
	WinCGI( PIObject *pTheObject, int iArgc, const char *ppArgv[] )
	:
		HandlerBaseHTTP( pTheObject ),
		pPipeIO( 0 ),
		pDefaultCommandLine( 0 ),
		iExtraHeaders( 1 ),
		iSendCRLF( 0 ),
		pDataFile( 0 ),
		pStdinFile( 0 ),
		pStdoutFile( 0 ),
		pDataBlock( 0 ),
		iFlags( 0 ),
		iKillAfter( -1 ),
		pFKContentType( PIDB_getFastKey( KEY_HTTP_CONTENTTYPE,PIDBTYPE_RFC822)),
		pFKFileInfo( PIDB_getFastKey( KEY_DS_FILEINFO, PIDBTYPE_OPAQUE ) ),
		pFKPath( PIDB_getFastKey( KEY_INT_PATH, PIDBTYPE_STRING ) ),
		pFKURI( PIDB_getFastKey( KEY_HTTP_URI, PIDBTYPE_STRING ) )
		{
		/* --- clear out callback functions --- */
		memset( aFunctions, 0, sizeof( FnPi3Write ) * 256 );
		aFunctions['d'] = f_d;
		aFunctions['i'] = f_i;
		aFunctions['o'] = f_o;
		aFunctions['p'] = f_p;
		aFunctions['u'] = f_u;
		aFunctions['q'] = f_q;
		aFunctions['h'] = f_h;
		aFunctions['a'] = f_a;
		aFunctions['l'] = f_l;
		aFunctions['e'] = f_e;
		aFunctions['g'] = f_g;
		aFunctions['f'] = f_f;

		/* --- read parameters --- */
		ReadParameters( iArgc, ppArgv );
		if ( !IsOK() )
			{ return; };

		if ( pDefaultCommandLine==0 )
			{
			CONFIG_ERR( Object(), "WinCGI: 'DefaultCommandLine' not \
defined" );
			SetOK( 0 );
			return;
			};

		/* ---
		Load prototype Pipe IO object
		--- */
		pPipeIO = PIObject_loadEx( Object(), KEY_CONF_FILEIOOBJECT );
		if ( !pPipeIO )
			{
			SetOK( 0 );
			return;
			};
		};

	~WinCGI()
		{
		PIObject_delete( pPipeIO, 0, 0 );
		for( DblListIterator i( lIgnoreHeaders ); !i.BadIndex(); i++ )
			{ PI_DELETE( (PIString *)i.Current() ); };
		for( DblListIterator j( lCommandLineByExt ); !j.BadIndex(); j++ )
			{ PI_DELETE( (FileExtToCommandLineMap *)j.Current() ); };
		Pi3Expression_delete( pDefaultCommandLine );
		Pi3Expression_delete( pDataFile );
		Pi3Expression_delete( pStdinFile );
		Pi3Expression_delete( pStdoutFile );
		Pi3Expression_delete( pDataBlock );
		};

	/* ---
	Add client request headers to the buffer. Returns 0 on
	success, 1 on buffer exhausted and -1 on error.
	--- */
	int AddClientRequestHeaders( PIHTTP *pPIHTTP, PIDB *pQ, char *pBuf,	int iBufSize )
		{
		int i = 0;
		/* --- loop over all RFC822 headers in request block --- */
		PIDBIterator *pIter = PIDB_getIterator( pQ, PIDBTYPE_RFC822, 0, 0 );
		if ( !pIter )
			{ return -1; };
		const char *pKey;
		const char *pValue;
		for(; PIDBIterator_atValidElement( pIter ); PIDBIterator_next( pIter ) )
			{
			pValue = (const char *)PIDBIterator_current( pIter,	&pKey );
			assert( pKey && pValue );
			if ( !pKey || !pValue )
				{ /* sanity */ continue; };

			/* ---
			If the key matches one of the keys in the ignore list, then
			skip to the end of the loop
			--- */
			DblListIterator k( lIgnoreHeaders );
			for(; !k.BadIndex(); k++ )
				{
				PIString &tTmp = *( (PIString *)k.Current() );
				if ( !PIUtil_stricmp( tTmp, pKey ) )
					{ break; }; 
				};
			if ( !k.BadIndex() )
				{ continue; };

			/* ---
			pKey is a variable that is not on the ignore list,
			therefore it should be added to the buffer
			--- */
			int iVarLen = strlen( pKey );
			int iVarPos = sExtraHeadersPrefix.Len();
			int iValSize = strlen( pValue );									
			/* ---
			Size available must be able to fit
				- Variable name (iVarPos)
				- Value (iValPos)
				- 2..3 other characters, '=', ('\r'), '\n' for entry.
				- one byte less for iBufSizedue to trailing '\0'
			--- */
#if WIN32
#	define ADDPSPACE 3
#else
#	define ADDPSPACE 2
#endif
			if ( (i+iVarPos+iVarLen+iValSize+ADDPSPACE) < iBufSize - 1 )
#undef ADDPSPACE
				{
				memcpy( &(pBuf[i]), sExtraHeadersPrefix, iVarPos );
				i += iVarPos;
				for( int j=0; j<iVarLen; j++, i++ )							
					{																
					if ( isalnum( pKey[j] ) )										
						{ pBuf[i]=toupper(pKey[j]); }						
					else															
						{ pBuf[i]='_'; };									
					};
				pBuf[i++] = '=';
				memcpy( &(pBuf[i]), pValue, iValSize ); i+=iValSize;		
#if WIN32
				pBuf[i++] = '\r';
#endif
				pBuf[i++] = '\n';
				}
			else
				{
				PIDBIterator_delete( pIter );
				pBuf[i++] = '\0';
				return 1;
				};
			};
		pBuf[i++] = '\0';
		PIDBIterator_delete( pIter );
		return 0;
		};

	/* ---
	Add client accept header to the buffer. Returns 0 on
	success, 1 buffer exhausted and -1 on error.
	--- */
	int AddClientAcceptHeader( PIHTTP *pPIHTTP, PIDB *pQ, char *pBuf, int iBufSize )
		{
		int i = 0;
		const char *pKey;
		const char *pValue = (const char *)PIDB_lookup( pQ,
			PIDBTYPE_RFC822, KEY_HTTP_ACCEPT, 0);
		if (!pValue) return 0;

		/* --- loop over all mime types in Accept header --- */
		StringTokenizer tTokens( pValue, "," );
		for( int j=0; j<tTokens.NumTokens(); j++)
			{
			pKey = tTokens.GetToken( j );
			pValue = VALUE_YES;
			assert( pKey && pValue );
			if ( !pKey || !pValue )
				{ /* sanity */ continue; };

			/* ---
			test for additional parameters
			--- */
			if ( char *pPos = strchr( pKey, ';' ))
				{
				memset (pPos, '\0', 1);
				pValue = ++pPos;
				if ( pPos = strchr( pValue, '=' ))
					{
					pValue = ++pPos;
					}
				}

			int iVarSize = strlen( pKey );
			int iValSize = strlen( pValue );
			/* ---
			Size available must be able to fit
				- Variable name (iVarPos)
				- Value (iValPos)
				- 2..3 other characters, '=', ('\r'), '\n' for entry.
				- one byte less for iBufSizedue to trailing '\0'
			--- */
#if WIN32
#	define ADDPSPACE 3
#else
#	define ADDPSPACE 2
#endif
			if ( (i+iVarSize+iValSize+ADDPSPACE) < iBufSize - 1 )
#undef ADDPSPACE
				{
				memcpy( &(pBuf[i]), pKey, iVarSize ); i+=iVarSize;
				pBuf[i++] = '=';
				memcpy( &(pBuf[i]), pValue, iValSize );	i+=iValSize;
#if WIN32
				pBuf[i++] = '\r';
#endif
				pBuf[i++] = '\n';
				}
			else
				{
				return 1;
				};
			};
		pBuf[i++] = '\0';
		return 0;
		};

	/* ---
	Write client form data to an external temporary file. Returns 0 on
	success, -1 on error. The key for the [Form External] section is in pResult.
	--- */
	int WriteExternal( PIHTTP *pPIHTTP, Pi3String *pDecoded, PIString &tResult )
		{
		PIDB *pH = pPIHTTP->pHostDB;
		PIDB *pR = pPIHTTP->pResponseDB;

		/* --- separate the value to be written into external file --- */
		const char *pValue = strchr(Pi3String_getPtr(pDecoded), '=');
		pValue++;

		/* --- generate the unique path for the temporary file --- */
		enum { BUFSIZE=68 };
		char szBuf[BUFSIZE+1];
		PIPlatform_getUniqueId(szBuf, BUFSIZE-4);
		PIString sTmp(sExternalPath);
		sTmp.Concatenate(szBuf);
		sTmp.Concatenate(".tmp");

		Pi3String *pPath = Pi3String_new( sTmp );
		HTTPCore_relativeToAbsolutePath( pH, (const char *)sTmp, pPath );

		/* --- Caclulate the result string 'key=pathname length' --- */
		StringTokenizer tTokens( Pi3String_getPtr(pDecoded), "=" );
		tResult = (const char *)tTokens.GetToken( 0 );
		tResult.Concatenate('=');
		tResult.Concatenate(Pi3String_getPtr(pPath));
		tResult.Concatenate(' ');
		sprintf( szBuf, "%lu", strlen(pValue) );
		tResult.Concatenate( szBuf );

		/* --- try to write this data block to the datafile --- */
		PIPLATFORM_FD tFd = PIFile_open( Pi3String_getPtr(pPath), "w" );
			
		if ( tFd!=PIPLATFORM_FD_INVALID )
			{
			/* --- mark datafile as a temporary file --- */
			PIDB_add( pR, PIDBTYPE_STRING, KEY_INT_TEMPORARYFILE,
				(void *)Pi3String_getPtr(pPath), 0 );
			};

		int iFailed = ( tFd == PIPLATFORM_FD_INVALID ||
			PIFile_write( tFd, strlen(pValue), pValue )!=0 );

		if ( tFd!=PIPLATFORM_FD_INVALID )
			{ PIFile_close( tFd ); };

		if ( iFailed )
			{
			HTTPCore_logError( pPIHTTP,  
				"WinCGI: Failed to write temporary file \
for external form data, path is '%s'.",	Pi3String_getPtr(pPath) );
			Pi3String_delete(pPath);
			return -1;
			};

		Pi3String_delete(pPath);
		return 0;
		};

	/* ---
	Add client form data to the buffer. Returns 0 on
	success and -1 on error.
	--- */
	int AddClientFormData( PIHTTP *pPIHTTP, PIDB *pQ, char *pBuf,
		int iBufSize, PIString &tDecoded, PIString &tExternal )
		{
		Pi3String *pDecoded = Pi3String_new("");
		PIString pEncoded(pBuf, iBufSize);
		const char *pValue;

		/* --- loop over all form fields --- */
		StringTokenizer tTokens( pEncoded, "&" );
		for( int j=0; j<tTokens.NumTokens(); j++)
			{
			pEncoded = tTokens.GetToken( j );
			HTTPUtil_urlDecode( (Pi3String *)&pEncoded, pDecoded );
			pValue = strchr(Pi3String_getPtr(pDecoded), '=');
			if ( pValue )
				{
				if ( strlen(++pValue)>254
				  || strcspn(pValue,"=\"\n") < strlen(pValue) )
					{
					if (WriteExternal(pPIHTTP, pDecoded, pEncoded))
						{
						Pi3String_delete( pDecoded );
						return -1;
						};
					tExternal.Concatenate(pEncoded);
					tExternal.Concatenate("\r\n");
					continue;
					}
				};
			tDecoded.Concatenate(Pi3String_getPtr(pDecoded));
			tDecoded.Concatenate("\r\n");
			}
		Pi3String_delete( pDecoded );
		return 0;
		};

	/* ---
	Generate pipes and exeute CGI program.
	return 0 on success, otherwise error code
	--- */
	int DoExecCGIChild(
		char *pPath,				/* exec command line */
		const char *pDirectory,		/* CGI program current directory */
		PROCDESC *pProcess,			/* child process id value argument */
		int iDoStdin,				/* do standard input standard handle */
		FILEDESC *pStdout_Parent,	
		FILEDESC *pStdout_CGI,
		FILEDESC *pStdin_Parent,			
		FILEDESC *pStdin_CGI,
		FILEDESC *pStderr_Parent,
		FILEDESC *pStderr_CGI,
		PIHTTP &tPIHTTP
#if VERBOSE_DEBUG
		, ostream &os
#endif
		)
		{
		int iSanity = pPath && pProcess && pStdin_Parent && pStdout_Parent &&
			pStdout_CGI && pStdin_CGI;
		assert( iSanity );
		if ( !iSanity )
			{ return -1; };

		/* --- CGI stdout handles --- */
		FILEDESC tStdout_CGI = INVALID_DESC;
		FILEDESC tStdout_Parent = INVALID_DESC;
		FILEDESC tStdout_ParentDup = INVALID_DESC;

		/* --- CGI stdin handles --- */
		FILEDESC tStdin_CGI = INVALID_DESC;
		FILEDESC tStdin_Parent = INVALID_DESC;
		FILEDESC tStdin_ParentDup = INVALID_DESC;

		/* --- CGI stderr handles --- */
		FILEDESC tStderr_CGI = INVALID_DESC;
		FILEDESC tStderr_Parent = INVALID_DESC;
		FILEDESC tStderr_ParentDup = INVALID_DESC;

#if WIN32
		BOOL bProcessCreated = FALSE;
		DWORD dwFlags = 0;
		char *AppName = 0;

		STARTUPINFO tStartUp;
		memset(&tStartUp, 0, sizeof(tStartUp));
		tStartUp.cb=sizeof(tStartUp);

		tStartUp.dwFlags = STARTF_USESHOWWINDOW;
		if ( HTTPCore_debugEnabled() )
			{ tStartUp.wShowWindow=SW_SHOW; }
		else
			{ tStartUp.wShowWindow=SW_HIDE; };

		/*
		** Execution flags
		*/
		if ( iFlags & FLG_16BIT )
			{ dwFlags |= CREATE_SEPARATE_WOW_VDM; };

		PROCESS_INFORMATION tProcessInfo;

		/*
		** complete command line for associations
		*/
		AppName = (char *)PIUtil_malloc(_MAX_PATH); 
		// Set AppName if the script name in pPath is associated
		if ((int)FindExecutable(pPath, NULL, AppName) < 33)
			{
			PIUtil_free(AppName);
			AppName = NULL;
			};

		/* --- spawn the CGI program --- */
		bProcessCreated = CreateProcess(
				AppName,			/* address of module name */
				(char *)pPath,		/* address of command line */
				NULL,				/* address of process security attributes */
				NULL,				/* address of thread security attributes */
				FALSE,              /* new process inherits handles */
				dwFlags,			/* creation flags */
				NULL,				/* address of new environment block */
				pDirectory,			/* address of current directory name */
				&tStartUp,			/* address of STARTUPINFO */
				&tProcessInfo		/* address of PROCESS_INFORMATION */
				);

		PIUtil_free(AppName);

		if ( !bProcessCreated )
			{ goto nt_error_return; };

		/* --- log debug information about the process --- */
		HTTPCore_logDebug( DBG_MED, "Process created!, \n\thProcess %p\
\n\thThread %p\n\tdwProcessId %lu\t\n\tdwThreadId %lu",
			tProcessInfo.dwThreadId,
			tProcessInfo.dwProcessId,
			tProcessInfo.hThread,
			tProcessInfo.hProcess );

#	if VERBOSE_DEBUG
		os << "Process created!" << endl;
		os << "	hProcess " << hex << tProcessInfo.hProcess << endl;
		os << "	hThread " << hex << tProcessInfo.hThread << endl;
		os << "	dwProcessId " << hex << tProcessInfo.dwProcessId << endl;
		os << "	dwThreadId " << hex << tProcessInfo.dwThreadId << endl;
#	endif

		/* --- Handle to main thread in new process --- */
		::CloseHandle( tProcessInfo.hThread );

		*pProcess = tProcessInfo.hProcess;
		*pStdout_Parent = tStdout_ParentDup;
		*pStdout_CGI = tStdout_CGI;
		if ( iDoStdin )
			{
			*pStdin_Parent = tStdin_ParentDup;
			*pStdin_CGI = tStdin_CGI;
			};
		*pStderr_Parent = tStderr_ParentDup;
		*pStderr_CGI = tStderr_CGI;

		/* --- OK --- */
		return 0;

nt_error_return:
		/* --- restore handles --- */
		int iTmp = GetLastError();
		goto generic_error_return;

#elif POSIX
		const char **ppCommandArgs=0;
		register int iPid = 0;

		/* ---
		Create command line
		--- */
		{
		/* --- scan once and NULL seperate path components --- */
		int i;
		int j = strlen( pPath );
		int k=0;
		for( i=0; i<j; i++ )
			{
			if ( (isspace(pPath[i])) )
				{
				pPath[i]='\0';
				k++;
				};
			}; 
	
		/* ---
		k contains number of spaces in command line.

		cmd1[sp]cmd2[sp]cmd3[sp]cmd4[0],

			increment k to include command terminated by final null.
			increment k again to allow NULL command at end of array.
		--- */
		k += 2;

		/* --- allocate and assign argument array --- */
		ppCommandArgs = (const char **)PIHTTP_allocMem( &tPIHTTP,
			sizeof( const char * ) * k ); 
		int x = 0;
		const char *pLastArg = 0;
		for( i=0; i<=j; i++ )
			{
			if ( pLastArg==0 )
				{ pLastArg=&(pPath[i]); };
			if ( pPath[i]=='\0' )
				{
				ppCommandArgs[x++] = pLastArg;
				pLastArg = 0;
				};
			};
		ppCommandArgs[x++] = 0;
		assert( x==k );
		};

		/* ---
		Try not to write any data to the processes data space to avoid
		copy-on-write penalty for CGI child process. 
		iPid is declared register so its assignment on fork in the child
		process will not write to the child process data space.
		--- */
#if 0
/*
** NOTE
*/
#define P ( PIThread_getSystemHandle( PIThread_getCurrent() ) )
cerr << P << " 1.." << tStdin_Parent << endl;
cerr << P << " 2.." << tStdin_CGI << endl;
cerr << P << " 3.." << tStdout_Parent << endl;
cerr << P << " 4.." << tStdout_CGI << endl;
cerr << P << " 5.." << tStderr_Parent << endl;
cerr << P << " 6.." << tStderr_CGI << endl;
#endif
		iPid = ::fork();
		switch( iPid )
			{
			case -1:
				break;

			case 0:
				/* --- change current directory --- */
				if ( pDirectory )
					{ ::chdir( pDirectory ); };
				extern char **environ;
				if ( ::execve( pPath, (char *const *)ppCommandArgs,
					environ )==-1 )
					{
					perror( "CGI child, execve() failed" );
					fprintf( stderr, "Path is '%s'\n", pPath );
					fflush( stderr );
					goto cgi_error;
					};
cgi_error:
				::_exit( 1 );

				/* none of the following should happen */
				::_exit( -1 );
				assert( 0 );
				break;

			default:
				/* --- this is the parent --- */
				*pProcess = iPid;
				*pStdout_CGI = tStdout_CGI;
				*pStdout_Parent = tStdout_Parent;
				*pStderr_CGI = tStderr_CGI;
				*pStderr_Parent = tStderr_Parent;
				if ( iDoStdin )
					{
					*pStdin_CGI = tStdin_CGI;
					*pStdin_Parent = tStdin_Parent;
					};

				/* --- log debug information about the process --- */
				HTTPCore_logDebug( DBG_MED, "Process created!, \n\tPid %d\
\n\tStdout_Parent %d\n\tStdout_CGI %d\n\tStderr_Parent %d\n\tStderr_CGI %d",
					iPid, tStdout_Parent, tStdout_CGI, tStderr_Parent,
					tStderr_CGI );

				/* --- OK --- */
				return 0;
			};

		/* --- error --- */
posix_error_return:
		int iTmp = errno;
		goto generic_error_return;

#endif


generic_error_return:
		/* --- stdout handles --- */
		CloseIfValidDesc( tStdout_CGI );
		CloseIfValidDesc( tStdout_Parent );
		CloseIfValidDesc( tStdout_ParentDup );

		/* --- stdin handles --- */
		CloseIfValidDesc( tStdin_CGI );
		CloseIfValidDesc( tStdin_Parent );
		CloseIfValidDesc( tStdin_ParentDup );

		/* --- stdin handles --- */
		CloseIfValidDesc( tStderr_CGI );
		CloseIfValidDesc( tStderr_Parent );
		CloseIfValidDesc( tStderr_ParentDup );

		HTTPCore_logError( &tPIHTTP, "WinCGI: error executing CGI program \
system specific error code is '%d'.", iTmp );

		/* --- return error --- */
		return iTmp ? iTmp : -1;
		};

	/* ---
	Wait for a child process to terminate
	--- */
	bool WaitForChild( const char *pszCmdLine, PIHTTP &tPIHTTP,
		PROCDESC tProcess )
		{
		int iRet = PIPlatform_waitForProcess( (int)tProcess, 0, iKillAfter );
		switch( iRet )
			{
			case PIAPI_COMPLETED:	return true;
			case PIAPI_TIMEOUT:
				HTTPCore_logError( &tPIHTTP,  "WinCGI: CGI \
process did \
not terminate after %d seconds.\nCGI program command line is '%s'.\n\
The process will be terminated.", iKillAfter, pszCmdLine );
#	if WIN32
				::TerminateProcess( tProcess, (UINT)-1 );
#	elif POSIX
				::kill( (pid_t)tProcess, SIGTERM );
#	endif
				return false;

			default:
				HTTPCore_logError( &tPIHTTP,  "WinCGI: Error waiting for \
CGI process to terminate.\nCGI program command line is '%s'",
					pszCmdLine );
				return false;
			};
		}

	/* ---
	Write output from one buffer into another. Transfer data until no more
	data can be read.
	Returns 0 on success.
	--- */
	int WriteBufferToBuffer( PIIOBuffer &tInBuffer, PIIOBuffer &tOutBuffer )
		{
		int iLen;
		const char *pBuffer = PIIOBuffer_read( &tInBuffer, &iLen );
		while( pBuffer )
			{
			if ( PIIOBuffer_write( &tOutBuffer, pBuffer, iLen, PIIOBUF_NONE )
				<=0 )
				{
				assert( 0 );
				return -1;
				};
			pBuffer = PIIOBuffer_read( &tInBuffer, &iLen );
			};
		return 0;
		};

	/* ---
	Write output from one buffer into another.
	iReadLen gives number of bytes to transfer, must be > 0
	Returns 0 on success.
	--- */
	int WriteBufferToBuffer( PIIOBuffer &tInBuffer, PIIOBuffer &tOutBuffer,
		int iReadLen, PIString &sResult )
		{
		assert( iReadLen>0 );
		if ( iReadLen<=0 )
			{ return 0; };
		enum { BUF_SIZE=4096 };
		char szBuffer[BUF_SIZE];
		int iAttemptToRead = (iReadLen >= BUF_SIZE) ? BUF_SIZE : iReadLen;
		int iInKey = 0;
		int iValLen = 0;
		int iStdinLen = iReadLen;

		int iLen;
		iLen = PIIOBuffer_readToBuffer( &tInBuffer, szBuffer, iAttemptToRead );

		while( iLen>0 )
			{
			if ( PIIOBuffer_write( &tOutBuffer, szBuffer, iLen,
				PIIOBUF_NONE )<=0 )
				{
				assert( 0 );
				return -1;
				};

		/* --- Scan the buffer for [Form Huge] literals --- */
		if (iStdinLen > 8192)
			{
			int iScanned = 0;
			enum { KEYSIZE=255 };
			char szKeyBuf[KEYSIZE+1];
			int i;
			for (i=0; i<iLen; i++)
				{
				if (szBuffer[i] == '=')
					{
					strncpy(szKeyBuf, &szBuffer[iScanned], i-iScanned+1);
					szKeyBuf[i-iScanned+1]='\0';
					sResult.Concatenate(szKeyBuf);
					sprintf( szKeyBuf, "%lu", iStdinLen-iReadLen+i+1 );
					sResult.Concatenate( szKeyBuf );
					sResult.Concatenate(' ');
					iScanned = i;
					iInKey = 0;
					}
				if (szBuffer[i] == '&')
					{
					sprintf( szKeyBuf, "%lu", iValLen+i-iScanned-1 );
					sResult.Concatenate( szKeyBuf );
#if WIN32
					sResult.Concatenate("\r\n");
#else
					sResult.Concatenate("\n");
#endif
					iScanned = i+1;
					iInKey = 1;
					iValLen = 0;
					}
				}
				if (iInKey)
					{
					strncpy(szKeyBuf, &szBuffer[iScanned], i-iScanned+1);
					szKeyBuf[i-iScanned+1]='\0';
					sResult.Concatenate(szKeyBuf);
					}
				else
					{
					if (iReadLen - iLen)
						{
						iValLen += i-iScanned;
						}
					else
						{
						sprintf( szKeyBuf, "%lu", iValLen+i-iScanned-1 );
						sResult.Concatenate( szKeyBuf );
#if WIN32
					sResult.Concatenate("\r\n");
#else
					sResult.Concatenate("\n");
#endif
						}
					};
			}

			iReadLen -= iLen;
			if ( !iReadLen )
				{
                return 0;
                };

    		iAttemptToRead = (iReadLen >= BUF_SIZE) ? BUF_SIZE : iReadLen;
			iLen = PIIOBuffer_readToBuffer( &tInBuffer, szBuffer,
				iAttemptToRead );
			};

		/* ---
		Done
		--- */
		return iLen;
		};

	/* ---
	Send input data to the CGI program using a copy of the pipe
	IO object. Return non-zero on success, 0 on error.
	--- */
	int SendCGIInput( PIIOBuffer &tIn, PIIOBuffer &tOut, int iLen, PIString &sResult )
		{
		int iRet = 1;	/* initially success */

		/* ---
		Write input to child stdin and close CGI side of pipe
		--- */
		if ( WriteBufferToBuffer( tIn, tOut, iLen, sResult ) )
			{
			iRet = 0;
			goto SendCGIInput_error;
			};

		if ( iSendCRLF )
			{
			/* --- Write CRLF to the CGI program --- */
			if ( PIIOBuffer_writeLn( &tOut, "", 0, PIIOBUF_NONE )<=0 )
				{
				iRet = 0;
				goto SendCGIInput_error;
				};
			};

		/* --- flush --- */
		if ( !PIIOBuffer_flush( &tOut ) )
			{
			iRet = 0;
			goto SendCGIInput_error;
			};

		/* --- done --- */
SendCGIInput_error:
		if ( !iRet )
			{
			HTTPCore_logDebug( DBG_MED, "WinCGI: SendCGIInput() failed." );
			};

		return iRet;
		};

	/* ---
	Execute the CGI program and examine the output.
	--- */
	bool DoExecCGI(
		PIFInfo *pFile,
		PIHTTP &tPIHTTP,
		PROCDESC &tProcess )
		{
		/* ---
		Normally the callee to DoExecCGI would generate an internal redirect
		to handle an error code. The flag bIgnoreStatus can be set to 
		true to tell the callee that a redirect should not happen, because
		the protocol headers have already been sent to the client. This
		happens for full response
		--- */
		bool bIgnoreStatus = false;

		PIDB *pQ = tPIHTTP.pRequestDB;
		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pC = tPIHTTP.pConnectionDB;
		PIDB *pH = tPIHTTP.pHostDB;
		PIIOBuffer &tB = *( tPIHTTP.pBuffer );

		/* --- sanity check --- */
		assert( pFile && pQ && pR );
		if ( !( pFile && pQ && pR ) )
			{
			tPIHTTP.iStatus = ST_INTERNALERROR;
			return bIgnoreStatus;
			};

		/* --- 
		Make CGI context data 
		--- */
		CGIContextData tContext;

		/* ---
		Determine whether or not there is any input to be sent to the
		CGI program
		--- */
		const char *pContentLength = (const char *)PIDB_lookup( pQ,
			PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, 0 );
		int iStdinLen = pContentLength ? atoi( pContentLength ) : 0;
		int iDoStdin = ( iStdinLen > 0 );

		/* ---
		Set filenames 
		--- */
		enum { BUFSIZE=1023 };
		char szFileBuf[BUFSIZE+1];
		if ( pDataFile ) {
			int iLen = Pi3Expression_write( pDataFile, &tPIHTTP, &tContext,
				szFileBuf, BUFSIZE );
			if ( iLen>0 )
				{
				szFileBuf[iLen] = '\0';
				Pi3String *pTmp = Pi3String_new( tContext.sDataFile );
				HTTPCore_relativeToAbsolutePath( pH, szFileBuf, pTmp );
				tContext.sDataFile = Pi3String_getPtr( pTmp );
				Pi3String_delete( pTmp );
				};
			};
		if ( pStdinFile ) {
			int iLen = Pi3Expression_write( pStdinFile, &tPIHTTP, &tContext,
				szFileBuf, BUFSIZE );
			if ( iLen>0 )
				{
				szFileBuf[iLen] = '\0';
				Pi3String *pTmp = Pi3String_new( tContext.sInputFile );
				HTTPCore_relativeToAbsolutePath( pH, szFileBuf, pTmp );
				tContext.sInputFile = Pi3String_getPtr( pTmp );
				Pi3String_delete( pTmp );
				};
			};
		if ( pStdoutFile ) {
			int iLen = Pi3Expression_write( pStdoutFile, &tPIHTTP, &tContext,
				szFileBuf, BUFSIZE );
			if ( iLen>0 )
				{
				szFileBuf[iLen] = '\0';
				Pi3String *pTmp = Pi3String_new( tContext.sOutputFile );
				HTTPCore_relativeToAbsolutePath( pH, szFileBuf, pTmp );
				tContext.sOutputFile = Pi3String_getPtr( pTmp );
				Pi3String_delete( pTmp );

				/* --- mark output file as a temporary file --- */
				PIDB_add( pR, PIDBTYPE_STRING, KEY_INT_TEMPORARYFILE,
					(void *)(const char *)tContext.sOutputFile, 0 );
				};
			};

		/* ---
		See if this is an 'ISINDEX' query, and if it is, set the
		ISINDEX value appropriately

		An ISINDEX query will not have a '=' encoded in the query string.
		--- */
		const char *pQS = (const char *)PIDB_lookup( pQ, PIDBTYPE_STRING,
			KEY_HTTP_QUERYSTRING, 0 );
		Pi3String *pEncodedQS = Pi3String_new( pQS );
		Pi3String *pDecodedQS = Pi3String_new( 0 );
		HTTPUtil_urlDecode( pEncodedQS, pDecodedQS );
		const char *pDQS = Pi3String_getPtr( pDecodedQS );
		int i;
		for( i=0; pDQS[i] && pDQS[i]!='='; i++ );
		if ( !pDQS[i] )
			{
			/* --- '=' not found, therefore this is an ISINDEX query --- */
		
			/* ---
			Only include up to the first punctuation character, to
			keep out 'special' parameters being passed to exec(). Thi
			is a security message
			--- */
			for( i=0; pDQS[i] && !ispunct(pDQS[i]); i++ );
			PIString sTmp( pDQS, i );
			tContext.sIsIndexValue = " ";
			tContext.sIsIndexValue.Concatenate( sTmp );
			};
		Pi3String_delete( pEncodedQS );
		Pi3String_delete( pDecodedQS );

		/* ---
		Write extra headers to context
		--- */
		if ( iExtraHeaders )
			{
			switch (AddClientRequestHeaders( &tPIHTTP, pQ, szFileBuf, BUFSIZE+1 )) {
				case 0:
					tContext.sExtraHeaders = szFileBuf;
				    break;
				case -1:
					HTTPCore_logError( &tPIHTTP,  
						"WinCGI: Internal error getting data of extra request headers." );
				    tPIHTTP.iStatus = ST_INTERNALERROR;
				    return bIgnoreStatus;
				case 1:
					HTTPCore_logError( &tPIHTTP,  
						"WinCGI: Buffer exhausted when getting data of extra request headers" );
				    tPIHTTP.iStatus = ST_INTERNALERROR;
				    return bIgnoreStatus;
				}
			};

		/* ---
		Write accept header to context
		--- */
		if ( iExtraHeaders )
			{
			switch (AddClientAcceptHeader( &tPIHTTP, pQ, szFileBuf, BUFSIZE+1 )) {
				case 0:
					tContext.sAcceptHeader = szFileBuf;
				    break;
				case -1:
					HTTPCore_logError( &tPIHTTP,  
						"WinCGI: Internal error getting data of accept header." );
				    tPIHTTP.iStatus = ST_INTERNALERROR;
				    return bIgnoreStatus;
				case 1:
					HTTPCore_logError( &tPIHTTP, 
						"WinCGI: Buffer exhausted when getting data of accept header." );
				    tPIHTTP.iStatus = ST_INTERNALERROR;
				    return bIgnoreStatus;
				}
			};

		FILEDESC tStdout_Parent = INVALID_DESC;
		FILEDESC tStdout_CGI = INVALID_DESC;
		FILEDESC tStdin_Parent = INVALID_DESC;
		FILEDESC tStdin_CGI = INVALID_DESC;
		FILEDESC tStderr_Parent = INVALID_DESC;
		FILEDESC tStderr_CGI = INVALID_DESC;
		/*
		** Define a macro to make sure these handles are all closed
		** if we return in the middle of these functions
		*/
#define CLOSE_ALL_HANDLES							\
		{										\
		CloseIfValidDesc( tStdin_Parent );		\
		CloseIfValidDesc( tStdin_CGI ); 		\
		CloseIfValidDesc( tStdout_Parent ); 	\
		CloseIfValidDesc( tStdout_CGI );		\
		CloseIfValidDesc( tStderr_Parent ); 	\
		CloseIfValidDesc( tStderr_CGI ); 		\
		}
		
		tContext.sExePath = PIFInfo_getPath( pFile );
		const char *pDirectory = PIFInfo_getPathRoot( pFile );

		/* ---
		Find the appropriate command line pattern 
		--- */
		Pi3Expression *pCommandPattern = pDefaultCommandLine;
		const char *pExtn = PIFInfo_getExtension( pFile );
		for( DblListIterator k( lCommandLineByExt ); !k.BadIndex(); k++ )
			{
			FileExtToCommandLineMap *pMap =
					(FileExtToCommandLineMap *)k.Current();
			if ( pMap->sFileExtension==pExtn ) 
				{
				pCommandPattern = pMap->pExpr;
				break;
				};
			};
	
		/* ---
		Write input to input file
		--- */
		PIPLATFORM_FD tFd = PIFile_open( tContext.sInputFile, "w" );

		if ( tFd==PIPLATFORM_FD_INVALID )
			{
			HTTPCore_logError(  &tPIHTTP, 
				"WinCGI: Failed to open input file, path is '%s'.",
				((const char *)tContext.sInputFile) );
			tPIHTTP.iStatus = ST_INTERNALERROR;
			return bIgnoreStatus;
			};

		/* --- mark input file as a temporary file --- */
		PIDB_add( pR, PIDBTYPE_STRING, KEY_INT_TEMPORARYFILE,
			(void *)(const char *)tContext.sInputFile, 0 );

		if ( iDoStdin && iStdinLen>0 )
			{
			const char *pArgs[3];
			pArgs[0] = (const char *)-1;
			pArgs[1] = (const char *)tFd;
			pArgs[2] = (const char *)0;     /* handle is a pipe - no? */
			PIObject *pChildIO = PIObject_copy( pPipeIO, 3, pArgs );
			PIIOBuffer *pCGIBuffer = 0;
			int iResult = 0;
			if ( pChildIO )
				{
				pCGIBuffer = PIIOBuffer_new( pChildIO );
				iResult = SendCGIInput( tB, *pCGIBuffer, iStdinLen, tContext.sFormHuge );

				if ( !iResult )
					{
					HTTPCore_logError(  &tPIHTTP, 
						"WinCGI: Failed to write input file, path is '%s'.",
						((const char *)tContext.sInputFile) );
					tPIHTTP.iStatus = ST_INTERNALERROR;
					PIIOBuffer_delete( pCGIBuffer );
					PIObject_delete( pChildIO, 0, 0 );
					PIFile_close( tFd );
					return bIgnoreStatus;
					};

				/*
				** Tricky! The flushed buffer points to the begin of the old data
				** (We expect a flushed buffer from SendCGIInput)
				**
				** We currently support only up to 8192 Bytes (PIIOBuffer size)
				** for [Form Literal] or [Form External], then [Form Huge] has been
				** used by SendCGIInput() already!
				*/
				char *pBuf;
				if ( iStdinLen <= PIIOBuffer_getOutputBuffer( pCGIBuffer, &pBuf ))
					{
					/* ---
					Write accept header to context
					--- */
					if (AddClientFormData( &tPIHTTP, pQ, pBuf, iStdinLen,
						tContext.sFormLiteral, tContext.sFormExternal))
						{
						HTTPCore_logError( &tPIHTTP,  
							"WinCGI: Internal error getting data of request form literals." );
						tPIHTTP.iStatus = ST_INTERNALERROR;
						PIIOBuffer_delete( pCGIBuffer );
						PIObject_delete( pChildIO, 0, 0 );
						PIFile_close( tFd );
						return bIgnoreStatus;
						}
					}

				PIIOBuffer_delete( pCGIBuffer );
				PIObject_delete( pChildIO, 0, 0 );
				};	
			};

		PIFile_close( tFd );

		/* ---
		If a datafile name pattern was specified write the data block
		to that file
		--- */
		if ( pDataFile && pDataBlock )
			{
			int iDataSize = 4096;	/* --- initial guess --- */
			int iLen = -1;
			char *pData = 0;
			for(;;)
				{
				pData = (char *)PIHTTP_allocMem( &tPIHTTP, iDataSize );
				iLen = Pi3Expression_write( pDataBlock, &tPIHTTP, &tContext,
					pData, iDataSize );

				if ( iLen==-1 )
					{ break; }; 

				/* --- try again if buffer was not big enough --- */
				if ( iLen>iDataSize ) {
					iDataSize = iLen; 
					continue;
					};

				break;
				};

			/* ---
			Error getting data block ?
			--- */
			if ( iLen==-1 )
				{
				assert( 0 );
				HTTPCore_logError( &tPIHTTP,  
						"WinCGI: Internal error getting data for datafile '%s'.",
					((const char *)tContext.sDataFile) );
				tPIHTTP.iStatus = ST_INTERNALERROR;
				return bIgnoreStatus;
				};

			/* --- try to write this data block to the datafile --- */
			PIPLATFORM_FD tFd = PIFile_open( tContext.sDataFile, "w" );
			
			if ( tFd!=PIPLATFORM_FD_INVALID )
				{
				/* --- mark datafile as a temporary file --- */
				PIDB_add( pR, PIDBTYPE_STRING, KEY_INT_TEMPORARYFILE,
					(void *)(const char *)tContext.sDataFile, 0 );
				};

			int iFailed = ( tFd == PIPLATFORM_FD_INVALID ||
							PIFile_write( tFd, iLen, pData )!=0 );
			if ( tFd!=PIPLATFORM_FD_INVALID )
				{ PIFile_close( tFd ); };
			if ( iFailed )
				{
				HTTPCore_logError( &tPIHTTP,  
					"WinCGI: Failed to write datafile, path is '%s'.",
					((const char *)tContext.sDataFile) );
				tPIHTTP.iStatus = ST_INTERNALERROR;
				return bIgnoreStatus;
				};
			};

		/* ---
		Generate command line pattern
		--- */
		enum { CMD_BUF_SIZE=255 };/* --- this will fit most command lines --- */
		char szCmdBuf[CMD_BUF_SIZE+1];
		char *pCommandLine = szCmdBuf;
		int iLen = Pi3Expression_write( pCommandPattern, &tPIHTTP, &tContext,
			pCommandLine, CMD_BUF_SIZE );
		szCmdBuf[CMD_BUF_SIZE] = '\0';
		if ( iLen>CMD_BUF_SIZE )
			{
			/* --- automatic buffer not big enough, allocate a new one --- */
			pCommandLine = (char *)PIHTTP_allocMem( &tPIHTTP, iLen+1 );
			Pi3Expression_write( pCommandPattern, &tPIHTTP, &tContext, 
				pCommandLine, iLen );
			Pi3Expression_write( pCommandPattern, &tPIHTTP, &tContext,
				pCommandLine, iLen );
			};
		pCommandLine[iLen] = '\0';

#if VERBOSE_DEBUG
		cerr << "Executing commandline: '" << pCommandLine << "'" << endl;
#endif
		int iRCode = DoExecCGIChild( 
				pCommandLine, 
				pDirectory,
				&tProcess, 
				iDoStdin,
				&tStdout_Parent,
				&tStdout_CGI,
				&tStdin_Parent,
				&tStdin_CGI,
				&tStderr_Parent,
				&tStderr_CGI,
				tPIHTTP
#if VERBOSE_DEBUG
				, cerr
#endif
				);

		/* ---
		Error in CGI program
		--- */
		if ( iRCode )
			{
			HTTPCore_logError( &tPIHTTP,  
				"WinCGI: DoExecCGIChild() failed. Error code is %d. \
Start of command line is '%s'.", iRCode, szCmdBuf );
			tPIHTTP.iStatus = ST_INTERNALERROR;
			CLOSE_ALL_HANDLES;
			return bIgnoreStatus;
			};

		/*
		** pError serves as error flag
		*/
		int iRet = 0;
		const char *pError = 0;
		const char *pArgs[3];
		PIObject *pChildIO = 0;
		PIIOBuffer *pCGIBuffer = 0;

		CloseIfValidDesc( tStdin_CGI );
		CloseIfValidDesc( tStdin_Parent );

		/* ---
        Wait for process to die.
		---- */
		if ( !WaitForChild( szCmdBuf, tPIHTTP, tProcess ) )
			{ iRet = -1;	/* error message already given */ };
		CloseProcessDesc( tProcess );
		tProcess = INVALID_DESC;

		/* ---
		Close CGI side of pipe
		--- */
		CloseFileDesc( tStdout_CGI );

		/* ---
		Read and discard stderr, write this CGI error to server error file
		--- */
		enum { BUF_SIZE=2047 };
		char szBuf[BUF_SIZE+1];

		/* ---
		CGI output is from a file, open the file
		--- */
		assert( tStdout_Parent == INVALID_DESC );
		tStdout_Parent = (FILEDESC)PIFile_open( tContext.sOutputFile, "r" );

		/* ---
		Make a clone of the prototype pipe IO object for communication
		with this CGI process
		--- */
		pArgs[0] = (const char *)tStdout_Parent;
		pArgs[1] = (const char *)tStdin_Parent;
		pArgs[2] = (const char *)0;		/* handle is a pipe? - no */
		pChildIO = PIObject_copy( pPipeIO, 3, pArgs );
		if ( !pChildIO )
			{
			pError = "WinCGI: Failed to open communication channel to CGI \
program.";
			}
		else
			{
			/* ---
			Make CGI buffer object
			--- */
			pCGIBuffer = PIIOBuffer_new( pChildIO );
			assert( pCGIBuffer );
			};

		/* ---	
		Read CGI request headers
		--- */
		if ( !pError && !iRet )
			{
			if ( HTTPCore_readHeaders( pCGIBuffer, pR, RH_RESPONSE ) )
				{ pError = "Error reading response headers from CGI program"; };
			};

		if ( pError )
			{
			if ( *pError )
				{
				HTTPCore_logError( &tPIHTTP, "WinCGI: %s.\nStart of command line \
is '%s'.", pError, szCmdBuf );
				}
			else
				{
				HTTPCore_logError( &tPIHTTP, "WinCGI: Start of command line \
is '%s'.", szCmdBuf );
				};
			iRet = -1;
			};
#if VERBOSE_DEBUG
		cerr << dec << iRet << endl;
#endif
		
		/* ---
		Check the type of the response
			- Partial (usual case)
			- Full
			- or redirect
		--- */
		const char *pCLF = 0;
		const char *pLocation = 0;
		if ( !iRet && ( pCLF = (const char *)
			PIDB_lookup( pR, PIDBTYPE_STRING, KEY_HTTP_CLF, 0 ) ) ) 
			{

			/* ++++ __________________________ +++ *
				Full Response (Non-Parsed Headers)
			 * ++++ __________________________ +++ */

			/* --- grap the status code sent by the CGI --- */
			int i=0;
			for(; pCLF[i] && !(isspace(pCLF[i])); i++); /* advance to space */
			for(; pCLF[i] && (isspace(pCLF[i])); i++); /* advance past space */
			tPIHTTP.iStatus = atoi( &(pCLF[i]) );

			/* ---
			disable keep open, hope the CGI sent the appropriate 
			'Connection' headers for this HTTP version
			--- */
			PIDB_replace( tPIHTTP.pConnectionDB, PIDBTYPE_OPAQUE, 
				KEY_INT_KEEPOPEN, (void *)0, 0 );

			/* --- send status header from CGI --- */
			PIIOBuffer_writeLn( &tB, pCLF, -1, PIIOBUF_NONE );

			/* --- loop over RFC822 headers to send them --- */
			PIDBIterator *pIter = PIDB_getIterator( pR, PIDBTYPE_RFC822, 0, 0 );
			for( ;
				PIDBIterator_atValidElement( pIter );
				PIDBIterator_next( pIter ) )
				{
				const char *pHeader;
				const char *pContent = (const char *)
					PIDBIterator_current( pIter, &pHeader );
				PIIOBuffer_write( &tB, pHeader, -1, PIIOBUF_NONE );
				PIIOBuffer_write( &tB, ": ", 2, PIIOBUF_NONE );
				PIIOBuffer_writeLn( &tB, pContent, -1, PIIOBUF_NONE );
				};
			PIDBIterator_delete( pIter );
			
			/* --- empty CRLF --- */
			PIIOBuffer_writeLn( &tB, "", 0, PIIOBUF_NONE );

			/* ---
			for a full reponse no REDIRECT action should be
			taken based on the status code, as the CGI program is handling
			the status itself
			--- */
			bIgnoreStatus = true;

			/* --- continue to allow the content-body to be sent --- */
			}
		else if ( !iRet && (pLocation=(const char *)
			PIDB_lookup( pR, PIDBTYPE_RFC822, KEY_HTTP_LOCATION, 0 ) ) )
			{

			/* ++++ __________________________ +++ *
					  Location Response
			 * ++++ __________________________ +++ */

			/* --- determine if this is a URL path or a URI --- */
			if ( *pLocation=='/' )
				{
				/* ---
				URL path, expand to full URI and replace in Location
				--- */
				PIOStrStream ostr;
				if ( PIDB_lookup( pC, PIDBTYPE_STRING, KEY_HTTPS_KEYSIZE, 0 ) )
					{
					/*
					** Running under SSL
					*/
					ostr << "https://";
					}
				else
					{
					/*
					** Not running under SSL
					*/
					ostr << "http://";
					};
				const char *pHostName = HTTPUtil_getHostName( &tPIHTTP );
				if ( pHostName )
					{ ostr << pHostName; };
				const char *pPort = HTTPUtil_getHostPort( &tPIHTTP );
				if ( pPort )
					{ ostr << ':' << pPort; };

				ostr << pLocation;
				ostr << ends;

				PIDB_replace( pR, PIDBTYPE_RFC822, KEY_HTTP_LOCATION,
						(void *)ostr.str(), 0 );
				};

			/* --- change status to permanent redirect --- */
			tPIHTTP.iStatus = ST_PERMANENTREDIRECT;

			/* ---
			Has the CGI program sent any content ?
			--- */
			if ( !PIIOBuffer_pollBeforeRead( pCGIBuffer ) )
				{
				/* --- 
				No content 
				--- */
				if ( pCGIBuffer )
					{ PIIOBuffer_delete( pCGIBuffer ); };
				PIObject_delete( pChildIO, 0, 0 );
				CLOSE_ALL_HANDLES;
				return bIgnoreStatus;
				};

			/* ---
			for a location reponse with no content no REDIRECT action should be
			taken based on the status code, as the CGI program is handling
			the content itself
			--- */
			bIgnoreStatus = true;

			/* ---
			start to send the response and allow the 
			output from the CGI to be sent to the client as entity 
			body for the request.
			--- */
			PIDB_replace( tPIHTTP.pConnectionDB, PIDBTYPE_OPAQUE, 
				KEY_INT_KEEPOPEN, (void *)0, 0 );
			if (	HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
					HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) ) 
				{ iRet = -1; };
			}
		else if ( !iRet )
			{

			/* ++++ __________________________ +++ *
			  Partial Response (aka Parsed Headers)
			 * ++++ __________________________ +++ */

			/* ---
				Make sure that at least 'Content-Type' exists.
			--- */
			if ( !PIDB_lookup( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE, 0 ))
				{
				HTTPCore_logError( &tPIHTTP, "WinCGI: CGI program \
did not provide 'Content-Type' header. Start of command line is '%s'.", szCmdBuf );
				iRet = -1;

				/* ---
				if 'Status' header exists,
				then set the status appropriately
				--- */
				const char *pStatus = (const char *)PIDB_lookup( pR,
					PIDBTYPE_RFC822, KEY_HTTP_STATUS, 0 );
				if ( pStatus )
					{
					tPIHTTP.iStatus = atoi( pStatus );
					};	
				}
			else
				{
				/* --- start the response --- */
				tPIHTTP.iStatus = ST_OK;

				PIDB_replace( tPIHTTP.pConnectionDB, PIDBTYPE_OPAQUE, 
					KEY_INT_KEEPOPEN, (void *)0, 0 );
				if (	HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
						HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) ) 
					{ iRet = -1; };
				};
			};

		if ( !iRet )
			{ iRet = WriteBufferToBuffer( *pCGIBuffer, tB ); };
		if ( pCGIBuffer )
			{ PIIOBuffer_delete( pCGIBuffer ); };


#if VERBOSE_DEBUG
		cerr << dec << iRet << endl;
#endif
		CloseIfValidDesc( tStdout_Parent );

		/* ---
		Swallow up stderr stream from CGI child
		--- */
		CloseFileDesc( tStderr_CGI );
		for(;;)
			{
			int iRead;

			int iRet = PIPlatform_pollPipeFD(
					(PIPLATFORM_FD)tStderr_Parent, PIPLATFORM_POLL_READ, 0 );

			if ( iRet != PIPLATFORM_POLL_READ )
				{
				if (iRet == PIAPI_ERROR)
					{
					iRet = PIPlatform_getLastError();
					if ((iRet != PIAPI_NOTSUPPORTED) && (iRet != PIAPI_EINVAL))
						{ break; };
					}
				else
					{ break; };
				};

//			if ( iRet!=PIPLATFORM_POLL_READ )
//				{ break; };

#if WIN32
			if ( !::ReadFile( tStderr_Parent, szBuf, BUF_SIZE, 
				(unsigned long *)&iRead, NULL ) || !iRead )
				{ break; };
#elif POSIX
			iRead = ::read( tStderr_Parent, szBuf, BUF_SIZE );
			if ( iRead==-1 && errno==EINTR )
				{ continue; };
			if ( iRead==-1 || !iRead )
				{ break; };
#endif
			szBuf[iRead] = '\0';

			/*
			** Write stderr output into the error logfile
			*/
			HTTPCore_logError( &tPIHTTP, "WinCGI: Stderr output from \
CGI program follows \n\
______________________________________________________________________________\
__\n%s\n\
______________________________________________________________________________\
__\nCGI program command line is '%s'.", szBuf, szCmdBuf );
			/* --- mark an error --- */

			iRet = -1;
			};
		CloseIfValidDesc( tStderr_Parent );

		PIObject_delete( pChildIO, 0, 0 );

		if ( iRet )
			{
			tPIHTTP.iStatus = ST_INTERNALERROR;
			return bIgnoreStatus;
			};

		/* --- 
		Done!
		--- */
		return bIgnoreStatus;
		};

	/* ---
	Handle request
	--- */
	int Handle( int /* iPhase */, PIHTTP &tPIHTTP, PIIOBuffer &/* tBuffer */ )
		{
		PIDB *pR = tPIHTTP.pResponseDB;

		/* ---
		Get file
		--- */
		const char *pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,
			pFKPath, PIDBFLAG_FASTKEY );
		PIFInfo *pFile = HTTPCore_getCachedFile( pPath );

		if ( !pFile )
			{
			HTTPCore_releaseCachedFile( pFile );
			HTTPCore_logError( &tPIHTTP, "Could not get file object for \
path '%s'", pPath ? pPath : "" );
			return PIAPI_ERROR;
			};

		/* --- DoExecCGI() --- */
		PROCDESC tProcess = INVALID_DESC; 
		bool bIgnoreStatus = DoExecCGI( pFile, tPIHTTP, tProcess );
		if ( tProcess!=INVALID_DESC )
			{
			/* ---
			Collect the process
			--- */
			if( !WaitForChild( pPath, tPIHTTP, tProcess ) )
				{
				/* --- error message already given --- */
				tPIHTTP.iStatus = ST_INTERNALERROR;
				};
			CloseProcessDesc( tProcess );
			tProcess = INVALID_DESC; 
			};

		HTTPCore_releaseCachedFile( pFile );

		if( bIgnoreStatus )
			{
			return PIAPI_COMPLETED;
			};

		if ( tPIHTTP.iStatus && tPIHTTP.iStatus!=ST_OK )
			{
			return HTTPUtil_doHTTPError( &tPIHTTP, tPIHTTP.iStatus );
			};

		return PIAPI_COMPLETED;
		};
};

/*____________________________________________________________________________*\
 *
 Description:
	Static data
\*____________________________________________________________________________*/
int WinCGI::CGIContextData::iUniqueIndex = 0;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int WinCGI_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	WinCGI *pHand = PI_NEW( WinCGI( pObj, iArgc, ppArgv) );

	HandlerBaseHTTP *pHandBase = (HandlerBaseHTTP *)pHand;
	return HandlerBaseHTTP_constructor(pObj, pHandBase );
}

