/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/IFaceBse.h,v $
 * $Date: 2003/05/13 18:42:06 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#ifndef IFACEBSE_H_
#define IFACEBSE_H_

#include <assert.h>

#include "Pi2API.h"
#include "IntrFace.h"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
#define CONFIG_ERR(this, msg)   \
	{ PILog_addMessage( PIObject_getDB(this), \
	PIObject_getConfigurationDB(this), PILOG_ERROR, "%s", (msg) ); }

#define SHEET ((IFaceBase *)GetWindowLong(((NMHDR *)lParam)->hwndFrom, GWL_USERDATA ))

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class IFaceBase
{
private:
	/* ---
	Configuration data
	--- */
	PIObject *pObject;				/* the object */
	int iOK;						/* status */
	static int iChanged;			/* changed state */
    Interface *pInterface;          /* the interface structure */

protected:
	IFaceBase( const IFaceBase & )	{ assert( 0 ); };

	/* ---
	Derived classes override this to handle parameters
	--- */
	virtual int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )=0;
	inline void SetOK( int iNewValue ) { iOK = iNewValue; };
	static int ParameterFn( void *pData, const char *pVar, const char *pVal,
		const char *pWhere )
		{
		assert( pData );
		return ((IFaceBase *)pData)->Parameter( pVar, pVal,
			pWhere ? pWhere : "" );
		};

public:
	IFaceBase( PIObject *pTheObject );

	virtual ~IFaceBase();

	virtual int Handle( Interface &tInterface )=0;
#if WIN32
	virtual int SaveConfig( Interface *pInterface, HWND hDlg );
	virtual int Cancel(Interface *pWindow, HWND hDlg);
	inline void Changed( HWND __hDlg ) {
		iChanged = 1;
		PropSheet_Changed( GetParent( __hDlg ), __hDlg );
	};

	inline void Unchanged( HWND __hDlg ) {
		iChanged = 0;
		PropSheet_UnChanged( GetParent( __hDlg ), __hDlg );
	};

	inline int IsChanged() { return	iChanged; };

    inline void HourglassOn()
        { if ( pInterface ) { pInterface->HourglassOn(); }; };

    inline void HourglassOff()
        { if ( pInterface ) { pInterface->HourglassOff(); }; };
#endif

	virtual int Execute( int iArgc, const char *ppArgv[] );
	inline int IsOK()				{ return iOK; };
	inline PIObject *Object()		{ return pObject; };
	void ReadParameters( int iArgc, const char *ppArgv[] );
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Function prototypes
\*____________________________________________________________________________*/
PUBLIC_PIAPI int IFaceBase_execute( PIObject *pObj, int iArgc,
		const char *ppArgv[] );
PUBLIC_PIAPI int IFaceBase_onClassLoad( PIClass_LoadAction eLoad, void * );
int IFaceBase_constructor( PIObject *pObj, IFaceBase *pHandler );
PUBLIC_PIAPI int IFaceBase_copyConstructor( PIObject *, int,
    const char *[]);
PUBLIC_PIAPI int IFaceBase_destructor( PIObject *pObj, int,
    const char *[] );

#endif /* IFACEBSE_H_ */

