/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission. 

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/PrpSheet.cpp,v $
 * $Date: 2003/05/13 18:42:07 $
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>
#include <windowsx.h>

#include "IFaceBse.h"
#include "PIStrStr.h"
#include "StrToken.h"
#include "QConfig.h"
#include "Admin.h"
#include "DeQuote.h"
#include "PrpSheet.hrc"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_TITLE					"Title"
#define KEY_CONF_PROPERTYPAGE			"PropertyPage"
#define KEY_CONF_FILENAME				"FileName"
#define KEY_CONF_HOST					"Host"
#define KEY_CONF_PORT					"Port"
#define KEY_CONF_USERNAME				"AdminUser"
#define KEY_CONF_PASSWORD				"AdminPassword"
#define KEY_CONF_OPTIONS				"Options"

/*____________________________________________________________________________*\
 *
 Flags and Options
\*____________________________________________________________________________*/
#define FLG_NONE				0x00000000
#define FLG_CREATEDEFAULT		0x00000001
#define FLG_LOADFILE			0x00000002
	
/*___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ *
		
			Map these flags to names

*___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ */
struct {
	const char *pName;
	int iFlag;
} aFlagMap[] =
	{
	{ "CreateDefault",	FLG_CREATEDEFAULT },
	{ "LoadFile",		FLG_LOADFILE },

	/* --- leave this last always --- */
	{ 0, 0 }
	};

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    PropSheet

Description:
    Generates a property sheet window for remote administration.
	This object is used by the WinLoop object and expects an I/O
	object of name &quot;ClientIOObject&quot; to be in the
	configuration in case of network based remote administration.


Options:

<TABLE BORDER=1>
<TH>Option
<TH>Default
<TH>Values
<TH>Short Description
<TH>Example(s)

<TR>
<TD>Host
<TD>-
<TD>&lt;string&gt;
<TD>A valid hostname for remote network connection
<TD>Host theserver.pi3.net

<TR>
<TD>Port
<TD>80
<TD>&lt;number&gt;
<TD>A valid port number for remote network connection
<TD>Port 443

<TR>
<TD>AdminUser
<TD>-
<TD>&lt;string&gt;
<TD>A valid user name for remote network connection
<TD>AdminUser

<TR>
<TD>AdminPassword
<TD>-
<TD>&lt;string&gt;
<TD>A valid password for remote network connection
<TD>AdminPassword 0xb458378ed3de4d4eed0f32c9571e5193

<TR>
<TD>FileName
<TD>-
<TD>&lt;string&gt;
<TD>A valid filename for file based local interface
<TD>FileName "../Conf/Config.pi3"

<TR>
<TD>Title
<TD>"Pi3Web Admin"
<TD>&lt;string&gt;
<TD>A title string for the property sheet
<TD>Title "Pi3Web Remote Administration"

<TR>
<TD>PropertyPage
<TD>-
<TD>&lt;configuration object&gt;
<TD>A valid name of a configuration object
<TD>PropertyPage SslPage

<TR>
<TD>Options
<TD>CreateDefault | LoadFile
<TD>Options flags as described below
<TD>Options which effect behaviour
<TD>Options CreateDefault

</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>
<H5>Host</H5>
The Host parameter is used to establish a remote interface connection to the
specified host. It must contain a valid hostname or IP address. A corresponding
Port parameter may exist. This parameter is mutual exclusive with the FileName
parameter.

<H5>Port</H5>
The Port parameter is optional used to establish a remote interface connection
using the specified port. It must contain a valid TCP port number.

<H5>AdminUser</H5>
The username is used to establish a remote interface connection. A corresponding
authentication setting (realm, user) must exist at the server. The interface
client determines the authentication type automatically. Supported authentication
types are &quot;Basic&quot; and &quot;Digest&quot; (recommended). If the AdminUser
is set but the AdminPassword is omitted, the Login Dialog is prefilled with the
value of AdminUser (recommended for Basic Authentication).

<H5>AdminPassword</H5>
The password is used to establish a remote interface connection. A corresponding
authentication setting (realm, user) must exist at the server. The interface
client determines the authentication type automatically. Supported authentication
types are &quot;Basic&quot; and &quot;Digest&quot; (recommended). If both AdminUser
and AdminPassword are set, the Login Dialog is omitted (not recommended for Basic
authentication). For Digest authentication the Admin password is the hexified MD5
hash of the username:password combination, which makes it nearly impossible to
compute the plain password from the stored password string.

<H5>FileName</H5>
The FileName is used to load the wanted configuration file locally. This parameter
is mutual exclusive with the Host parameter.

<H5>Title</H5>
The given string is used in the title bar of the PropSheet window.

<H5>PropertyPage</H5>
The PropertyPage directive may be repeated multiple times in order to fill the
PropSheet with the wanted property pages. You may simply comment out all pages,
which are unwanted for remote administration.

<H5>Options</H5>
Specifies options which effect the behaviour of the interface application.
<CENTER>
<TABLE BORDER=1>
<TH>Option
<TH>Meaning
<TR>
<TD>CreateDefault
<TD>Set appropriate default values in all PropertyPage objects.
Makes sense as an alternative to LoadFile option in order to
generate an initial configuration.

<TR>
<TD>LoadFile
<TD>Loads the configuration file dependant on the Host/FileName settings
either remotely or locally. Makes sense as an alternative to CreateDefault
option in order to edit an already existing configuration file.

</TABLE>
</CENTER>
Multiple options may be listed together by seperating them using the pipe
('|') character. Whitespace will be stripped from the option pattern.
Options are matched without regard to case sensitivity. Unknown options
will raise a configuration error. The Option directive may be repeated
multiple times.

Example:
	<PRE>
	&lt;Object&gt;
		Name PropSheet
		Class PropSheetClass
		Host localhost
		Port 80
		Title "Pi3Web Remote Administration"
		Options LoadFile
		PropertyPage GeneralPage
		PropertyPage PerformancePage
		PropertyPage HttpPage
		PropertyPage UsersPage
		PropertyPage MIMETypesPage
		PropertyPage MappingsPage
		PropertyPage LogFilePage
		PropertyPage IPVHostsPage
		PropertyPage NonIPVHostsPage
		PropertyPage DirIndexPage
		PropertyPage SslPage
	&lt;/Object&gt;

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

static BOOL CALLBACK fnAuthDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

static int GetAuthenticationData( void *pUserData )
	{
	HINSTANCE hInstance = (HINSTANCE)PIDLL_getSystemHandle( PIClass_getLibrary(
		PIObject_getClass( ((IFaceBase *)pUserData)->Object() )));

	return DialogBoxParam(
		hInstance,
		MAKEINTRESOURCE(IDD_AUTHENTICATE),
		NULL,
		(DLGPROC)fnAuthDialogProc,
		(LPARAM)pUserData
		);
	}

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class PropSheet : public IFaceBase
{
private:
    Interface *pInterface;
    Admin *pAdmin;
    PIString sTitle;
    DblList lPages;
    PIString sFileName;
    PIString sUserName;
    PIString sPassword;
    PIString sHost;
    QConfig *pConfig;
	int iPort;
    int iOptions;

	HWND hWndOK,
		 hWndRealm,
		 hWndUser,
		 hWndPassword;

protected:

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );

		PIOStrStream os;
		os << pWhere << "PropSheet: ";

        if ( !PIUtil_stricmp( pVariable, KEY_CONF_TITLE ) )
			{
            sTitle = pValue;
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_PROPERTYPAGE ) )
			{
            /*
            ** Attempt to load the object
            */
            PIObject *pPageObject = PIObject_load(
                PIObject_getDB( Object() ), 0, pValue, 0, 0 );

			/* --- error message already set --- */
            if ( !pPageObject ) { return 0; };

            /*
            ** Page created
            */
            lPages.Append( (DblList::type)pPageObject );
			}
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_OPTIONS ) )
			{
			/* ---
			Split the options up by the bar ('|') character and then
			strip trailing and leading whitespace from option before
			attempting to match it.
			--- */
			StringTokenizer tTokens( pValue, "|" );
			for(int i=0; i<tTokens.NumTokens(); i++)
				{
				const char *pToken = tTokens.GetToken( i );

				/* --- skip leading whitespace --- */
				for( ; *pToken && (isspace(*pToken)); pToken++ );

				/* ---
				include only to first whitespace trailing whitespace
				--- */
				int j=0;
				for( ; pToken[j] && !(isspace(pToken[j])); j++ );
				/* ---
				j now contains the length of the first word in pToken
				--- */

				/* ---
				cycle through the list of available flags
				comparing them with this one
				--- */
				int i=0;
				for( ; aFlagMap[i].pName; i++ )
					{
					if ( !PIUtil_strncmpi( aFlagMap[i].pName, pToken, j ) )
						{ break; };
					};
				if ( !aFlagMap[i].pName )
					{
					/* --- flag not found --- */
					PIString sTmp( pToken, j );
					os << "Unknown option flag '" << sTmp << "'." << ends;
					CONFIG_ERR( Object(), os.str() );
					return 0;
					};
				iOptions |= aFlagMap[i].iFlag;
				};	/* --- loop over tokens seperated by '|' --- */
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_FILENAME ) )
			{
			// Configuration error, one resource must be specified, 2 are specified
			if (sFileName.Len())
				{
				os << "Local file not allowed here. "
					<< "A remote host is already specified."
					<< ends;
				CONFIG_ERR( Object(), os.str() );
				return 0;
				}

            sFileName = pValue;
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_HOST ) )
			{
			// Configuration error, one resource must be specified, 2 are specified
			if (sFileName.Len())
				{
				os << "Remote host not allowed here. "
					<< "A local filename is already specified."
					<< ends;
				CONFIG_ERR( Object(), os.str() );
				return 0;
				}

            /* --- host is read below --- */
			sHost = (const char *)DeQuote( pValue );
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_PORT ) )
			{
            iPort = atoi(pValue);
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_USERNAME ) )
			{
            sUserName = pValue;
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_PASSWORD ) )
			{
            sPassword = pValue;
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	PropSheet( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	IFaceBase( pObject ),
        pAdmin( 0 ),
        sTitle( "Pi3Web Admin" ),
        pConfig( CreateQConfig() ),
        iOptions( 0 ),
		iPort( 80 )
		{
        PIDB_add( PIObject_getDB( Object() ), PIDBTYPE_OPAQUE,
            QCONFIG_KEY, pConfig, 0 );

        ReadParameters( iArgc, ppArgv );

		if (sHost.Len()) {
			pAdmin = Admin_newRemote( Object(), sHost, iPort,
				sUserName, sPassword, &GetAuthenticationData, this );
			}

		if (sFileName.Len()) { pAdmin = Admin_newLocal( sFileName ); }
		}

	~PropSheet()
		{
		DestroyQConfig( pConfig );
		Admin_delete( pAdmin );
		for( DblListIterator i( lPages ); !i.BadIndex(); i++ )
			{
			PIObject_delete( (PIObject *)i.Current(), 0, 0 );
			};
		};
/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
    virtual int Handle( Interface &tWindow )
        {
        switch( tWindow.iMessage )
            {
            case INTRFACE_MSG_INIT:
				pInterface = &tWindow;
				return Init();

            case INTRFACE_MSG_WIN32CREATE:
                return Win32Create();

            case INTRFACE_MSG_LOAD:
                return Load();

            case INTRFACE_MSG_DEFAULTS:
                return SetDefaults();

            default:
                assert( 0 );
                return PIAPI_ABORT;
            };

        return PIAPI_COMPLETED;
        };

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
	virtual int SaveConfig(Interface *pWindow, HWND hDlg)
		{
		if (IsChanged())
			{
			HourglassOn();
			int iRet = Admin_saveConfig( pAdmin, pConfig, pWindow->sError );
			HourglassOff();
			if (iRet)
				{
				if ( pWindow->sError.Len() )
					{
					::MessageBox( GetParent(hDlg),
						pWindow->sError, "Save Configuration Error",
						MB_OK | MB_ICONERROR );
					}
				else
					{
					::MessageBox( GetParent(hDlg),
						"Can't upload configuration changes to server",
						"Save Error", MB_OK | MB_ICONERROR );
					}
				SetWindowLong( hDlg, DWL_MSGRESULT, PSNRET_INVALID );
				}
			else
				{
				::MessageBox( GetParent(hDlg),
					"The configuration has been uploaded successfully",
					"Configuration Saved", MB_OK | MB_ICONINFORMATION );
				SetWindowLong( hDlg, DWL_MSGRESULT, PSNRET_NOERROR );
				Unchanged( hDlg );
				}
			}
		return (TRUE);
		}

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
	virtual int Cancel(Interface *pWindow, HWND hDlg)
		{
		if (IsChanged())
			{
			int mbRes = MessageBox( GetParent(hDlg), "Discard changes?",
				"Warning", MB_YESNOCANCEL | MB_ICONWARNING );
			switch (mbRes)
				{
				case IDYES:
					SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
					break;
				case IDNO:
					NMHDR nmhdr;
					memset((void *)&nmhdr, sizeof(nmhdr), 0);
					nmhdr.hwndFrom = hDlg;
					nmhdr.code = PSN_KILLACTIVE;
					/* ---
					Force to write back changes of property sheet page
					before safe configuration
					--- */
					SendMessage( hDlg, WM_NOTIFY, 0, (LPARAM)&nmhdr );
					SaveConfig( pWindow, hDlg );
 					SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
					break;
				case IDCANCEL:
				default:
					SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );
				}
			}
		return (TRUE);
		}

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
   int Init()
		{
        /*
        ** Iterate through the property sheet pages
        */
        int iNumPages = lPages.Size();
        for( DblListIterator i( lPages ); !i.BadIndex(); i++)
            {
            PIObject *pPageObject = (PIObject *)i.Current();
			if ( PILogic_execute( pPageObject, 1, (const char **)&pInterface )
				!= PIAPI_COMPLETED ) 
				{
				pInterface->sError = "Failed to initialize property page '";
				pInterface->sError.Concatenate( PIObject_getName( pPageObject ));
				pInterface->sError.Concatenate( "'." );
				return PIAPI_ERROR;
				}
			}

        /*
        ** Done
        */
        return PIAPI_COMPLETED;
		}

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
   int SetDefaults()
		{
		if ( !(iOptions & FLG_CREATEDEFAULT) ) { return PIAPI_ABORT; };

        /*
        ** Iterate through the property sheet pages
        */
        int iNumPages = lPages.Size();
        for( DblListIterator i( lPages ); !i.BadIndex(); i++)
            {
            PIObject *pPageObject = (PIObject *)i.Current();
			if ( PILogic_execute( pPageObject, 1, (const char **)&pInterface )
				!= PIAPI_COMPLETED ) 
				{
				pInterface->sError = "Failed to setup defaults for property page '";
				pInterface->sError.Concatenate( PIObject_getName( pPageObject ));
				pInterface->sError.Concatenate( "'." );
				return PIAPI_ERROR;
				}
			}

        /*
        ** Done
        */
        return PIAPI_COMPLETED;
		};

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
    int Win32Create()
		{
        /*
        ** Create the property sheet
        */
        int iNumPages = lPages.Size();

        if ( !iNumPages )
            {
            pInterface->sError = "PropSheet: Cannot create property sheet";
			pInterface->sError.Concatenate( " with no property pages." );
            return PIAPI_ERROR;
            }

        HPROPSHEETPAGE *pSheets = PI_NEW( HPROPSHEETPAGE[iNumPages] );
        
        int j=0;
        for( DblListIterator i( lPages ); !i.BadIndex(); i++)
            {
            PIObject *pPageObject = (PIObject *)i.Current();
            PROPSHEETPAGE *pPsp = (PROPSHEETPAGE *)PIDB_lookup(
                PIObject_getDB( pPageObject ), PIDBTYPE_OPAQUE, 
                INTRFACE_PROPSHEETPAGE, 0 );
            HPROPSHEETPAGE hPropSheetPage = ::CreatePropertySheetPage( pPsp );
                    
            if ( !hPropSheetPage )
                {
				pInterface->sError = "PropSheet: Failed to create property page '";
				pInterface->sError.Concatenate( pPsp->pszTemplate );
				pInterface->sError.Concatenate( "'." );
                return PIAPI_ERROR;
                };

            pSheets[j++] = hPropSheetPage;
            };
        
        /*
        ** setup property structure
        */
        PROPSHEETHEADER tPsh;
        memset( &tPsh, 0, sizeof( PROPSHEETHEADER ) );
        tPsh.dwSize = sizeof( PROPSHEETHEADER );
        tPsh.dwFlags = PSH_MODELESS | PSH_DEFAULT | PSH_USEICONID;
        tPsh.hInstance = (HINSTANCE)PIDLL_getSystemHandle( PIClass_getLibrary(
            PIObject_getClass( Object() ) ) );
        tPsh.hwndParent = NULL;
        tPsh.pszIcon = MAKEINTRESOURCE(IDI_PI3ADMIN);
        tPsh.nPages = iNumPages;
        tPsh.phpage = pSheets;
        tPsh.pszCaption = sTitle;
    
        /*
        ** Create modeless property sheet
        */
		HourglassOn();
        pInterface->hWnd = (HWND)PropertySheet( &tPsh );
		HourglassOff();
        PI_DELETE( [] pSheets );

        if ( pInterface->hWnd == (HWND)-1 ) {
            pInterface->sError = "PropSheet: Failed to create property sheet.";
            return PIAPI_ERROR;
        }

		::SetWindowLong( pInterface->hWnd, GWL_USERDATA, (long)this );

        /*
        ** Done
        */
        return PIAPI_COMPLETED;
		};

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
   int Load()
		{
		if ( !(iOptions & FLG_LOADFILE)) { return PIAPI_ABORT; };

        /*
        ** Attempt to load configuration file
        */
		if ( Admin_loadConfig( pAdmin, pConfig, pInterface->sError )) {
			if ( !pInterface->sError.Len() ) {
				pInterface->sError = "PropSheet: Failed to load configuration file.";
			}
			return PIAPI_ERROR;
		}

        /*
        ** Done
        */
        return PIAPI_COMPLETED;
		}

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
	BOOL AuthDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
		{
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
	
		switch (uMsg)
			{
			case WM_INITDIALOG:
                hWndOK = GetDlgItem( hDlg, IDOK );
                hWndRealm = GetDlgItem( hDlg, IDC_REALM );
				hWndUser = GetDlgItem( hDlg, IDC_USER );
				hWndPassword = GetDlgItem( hDlg, IDC_PASSWORD );

				assert( hWndOK && hWndRealm && hWndUser	&& hWndPassword);

				SendMessage( hWndUser, EM_SETLIMITTEXT, 256, 0 );
				SendMessage( hWndPassword, EM_SETLIMITTEXT, 256, 0 );
				EnableWindow( hWndOK, FALSE );
				SendMessage( hWndUser, WM_SETTEXT, 0,
					(LPARAM)Admin_GetUsername( pAdmin ));
				SendMessage( hWndRealm, WM_SETTEXT, 0,
					(LPARAM)Admin_GetRealm( pAdmin ));

			case WM_COMMAND:
				switch (GET_WM_COMMAND_ID(wParam, lParam))
					{
					case IDOK:
						/*
						** Validate content of edit fields
						*/
						*szBuf = '\0';
						SendMessage( hWndUser, WM_GETTEXT, BUF_SIZE,(LPARAM)szBuf );
						Admin_SetUsername( pAdmin, szBuf );
						*szBuf = '\0';
						SendMessage( hWndPassword, WM_GETTEXT, BUF_SIZE,(LPARAM)szBuf );
						Admin_SetPassword( pAdmin, szBuf );
						EndDialog( hDlg, 1 );
						break;
	
					case IDCANCEL:
						EndDialog( hDlg, 0 );
						break;
	
					case IDC_USER:
						if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE )
							{
							if ( SendMessage( GET_WM_COMMAND_HWND(wParam,
                                lParam ), EM_LINELENGTH, 0, 0 )>0 )
								{ EnableWindow( hWndOK, TRUE ); }
							else
								{ EnableWindow( hWndOK, FALSE ); }
							}
						else
							return (FALSE);
						break;
	
					default:
						return (FALSE);
					};
	
			default:
				return (FALSE);
			};
	
		return (TRUE);
		}
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnAuthDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, lParam );
        };

    PropSheet *pSheet = (PropSheet *)::GetWindowLong( hDlg, DWL_USER );
    return pSheet ? pSheet->AuthDialogProc( hDlg, uMsg, wParam, lParam ) :
        FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PropSheet_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return IFaceBase_constructor( pObj, PI_NEW( PropSheet( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name PropSheetClass
		Type LogicExtension
		Library IntrFace
		OnClassLoad IFaceBase_onClassLoad
		Constructor PropSheet_constructor
		CopyConstructor IFaceBase_copyConstructor
		Destructor IFaceBase_destructor
		Execute IFaceBase_execute
	</Class>

	<Object>
		Name PropSheet
		Class PropSheetClass
	</Object>

/*___+++CNF_END+++___*/
#endif

