/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Pi2API/PIProg.h,v $
 * $Date: 2003/05/13 18:42:09 $
 *
 Description:
	Functions which manipulate the program database to do standard things
	like read configuration files, load libraries and classes and set
	standard program values.
\*____________________________________________________________________________*/
/* $HeaderTop:$ */

#ifndef PIPROG_H_
#define PIPROG_H_

#include "Pi2API.h"

/*____________________________________________________________________________*\
 *
 Typedefs:
\*____________________________________________________________________________*/
typedef void (* PIProgramFn)( PIDB *pDB );
#define PIPROGRAM_VERSION_1_0	100

/*____________________________________________________________________________*\
 *
 Name:
	PIProgram_enter

 Synopsis:
	int PIProgram_enter( const char *pProgramName, int iVersion,
		PIProgramFn fn )

 Description:
	Initializes the program database and platform facilities. If successful
	it invokes the user supplied function fn.

 Notes:
	PIProgam_enter should be invoked with the value PIPROGRAM_VERSION_1_0
	for parameter iVersion.

	PIProgram_enter automatically invokes PIPlatform_enter with the
	platform version PIPLATFORM_VERSION_1_0 to initialize multithreading
	and custom memory management facilities.

	The parameter pProgramName should be passed argv[0] from the main()
	entry point to the program. This may be used by certain platform
	implementations.

 Return Values:
	On success PIProgram_enter() returns zero (PIAPI_COMPLETED)

 Errors:
	PIAPI_EINVAL 	if iVersion is not supported or fn is NULL.
	PIAPI_ERROR		if another error occurred.

 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIProgram_enter( const char *pProgramName, int iVersion,
	PIProgramFn fn );

/*____________________________________________________________________________*\
 *
 Name:
	PIProgram_dbgDump

 Synopsis:
	int PIProgram_dbgDump( PIDB *pDB, const char *pDumpFile )

 Description:
	Dump the entire contents of the database referenced by pDB to the
	file pDumpFile. The file is opened in write truncate mode.
	If pDumpFile is NULL PIProgram_dbgDump will attempt to write data
	to the standard output stream.

 Notes:
	PIProgram_dbgDump formats the data in a hierarchical tree
	format.

 Return Values:
	On success PIProgram_dbgDump() returns zero (PIAPI_COMPLETED).

 Errors:
	PIAPI_EINVAL	if pDB is NULL.
	PIAPI_ERROR 	if another error occurred.

 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIProgram_dbgDump( PIDB *pDB, const char *pDumpFile );

#endif	/* PIPROG_H_ */

