/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Pi3API/PIHTTP.cpp,v $
 * $Date: 2003/05/28 06:14:58 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$


#include <assert.h>

#include "PICompat.h"
#include "PIHTTP.h"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIHTTP::PIHTTP( const PIHTTP &tH )	
:	ciPhase( PH_INVALID ),
	iStatus( 0 ),
	pConnectionDB( 0 ),
	pHostDB( 0 ),
	pRequestDB( 0 ),
	pResponseDB( 0 ),
	pBuffer( tH.pBuffer ),
    pUser1( 0 ),
    pUser2( 0 ),
	pParent( 0 ),
	pDB( 0 ),
	iLastResultCode( PIAPI_EINVAL ),
	pTimer( PITimer_new() ),
	pHandlerName( "" ),
	iDelta( 0 ),
	iDepth( 0 )
{
	/* --- disallow copy constructor --- */
	assert( 0 );	
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIHTTP::PIHTTP( PIHTTP *pPIHTTP )
:	ciPhase( PH_INVALID ),
	iStatus( 0 ),
	pConnectionDB( pPIHTTP->pConnectionDB ),
	pHostDB( pPIHTTP->pHostDB ),
	pRequestDB( pPIHTTP->pRequestDB ),
	pResponseDB( pPIHTTP->pResponseDB ),
	pBuffer( pPIHTTP->pBuffer ),
	pParent( pPIHTTP ),
	pDB( pPIHTTP->pDB ),
	iLastResultCode( PIAPI_EINVAL ),
	pTimer( PITimer_new() ),
	pHandlerName( "" ),
	iDelta( 0 ),
	iDepth( 0 )
{
	pResponseDB = PIDB_new( pResponseDB, HTTP_RESPONSE_DB );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIHTTP::PIHTTP( PIObject *pTheIOObject, PIDB *pTheHostDB )
:	ciPhase( PH_INVALID ),
	iStatus( 0 ),
	pConnectionDB( PIObject_getDB( pTheIOObject ) ),
	pHostDB( pTheHostDB ),
	pRequestDB( 0 ),
	pResponseDB( 0 ),
	pBuffer( PI_NEW( PIIOBuffer( pTheIOObject ) ) ),
	pParent( 0 ),
	pDB( 0 ),
	iLastResultCode( PIAPI_EINVAL ),
	pTimer( PITimer_new() ),
	pHandlerName( "" ),
	iDelta( 0 ),
	iDepth( 0 )
{
	pDB = PIDB_new( 0, HTTP_DB );
	pRequestDB = PIDB_new( pDB, HTTP_REQUEST_DB );
	pResponseDB = PIDB_new( pDB, HTTP_RESPONSE_DB );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Special constructor for preevaluation of Pi3Expressions outside
	the context of requests.
\*____________________________________________________________________________*/
PIHTTP::PIHTTP( PIDB *pTheHostDB )
:	ciPhase( PH_INVALID ),
	iStatus( 0 ),
	pConnectionDB( 0 ),
	pHostDB( pTheHostDB ),
	pRequestDB( 0 ),
	pResponseDB( 0 ),
	pBuffer( 0 ),
	pParent( 0 ),
	pDB( 0 ),
	iLastResultCode( PIAPI_EINVAL ),
	pTimer( PITimer_new() ),
	pHandlerName( "" ),
	iDelta( 0 ),
	iDepth( 0 )
{
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIHTTP::ClearRequest( int ClearMemory )
{
	/* ---
	Clear memory
	--- */
	if ( ClearMemory )
		{
		for( DblListIterator i( lMemoryRecords ); !i.BadIndex(); i++ )
			{
			PIUtil_free( (void *)i.Current() );
			};
		}

	if ( pParent )
		{ return;	/* parent will cleanup */ };

	if ( pDB )
		{
#	if !defined(NDEBUG)
		PIProgram_dbgDump( pDB, "./lastdisp.dbg" );
		PIProgram_dbgDump( pConnectionDB, "./lastconn.dbg" );
		if ( pHostDB )
			{
			PIProgram_dbgDump( pHostDB, "./lastserv.dbg" );
			};
#endif
		PIDB_delete( pRequestDB );
		PIDB_delete( pResponseDB );
		PIDB_delete( pDB );
		pDB = 0;
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIHTTP::NewRequest()
{
	/* --- clear out old request data --- */
	ClearRequest( 0 );

	/* --- setup new DB's --- */
	pDB = PIDB_new( 0, HTTP_DB );
	pRequestDB = PIDB_new( pDB, HTTP_REQUEST_DB );
	pResponseDB = PIDB_new( pDB, HTTP_RESPONSE_DB );

	/* --- reset bytes sent to client --- */
	pBuffer->ResetBytesSent();
	
	/* --- reset timer --- */
	PITimer_reset( pTimer );

	/* --- other request stuff --- */
	int *piTmp = (int *)&ciPhase;		/* cast away const */
	*piTmp = PH_INVALID;
	iStatus = 0;
	iLastResultCode = PIAPI_EINVAL;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIHTTP::~PIHTTP()
{
	ClearRequest( 1 );
	if ( !pParent )
		{ PI_DELETE( pBuffer ); };
	PITimer_delete( pTimer );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void *PIHTTP::AllocMem( size_t tSize )
{
	void *pData = PIUtil_malloc( tSize );
	lMemoryRecords.Append( (DblList::type)pData );
	return pData;
}

	/* -------------- +++++++++++++++++++++++++++++++ ------------------ *\
	
						Implement C Functions

	\* -------------- +++++++++++++++++++++++++++++++ ------------------ */

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIHTTP *PIHTTP_new( PIObject *pIOObject, PIDB *pHostDB )
{
	assert( pIOObject );
	assert( pHostDB );
	if ( !pIOObject || !pHostDB )
		{ return 0; };
	return PI_NEW( PIHTTP( pIOObject, pHostDB ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIHTTP *PIHTTP_newChild( PIHTTP *pParent )
{
	assert( pParent );
	if ( !pParent )
		{ return 0; };
	return PI_NEW( PIHTTP( pParent ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIHTTP_delete( PIHTTP *pPIHTTP )
{
	assert( pPIHTTP );
	if ( !pPIHTTP )
		{ return PIAPI_ERROR; };
	PI_DELETE( pPIHTTP );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void *PIHTTP_allocMem( PIHTTP *pPIHTTP, unsigned int uiSize )
{
	
	assert( pPIHTTP );
	if ( !pPIHTTP )
		{ return 0; };
	return pPIHTTP->AllocMem( (size_t)uiSize );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIHTTP_isChild( PIHTTP *pPIHTTP )
{
	assert( pPIHTTP );
	return ( pPIHTTP ) ? pPIHTTP->IsSubRequest() : 0;
};