/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Pi3API/Pi3Str.h,v $
 * $Date: 2003/05/13 18:42:11 $
 *
 Description:
\*____________________________________________________________________________*/
/* $HeaderTop:$ */

#ifndef PI3STR_H_
#define PI3STR_H_

#include "Pi3API.h"

/*____________________________________________________________________________*\
 *
 * For binary compatibility, the type Pi3String is opaque outside of the 
	implementation library.
\*____________________________________________________________________________*/
#if defined(PI3_INTERNAL)

/* Internal */
typedef struct PIString Pi3String;

#else

/* External */
typedef int Pi3String;

#endif

/*____________________________________________________________________________*\
 *
 Name:
	Pi3String_new

 Synopsis:
	Pi3String *Pi3String_new( const char *pString );

 Description:
	This function creates an new Pi3String object, an encapsulation
	for PIString. The value is based on the pString parameter.

 Notes:
	The parameter pString may be NULL to indicate an empty string.

 Return Values:
	A new Pi3String or NULL on error.

 Errors:
 See Also:
	Pi3String_newEx
 	Pi3String_delete
\*____________________________________________________________________________*/
PUBLIC_PIAPI Pi3String *Pi3String_new( const char *pString );

/*____________________________________________________________________________*\
 *
 Name:
	Pi3String_newEx

 Synopsis:
	Pi3String *Pi3String_newEx( const char *pString, int iLength );

 Description:
	This function creates an new Pi3String object, an encapsulation
	for PIString. The value is based on the first iLength characters of
	the pString parameter.

 Notes:
 	The parameter pString may be NULL to indicate an empty string.

 Return Values:
 	A new Pi3String or NULL on error.

 Errors:
 See Also:
 	Pi3String_new
	Pi3String_delete
\*____________________________________________________________________________*/
PUBLIC_PIAPI Pi3String *Pi3String_newEx( const char *pString, int iLength );

/*____________________________________________________________________________*\
 *
 Name:
	Pi3String_delete

 Synopsis:
	int Pi3String_delete( Pi3String *pString );

 Description:
	This deletes the Pi3String in pString.

 Notes:
 Return Values:
	PIAPI_COMPLETED on success or PIAPI_EINVAL on invalid pString.

 Errors:
	PIAPI_EINVAL

 See Also:
	Pi3String_new
 	Pi3String_newEx
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Pi3String_delete( Pi3String *pString );

/*____________________________________________________________________________*\
 *
 Name:
	Pi3String_getPtr

 Synopsis:
	const char *Pi3String_getPtr( Pi3String *pString );

 Description:
	This function returns a read-only pointer of the actual string in pString.

 Notes:
 Return Values:
	A valid pointer to the internal string or NULL on invalid pString.

 Errors:
 See Also:
	Pi3String_getLength
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *Pi3String_getPtr( const Pi3String *pString );

/*____________________________________________________________________________*\
 *
 Name:
	Pi3String_getLength

 Synopsis:
	int Pi3String_getLength( Pi3String *pString );

 Description:
	This function returns the length of the actual string in pString.

 Notes:
 Return Values:
	The length of the string or PIAPI_EINVAL on invalid pString.

 Errors:
	PIAPI_EINVAL

 See Also:
	Pi3String_getPtr
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Pi3String_getLength( const Pi3String *pString );

#endif	/* PI3STR_H_ */

