/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Platform/Win16OSE.h,v $
 * $Date: 2003/05/13 18:42:14 $
 *
 Description:
\*____________________________________________________________________________*/
//$HeaderTop:$

#ifndef WIN16OSE_H_
#define WIN16OSE_H_

#include "OSConfig.h"

#include <windows.h>

#include "PICompat.h"
#include "OSEntity.h"
#include "Allocate.h"
#include "FSType.h"
#include "FSEntity.h"

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class Win16FATFileSysType : public FileSysType
{
public:
	Win16FATFileSysType()		{};
	virtual const char *GetName() const
		{ return "FAT"; };
	virtual const char *GetExtension( const char *pName ) const;
	virtual char DirectorySeperatorCharacter() const
		{ return '\\'; };
	virtual bool IsDirectorySeperatorCharacter( char c ) const
		{ return c=='\\' || c==':' || c=='/'; };
	virtual bool CaseSensitive() const
		{ return false; };
	virtual bool NormaliseFileName( const PIString &sBefore,
        PIString &sAfter ) const
		{ sAfter=sBefore; return true; };
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
    Derived from abstract class DLL, this class encapsulates a Windows DLL.
\*____________________________________________________________________________*/
class Win16DLL : public DLL
{
private:
	HINSTANCE hInstance;
	PIString sPath;
	PIString sLastErrorMsg;
	void CaptureWin16Error( HINSTANCE hInstance );
	
public:
	Win16DLL( const char * );
	~Win16DLL();
	virtual void *GetAddress( const char *pSymbolName );
	virtual const char *GetLastErrorDescription() const
					{ return sLastErrorMsg; };
	virtual bool IsOK() const	{ return hInstance!=0; };
};

#if 0
// no file mapping in win 16
/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class Win16MappedFile : public MappedFile
{
private:
	HFILE hFile;
	HANDLE hMapping;
	LPVOID lpAddress;
	DWORD dwLength;
	void Setup( const char *pFileName, DWORD dwTheLength );
	void *DoLock( int *piLength );
	bool DoUnlock();

public:
	Win16MappedFile( FileSystemEntity &tFSE );
	~Win16MappedFile();
	virtual void *Lock( int *piLength ) { return DoLock( piLength ); };
	virtual bool UnLock()	{ return DoUnlock(); };
	virtual bool IsOK()		{ return hMapping!=0; };
};
#endif

/*____________________________________________________________________________*\
 *
 Class:
 Description:
	Derived from Timer, this abstract class implements a microsecond
	time. Under windows this only gives millisecond resoltion.
\*____________________________________________________________________________*/
class Win16Timer : public Timer
{
private:
	DWORD dwTime;
	
public:
	Win16Timer();
	virtual bool Reset();
	virtual bool Elapsed( long *plSeconds, long *plMicroSeconds );
	virtual bool IsOK()			{ return true; };
};

#endif	/* WIN32OSE_H_ */

