/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/SLL/SLLMisc.h,v $
 * $Date: 2003/05/13 18:42:16 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#ifndef SLLMISC_H_
#define SLLMISC_H_

#include <assert.h>
#include "PIString.h"
#include "DblList.h"
#include "PICompat.h"

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SLLList;
class SLLBase
{
public:
	enum Type { FUNCTION, NUMBER, LIST, STRING };
	virtual Type GetType() const=0;
	virtual void Render( ostream &os ) const=0;

public:
	virtual ~SLLBase()				{};
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SLLList : public SLLBase
{
private:
	DblList tList;

	/* --- forbid copy constructor --- */
	SLLList( const SLLList &  )
		{ assert( 0 ); };

public:
	
	SLLList()
		{};
	virtual ~SLLList()
		{
		for( DblListIterator i( tList ); !i.BadIndex(); i++)
			{ PI_DELETE( (SLLBase *)i.Current() ); };
		};

	/* --- implement virtual functions --- */
	virtual SLLBase::Type GetType() const
		{ return SLLBase::LIST; };
	virtual void Render( ostream &os ) const
		{
		os << "( ";
		for( ConstDblListIterator i( tList ); !i.BadIndex(); i++)
			{
			SLLBase *pB = (SLLBase *)i.Current();
			assert( pB );
			pB->Render( os );
			};
		os << ") ";
		};

	/* --- accessor functions --- */
	inline void Add( SLLBase *pBase )
		{ tList.Append( (DblList::type)pBase ); };
	inline const DblList &GetList() const	{ return tList; };
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SLLFunction : public SLLBase
{
private:
	void *pValue;

public:
	SLLFunction( void *pTheValue )
	:	pValue( pTheValue ) 			{};

	/* --- implement virtual functions --- */
	virtual SLLBase::Type GetType() const
		{ return SLLBase::FUNCTION; };
	virtual void Render( ostream &os ) const
		{ assert( pValue );
		os << "FUNCTION:" << pValue << " "; };

	/* --- accessor functions --- */
	void *GetFunction() const 		{ return pValue; };
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SLLString : public SLLBase
{
private:
	PIString sString;

public:
	SLLString( const char *pTheString )
	:	sString( pTheString ) 		{};

	/* --- implement virtual functions --- */
	virtual SLLBase::Type GetType() const
		{ return SLLBase::STRING; };
	virtual void Render( ostream &os ) const
		{ os << "STRING:\"" << sString << "\" "; };

	/* --- accessor functions --- */
	const PIString &GetString() const		{ return sString; };
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SLLNumber : public SLLBase
{
private:
	int iNum;

public:
	SLLNumber( int iTheNumber )
	:	iNum( iTheNumber ) 		{};

	/* --- implement virtual functions --- */
	virtual SLLBase::Type GetType() const
		{ return SLLBase::NUMBER; };
	virtual void Render( ostream &os ) const
		{ os << "NUM:" << iNum << " "; };

	/* --- implement accessor functions --- */
	int GetNumber() const			{ return iNum; };
};

#endif	/* SLLMISC_H_ */

