#!/usr/bin/perl

#*____________________________________________________________________________*\
#*
#
# Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.
#
# These sources, libraries and applications are
# FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
# as long as the following conditions are adhered to.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission. 
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
# GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
#*____________________________________________________________________________*|
#*
#* $Source: /cvsroot/pi3web/Pi3Web_200/Source/Scripts/genconf.pl,v $
#* $Date: 2003/05/13 18:42:17 $
#*
# Description:
#	Create .cnf files from embedded comments in source files.
#	Create generic HTML documentation from fragements of HTML embedded
#	in header files.
#
# Parameters
#	ARGV[0] - name of CNF file to create. Full path or relative from
#				current directory.
#	ARGV[1..n] - Files (relative from current directory, or full path )
#				to look in
#*____________________________________________________________________________*/

#/*___________________________________________________________________________*\
# Global declarations:
#\*___________________________________________________________________________*/
%Hoh;
$current_file = "";
$entry;

use English;

&do_main( @ARGV );

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub do_main
	{
	local( $index_file ) = shift;
	print "#\n# genconf.pl, generating file '$index_file'\n#\n";
	if ( $^O =~ /MSWin32/ )
		{
		local( @args );
		foreach( @_ ) { push @args, glob; };
		@_=@args;
		};
	foreach( @_ )
		{
		print "genconf.pl: Scanning $_...\n";
		open( INP, '<'.$_ ) || die "Cannot open file $_ $!\n";
		$current_file = $_;
		$entry = 64;
		&process_file();
		close( INP );
		};
	open( OUT, '>>'.$index_file ) ||
		die "Cannot open file $index_file $!\n";
	print OUT '#<!--
#/*___________________________________________________________________________*\
#*
#	*** AUTOMATICALLY GENERATED FILE CONTENT: DO NOT MODIFY ***
#\*___________________________________________________________________________*/
#/* $SourceTop:$ */
#
#-->

';
	# print index
	local( $file );
	# print data

	local( $tmp );
	foreach $tmp ( sort( keys( %Hoh ) ) )
		{
		local( $text ) = $Hoh{$tmp};
		&PrintParagraph( $text );
		};

	print OUT "\n";
	close( OUT );
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#	Recognise sections of HTML by moving the scanner through the
#	following states.
#\*___________________________________________________________________________*/
sub process_file
	{
	local( $state, $text );
	$state = 1;
	while( <INP> )
		{
		if ( $state == 1 )
			{
			if ( /\/\*___\+\+\+CNF_BEGIN\+\+\+___\*\/.*/ )
				{ $state = 2; $text = ""; next; };
			}
		elsif ( $state == 2 )
			{
			if ( /\/\*___\+\+\+CNF_END\+\+\+___\*\/.*/ )
				{
				&emit( $text );
				$text="";

				$state = 1;
				next;
				};
			};
		$text = $text.$_;
		};
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub emit
	{
	local( $text ) = @_;

	$Hoh{$current_file.": ".pack("c", $entry)} = $text;
	$entry = $entry+1;
	}

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub PrintParagraph
	{
	( $text ) = @_;

	print OUT $text;
	}

