#!/usr/bin/perl

#*____________________________________________________________________________*\
#*
#
# Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.
#
# These sources, libraries and applications are
# FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
# as long as the following conditions are adhered to.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission. 
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
# GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
#*____________________________________________________________________________*|
#*
#* $Source: /cvsroot/pi3web/Pi3Web_200/Source/Scripts/genmake.pl,v $
#* $Date: 2003/05/13 18:42:17 $
#*
# Description:
#	Generate makefiles based on templates
#*____________________________________________________________________________*/

#/*___________________________________________________________________________*\
# Global declarations:
#\*___________________________________________________________________________*/
%Names;
$name_index = 0;
@subdirs;
@all_targets;
@clean_targets;
@genmake_targets;
$line = 0;
$libpath;
$ifstate = 0;
$condition = 1;

$config_ld;
$config_ld_shared;
$config_archive;
$config_objextn;
$config_fulllib;
$config_linklib;
$config_libpath;
$config_cmdseperator;

$_=$0; s/genmake\.pl//i; $scriptdir=$_;
$basedir = $scriptdir.'..';
&toplevel( "./Config.in", "./Makefile", $scriptdir );
exit 0;

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub toplevel
	{
	local( $infile, $outfile, $scriptdir ) = @_;
	local( $tmp );
	open( INP, $infile ) || die "Can't open file $infile: $!\n";
	$_ = $ENV{"PWD"} || `cd` || `pwd`;
	s/\n//;
	$current_directory = $_;
	print "\ngenmake.pl: In $current_directory";
	print "\ngenmake.pl: Generating makefile '$outfile' using '$infile'\n\n";
	open( OUT, '>'.$outfile ) || die "Can't open file $outfile: $!\n";
	print OUT '#*____________________________________________________________________________*\
#*
#	*** AUTOMATICALLY GENERATED FILE: DO NOT MODIFY ***
#*____________________________________________________________________________*/

#
# Read global configuration
';
#	open( CONFIG_MAK, "<$scriptdir".'/config.mak' ) || die
	open( CONFIG_MAK, "<$basedir".'/config.mak' ) || die
		"Could not open file $basedir".'/config.mak'."\n";
	print OUT "# This makefile was generated from '$basedir/config.mak'\n";
	while( <CONFIG_MAK> )
		{
		#
		# echo out the contents of config.mak into the makefile 
		print OUT $_;
		};

	print OUT $tmp;

	#
	# Read some values from config.mak
	#
	open( CFG, '<'.$basedir.'/config.mak' )
		|| die "Cannot open $basedir.'/config.mak': $!\n";

	while( <CFG> )
		{
		if ( /.*CONFIG_OBJEXTN=.*/ )
			{ s/.*CONFIG_OBJEXTN=(.*)\n$/$1/; $config_objextn=$_; }
		elsif ( /.*CONFIG_ARCHIVE=.*/ )
			{ s/.*CONFIG_ARCHIVE=(.*)\n$/$1/; $config_archive=$_; }
		elsif ( /.*CONFIG_LIBPATH=.*/ )
			{ s/.*CONFIG_LIBPATH=(.*)\n$/$1/; $config_libpath=$_; }
		elsif ( /.*CONFIG_FULLLIB=.*/ )
			{ s/.*CONFIG_FULLLIB=(.*)\n$/$1/; $config_fulllib=$_; }
		elsif ( /.*CONFIG_LINKLIB=.*/ )
			{ s/.*CONFIG_LINKLIB=(.*)\n$/$1/; $config_linklib=$_; }
		elsif ( /.*CONFIG_LD=.*/ )
			{ s/.*CONFIG_LD=(.*)\n$/$1/; $config_ld=$_; }
		elsif ( /.*CONFIG_LD_SHARED=.*/ )
			{ s/.*CONFIG_LD_SHARED=(.*)\n$/$1/; $config_ld_shared=$_; }
		elsif ( /.*CMD_SEPERATOR=.*/ )
			{ s/.*CMD_SEPERATOR=(.*)\n$/$1/; $config_cmdseperator=$_; }
		};
	close( CFG );

	print OUT '

CC=$(CONFIG_CC)
CXX=$(CONFIG_CXX)

.SUFFIXES: ';
	print OUT $config_objextn.' .cpp .cc

CFLAGS=$(CONFIG_CCOPTIMIZE) $(CONFIG_THREADED) $(CONFIG_CCNDEBUG) $(CONFIG_CCDEBUG) $(CONFIG_OS)
CXXFLAGS=$(CONFIG_CXXOPTIMIZE) $(CONFIG_THREADED) $(CONFIG_CXXNDEBUG) $(CONFIG_CXXDEBUG) $(CONFIG_OS)

default: all

';
	while(<INP>)
		{
		$_ = &processline( $_ );
		if ( /^#if\s*(.*)/ )
			{
			(!$ifstate) || die
				"Nested '#if' statements are not supported!\n";
			$condition = eval $1;
			$ifstate = 1;
			}
		elsif ( /^#elsif\s*$/ )
			{
			($ifstate eq 1) || die
				"Syntax error: '#elsif' without '#if' statement!\n";
			$ifstate = 2;
			$condition = !$condition;
			}
		elsif ( /^#endif\s*$/ )
			{
			($ifstate) || die
				"Syntax error: '#endif' without '#if' or '#elsif' statement!\n";
			$ifstate = 0;
			$condition = 1;
			}
		elsif ( /^(\s)*#/ )
			{ next; }
		elsif ( /\<subdirs\>*/i )
			{ $condition || next; &subdirectories( INP, 'all' ); }
		elsif( /\<target\>/i )
			{ $condition || next; &target( INP ); }
		elsif( /\<global\>/i )
			{ $condition || next; &global( INP );}
		next;
		};

	print OUT "\nall: @all_targets\n";

	print OUT "\ngenmake: @genmake_targets\n";

	print OUT "\nclean: @clean_targets\n";
print OUT <<END;
	\$(CMD_RM) *.dbg
	\$(CMD_RM) *.o
	\$(CMD_RM) *.obj
	\$(CMD_RM) *.exe
	\$(CMD_RM) *.so
	\$(CMD_RM) *.dll
	\$(CMD_RM) *.a
	\$(CMD_RM) *.pch 
	\$(CMD_RM) *.lib
	\$(CMD_RM) *.bak
	\$(CMD_RM) make.out
	\$(CMD_RM) core
	\$(CMD_RM) out.lst
	\$(CMD_RM) *.tmp* 
	\$(CMD_RM) *.pdb
	\$(CMD_RM) *.pch
	\$(CMD_RM) *.plg
	\$(CMD_RM) vc40.*
	\$(CMD_RM) *.idb
	\$(CMD_RM) *.ilk
	\$(CMD_RM) *.exp
	\$(CMD_RM) *.def 
	\$(CMD_RM) so_locations
	\$(CMD_RM) *.map
	\$(CMD_RM) *.res
	\$(CMD_RM) *.dws
	\$(CMD_RM) *.obr 
END
#	print OUT '$(CMD_RM) *.dbg *.o *.obj *.exe *.so *.dll *.a *.pch '."\n\t"; 
#	print OUT '$(CMD_RM) *.lib *.bak make.out core out.lst *.tmp* '."\n\t";
#	print OUT '$(CMD_RM) *.pdb *.pch vc40.* *.idb *.ilk *.exp *.def '."\n\t";
#	print OUT '$(CMD_RM) so_locations *.map *.res *.dws *.obr '."\n";
	print OUT "\n";

	print OUT "\ndist_clean: clean\n";
	print OUT "\t";
	print OUT '$(CMD_RM) Makefile'."\n\t"; 
	print OUT "\n";

#2unix:';
#	$tmp ="\n\t";
#	$tmp .= `cat $scriptdir/sed.txt` unless $^O =~ /MSWin32/;
#	print OUT $tmp;

	print OUT "\n".'.cpp'.$config_objextn.':
	$(CXX) -c $(CXXFLAGS) $(LOCAL_CXXFLAGS) $(PIINCLUDE) $<

';
	print OUT "\n".'.c'.$config_objextn.':
	$(CC) -c $(CFLAGS) $(LOCAL_CFLAGS) $(PIINCLUDE) $<

';
	print OUT "\n".'.cc'.$config_objextn.':
	$(CXX) -c $(CXXFLAGS) $(LOCAL_CXXFLAGS) $(PIINCLUDE) $<

';

	close( INP );
	close( OUT );
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub subdirectories
	{
	( $filehandle, $target ) = @_;
	local( $subdir_target, $build, @local_subdirs );
	$build='$(MAKE)';
	$subdir_target = $target;
	while( <$filehandle> )
		{
		$_ = &processline( $_ );
		if ( /^#if\s*(.*)/ )
			{
			(!$ifstate) || die
				"Nested '#if' statements are not supported!\n";
			$condition = eval $1;
			$ifstate = 1;
			}
		elsif ( /^#elsif\s*$/ )
			{
			($ifstate eq 1) || die
				"Syntax error: '#elsif' without '#if' statement!\n";
			$ifstate = 2;
			$condition = !$condition;
			}
		elsif ( /^#endif\s*$/ )
			{
			($ifstate) || die
				"Syntax error: '#endif' without '#if' or '#elsif' statement!\n";
			$ifstate = 0;
			$condition = 1;
			}
		elsif ( /^(\s)*#/ )
			{ next; }
		elsif ( /\<\/subdirs\>*/i )
			{
			$condition || next; 
			local( $tmp );
			foreach $tmp ( @local_subdirs )
				{ print OUT "\tcd $tmp $config_cmdseperator $build $subdir_target $config_cmdseperator cd ..\n"; };
			push( @subdirs, @local_subdirs );
			return;
			}
		elsif ( /^build*=*/i )
			{ $condition || next; s/^build*=//i; $build = $_; }
		elsif ( /^target*=*/i )
			{ $condition || next; s/^target*=//i; $subdir_target = $_; }
		elsif ( /\<\/.*/ )
			{ print OUT "Expecting </subdirs> at line $line\n"; }
		elsif ( /^subdirs\s*=/i )
			{
			$condition || next;
			s/^subdirs\s*=//i;
			s/\s\s/ /g;
			push( @local_subdirs, split( /\s/, $_ ) );
			}
		else
			{ die "Unknown attribute $_\n"; };
		next;
		};	
	print OUT "\n";
	return;
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub target
	{
	( $filehandle ) = @_;
	@subdirs = ();
	local( $name, $begin_build, $end_build, $objs, $depends, @objlist,
		@libraries, $tmp );
	while( <$filehandle> )
		{
		$_ = &processline( $_ );
		if ( /^#if\s*(.*)/ )
			{
			(!$ifstate) || die
				"Nested '#if' statements are not supported!\n";
			$condition = eval $1;
			$ifstate = 1;
			}
		elsif ( /^#elsif\s*$/ )
			{
			($ifstate eq 1) || die
				"Syntax error: '#elsif' without '#if' statement!\n";
			$ifstate = 2;
			$condition = !$condition;
			}
		elsif ( /^#endif\s*$/ )
			{
			($ifstate) || die
				"Syntax error: '#endif' without '#if' or '#elsif' statement!\n";
			$ifstate = 0;
			$condition = 1;
			}
		elsif ( /^(\s)*#/ )
			{ next; }
		elsif ( /\<subdirs\>*/i )
			{
			$condition || next;
			print OUT &canonical($name).'_'."SUBDIRS:\n";
			&subdirectories( INP, $name ); 
			}
		elsif ( /\<\/target\>*/i )
			{ $condition || next; last; }
		elsif ( /\<\/.*/ )
			{ die "Expecting </target> at line $line\n"; }
		elsif ( /^name*=*/i )
			{
			$condition || next;
			s/^name*=//i; $name = $_;
			print OUT "#\n# Target: \'$_\'\n#\n";
#			print OUT &canonical($name).'_'."SUBDIRS:\n";
			}
		elsif ( /^build.*=.*/i )
			{
			$condition || next;
			s/^build.*=//i;
			if ($begin_build ne "")
			    {
			    $begin_build .= "\n\t";
			    }
			$begin_build .= $_;
			}
		elsif ( /^executable[\w|\s]*=.*/i )
			{
			$condition || next;
			s/^executable[\w|\s]*=//i;
			$tmp=&mergepattern( $config_ld, $_ );
			$begin_build=$tmp.' '.$libpath.' ';
			$end_build = '$(CONFIG_EXTRALIBS)';
			}
		elsif ( /^source*=*/i )
			{
			$condition || next;
			s/^source*=//i;
			local( @tmp, $src_suffix, $obj_suffix, @filepatterns, $tmp2 );
			@tmp = split( /\s/, $_ );
			$src_suffix = @tmp[0]; 
			@filepatterns = splice( @tmp, 1, $#tmp );
			$tmp2 = &dosources( $name, $src_suffix, $config_objextn,
				@filepatterns );
			push( @objlist, ( "\$\(".$tmp2."\)" ) );
			}
		elsif ( /^objs*=*/i )
			{ $condition || next; s/^objs*=//i; $objs = $_; }
		elsif ( /^depends*=*/i )
			{ $condition || next; s/^depends*=//i; $depends = $_; }
		elsif ( /^libraries*=*/i )
			{ $condition || next; s/^libraries\s*=//i;
			push( @libraries, split( /\s/, $_ ) ); }
		elsif ( /^staticlibdeps.*=.*$/i )
			{
			$condition || next;
			s/^staticlibdeps\s*=//i;
			foreach $tmp ( split( /\s/, $_ ) )
				{
				push( @libraries, ( $tmp ) );
				};
			}
		elsif ( /^staticlib[\w|\s]*=.*/i )
			{
			$condition || next;
			s/^staticlib[\w|\s]*=//i;
			$tmp=&mergepattern( $config_fulllib, $_ );
			$_=$config_archive; s/^(.*)__substme(.*)$/$1/;
			$begin_build = $_.$tmp;
			}
		elsif ( /^sharedlib[\w|\s]*=.*/i )
			{
			$condition || next;
			s/^sharedlib[\w|\s]*=//i;
			$begin_build = &mergepattern( $config_ld_shared, $_ );
			$begin_build=$begin_build.' '.$libpath.' ';
			$end_build = '$(CONFIG_EXTRALIBS)';
			}
		elsif ( /^AddToAll/i )
			{ $condition || next; push( @all_targets, ( $name ) ); }
		elsif ( /^AddToClean/i )
			{ $condition || next; push( @clean_targets, ( $name ) ); }
		elsif ( /^AddToGenmake/i )
			{ $condition || next; push( @genmake_targets, ( $name ) ); }
		else    
			{ print OUT "$_\n"; };
#			{ die "Unknown target attribute $_\n"; };
		next;
		};	

	print OUT "$name: $depends $objs @objlist ";
	if ( ($#subdirs) >= 0 )
		{ print OUT &canonical($name)."_SUBDIRS"; };
	print OUT "\n";
	print OUT "\t$begin_build $objs @objlist ";
	foreach $tmp ( @libraries )
		{ print OUT &mergepattern($config_linklib, $tmp).' '; };
	print OUT " $end_build\n";

	return;
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub dosources
	{
	( $name, $src_suffix, $obj_suffix, @filepatterns ) = @_;
	local( $filepattern, $target, @sources, $src, $obj );

	foreach $filepattern ( @filepatterns )
		{
		push( @sources, <${filepattern}> );
		};

	$target = &canonical( $name );
	$_ = $src_suffix; tr/a-z/_/cs;
	$src = $target."_".$_;
	$_ = $obj_suffix; tr/a-z/_/cs;
	$obj = $src."_OBJS";
	print OUT $src."=";
	foreach $file ( @sources )
		{ print OUT "$file "; };

	print OUT "\n".$obj."=\$(".$src.":$src_suffix=$obj_suffix)\n";
	return $obj;
	}

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub verbose_canonical
	{
	( $_ ) = @_;
	tr/a-zA-Z0-9/\_/c;
	tr/a-z/A-Z/;
	return $_;
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#	The canonical name generated by canonical is really big and ugly, makefiles
#	are almost impossible to read. Instead the a unique canonical name
#	like "TARG1", "TARG2" will be created and associated with the
#	canonical name via a hash table.
#\*___________________________________________________________________________*/
sub canonical
	{
	( $_ ) = @_;
	local( $can ) = &verbose_canonical( $_ );
	if ( length( $Names{$can} )==0 )
		{
		# new name
		$Names{$can} = "TARG".$name_index;
		$name_index++;
		};
	return $Names{$can};
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub global
	{
	( $filehandle ) = @_;
	local( @includes, @local_cflags, @local_cxxflags, $ifstate );
	while( <$filehandle> )
		{
		$_ = &processline( $_ );
		if ( /^#if\s*(.*)/ )
			{
			(!$ifstate) || die
				"Nested '#if' statements are not supported!\n";
			$condition = eval $1;
			$ifstate = 1;
			}
		elsif ( /^#elsif\s*$/ )
			{
			($ifstate eq 1) || die
				"Syntax error: '#elsif' without '#if' statement!\n";
			$ifstate = 2;
			$condition = !$condition;
			}
		elsif ( /^#endif\s*$/ )
			{
			($ifstate) || die
				"Syntax error: '#endif' without '#if' or '#elsif' statement!\n";
			$ifstate = 0;
			$condition = 1;
			}
		elsif ( /^(\s)*#/ )
			{ next; }
		elsif ( /\<\/global\>*/i )
			{ $condition || next; last; }
		elsif ( /\<\/.*/ )
			{ die "Expecting </global> at line $line\n"; }
		elsif ( /^libpath.*=.*/i )
			{
			$condition || next;
			$libpath='';
			s/^libpath*=//i; 
			local( $tmp, $begin, $end, @thislines );
			@thisline=split( /\s/, $_ );
			$_=$config_libpath; s/^(.*)__substme(.*)$/$1/; $begin=$_;
			$_=$config_libpath; s/^(.*)__substme(.*)$/$2/; $end=$_;
			foreach $tmp ( @thisline )
				{ $libpath=$libpath.$begin.$tmp.$end.' '; };
			}
		elsif ( /^includes*=*/i )
			{ $condition || next; s/^includes*=//i; @includes = split( /\s/, $_ ); }
		elsif ( /^local_cxxflags*=*/i )
			{ $condition || next; s/^local_cxxflags*=//i; @local_cxxflags = split( /\s/, $_ ); }
		elsif ( /^local_cflags*=*/i )
			{ $condition || next; s/^local_cflags*=//i; @local_cflags = split( /\s/, $_ ); }
		elsif ( /\<subdirs\>*/i )
			{ $condition || next; &subdirectories( INP, '' ); }
		else
			{
			$condition || next; print OUT "$_\n";
			};
		next;
		};	

	print OUT "PIINCLUDE=@includes\n";
	print OUT "LOCAL_CFLAGS=@local_cflags\n";
	print OUT "LOCAL_CXXFLAGS=@local_cxxflags\n";
	print OUT "\n";
	return;
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub dosubdir
	{
	( $subdir ) = @_;
	print OUT 'Subdirectory '.$subdir."\n";
	`cd $subdir; perl $scriptdir/genmake.pl; cd ..`
		|| die 'Genmake failed for $subdir\n'; 
	return;
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub mergepattern
	{
	( $pattern, $value ) = @_;
	$_=$pattern; s/(.*)__substme(.*)/$1/;
	$tmp=$_;
	$_=$pattern; s/(.*)__substme(.*)/$2/;
	$_=$tmp.$value.$_;
	if ( /(.*)__substme(.*)/ )
		{ return &mergepattern( $_, $value ); }
	else
		{ return $_; }
	}
	
#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub processline
	{
	( $newline ) = @_;
	$_ = $newline;
	$line++;
	s/^[\s*]//g;	# remove leading whitespace
	s/[\s*]$//g;	# remove trailing whitespace
	return $_;
	};

