/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission. 

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Servlet/Pi3ServletContext.cpp,v $
 * $Date: 2003/05/13 18:42:19 $
 *
 Description:
	Execute an servlet extension.

\*____________________________________________________________________________*/
//$SourceTop:$

#include <jni.h>                                    /* where everything is defined */
#include "org_pi3_servlet_core_Pi3ServletContext.h" /* automatically created JNI header */
#include "PiAPI.h"
#include "Pi3API.h"
#include "ServletContainer.h"


/*
 * Class:     Pi3ServletContext
 * Method:    getMimeType
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_pi3_servlet_core_Pi3ServletContext_getMimeType
  (JNIEnv *env, jobject context, jstring file)
{
	const char *buf;
	jboolean isCopy;
	const char *filename = env->GetStringUTFChars(file, &isCopy);

	if (filename) {
		if (strrchr(filename, '.')) {
			buf = HTTPCore_getMIMETypeFromExtension( strrchr(filename, '.') + 1);
		} else {
			buf = HTTPCore_getMIMETypeFromExtension( filename );
		};
	} else {
		// this should retrieve the default mime type
		buf = HTTPCore_getMIMETypeFromExtension( "" );
	};

	env->ReleaseStringUTFChars(file, filename);
	return buf ? env->NewStringUTF( buf ) : env->NewStringUTF("");
};


/*
 * Class:     Pi3ServletContext
 * Method:    getServerInfo
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_pi3_servlet_core_Pi3ServletContext_getServerInfo
  (JNIEnv *env, jobject context)
{
	return env->NewStringUTF(HTTPCore_getServerStamp());
};

/*
 * Class:     Pi3ServletContext
 * Method:    log
 * Signature: (Ljava/lang/Exception;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_pi3_servlet_core_Pi3ServletContext_log__Ljava_lang_Exception_2Ljava_lang_String_2
  (JNIEnv *env, jobject context, jthrowable, jstring msg)
{
	jclass cls = env->GetObjectClass(context);
	jmethodID getContainer;
	getContainer = env->GetMethodID( cls, "getContainer", "()I");
	// Errorhandling!
	if (!getContainer) { return; };
	env->ExceptionClear();

	int hContainer = env->CallIntMethod(context, getContainer);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return;
		};

	jboolean isCopy;
	const char *msgbuf = env->GetStringUTFChars(msg, &isCopy);
	ServletContainer *sc = (ServletContainer *)hContainer;
	sc->Log(msgbuf);
	env->ReleaseStringUTFChars(msg, msgbuf);
};

/*
 * Class:     Pi3ServletContext
 * Method:    log
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_pi3_servlet_core_Pi3ServletContext_log__Ljava_lang_String_2
  (JNIEnv *env, jobject context, jstring msg)
{
	jclass cls = env->GetObjectClass(context);
	jmethodID getContainer;
	getContainer = env->GetMethodID( cls, "getContainer", "()I");
	// Errorhandling!
	if (!getContainer) { return; };
	env->ExceptionClear();

	int hContainer = env->CallIntMethod(context, getContainer);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return;
		};

	jboolean isCopy;
	const char *msgbuf = env->GetStringUTFChars(msg, &isCopy);
	ServletContainer *sc = (ServletContainer *)hContainer;
	sc->Log(msgbuf);
	env->ReleaseStringUTFChars(msg, msgbuf);
};

/*
 * Class:     Pi3ServletContext
 * Method:    log
 * Signature: (Ljava/lang/String;Ljava/lang/Throwable;)V
 */
JNIEXPORT void JNICALL Java_org_pi3_servlet_core_Pi3ServletContext_log__Ljava_lang_String_2Ljava_lang_Throwable_2
  (JNIEnv *env, jobject context, jstring msg, jthrowable)
{
	jclass cls = env->GetObjectClass(context);
	jmethodID getContainer;
	getContainer = env->GetMethodID( cls, "getContainer", "()I");
	// Errorhandling!
	if (!getContainer) { return; };
	env->ExceptionClear();

	int hContainer = env->CallIntMethod(context, getContainer);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return;
		};

	jboolean isCopy;
	const char *msgbuf = env->GetStringUTFChars(msg, &isCopy);
	ServletContainer *sc = (ServletContainer *)hContainer;
	sc->Log(msgbuf);
	env->ReleaseStringUTFChars(msg, msgbuf);
};

