/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission. 

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Servlet/Pi3ServletUtils.cpp,v $
 * $Date: 2003/05/13 18:42:19 $
 *
 Description:
	Execute an servlet extension.

\*____________________________________________________________________________*/
//$SourceTop:$

#include <jni.h>                                  /* where everything is defined */
#include "org_pi3_servlet_util_Pi3ServletUtils.h" /* automatically created JNI header */
#include "PIString.h"
#include "PiAPI.h"
#include "Pi3API.h"
#include "ServletHandler.h"
/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    addCookie
 * Signature: (Ljavax/servlet/http/Cookie;)V
 */
JNIEXPORT void JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_addCookie
(JNIEnv *env, jobject sutils, jobject cookie) {

	jclass cls = env->GetObjectClass(sutils);
	jmethodID getPiHttp;
	getPiHttp = env->GetMethodID( cls, "getPiHttp", "()I");
	// Errorhandling!
	if (!getPiHttp) { return; };
	env->ExceptionClear();

	int PiHttp = env->CallIntMethod(sutils, getPiHttp);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return;
		};

	PIHTTP *pPIHTTP = (PIHTTP *)PiHttp;
	jboolean isCopy;
	const char *fmtCookie = "%s=\"%s\"";
	const char *fmtStrParam = "; %s=\"%s\"";
	const char *fmtIntParam = "; %s=\"%d\"";

	int lenCookie = 1;
	jclass CookieCls = env->GetObjectClass(cookie);
	jstring param;
	const char *szName = NULL;
	const char *szValue = NULL;
	const char *szComment = NULL;
	const char *szDomain = NULL;
	PIString sExpires;

	jint maxAge = 0;
	const char *szPath = NULL;
	jboolean secure = false;
	int version = 0;

	if (CookieCls) {
		
		/* get the cookie methods */
		jmethodID getNameMID = env->GetMethodID( CookieCls, "getName", "()Ljava/lang/String;");
		if (!getNameMID) { return; };
		param = (jstring)env->CallObjectMethod(cookie, getNameMID);
		szName = param ? env->GetStringUTFChars(param, &isCopy) : NULL;

		jmethodID getValueMID = env->GetMethodID( CookieCls, "getValue", "()Ljava/lang/String;");
		if (!getValueMID) { return; };
		param = (jstring)env->CallObjectMethod(cookie, getValueMID);
		szValue = param ? env->GetStringUTFChars(param, &isCopy) : NULL;
		lenCookie += strlen(fmtCookie) + strlen(szName) + strlen(szValue);

		jmethodID getCommentMID = env->GetMethodID( CookieCls, "getComment", "()Ljava/lang/String;");
		if (!getCommentMID) { return; };
		param = (jstring)env->CallObjectMethod(cookie, getCommentMID);
		szComment = param ? env->GetStringUTFChars(param, &isCopy) : NULL;
		if (szComment) {
			lenCookie += strlen(fmtStrParam) + strlen("Comment") + strlen(szComment);
		};

		jmethodID getDomainMID = env->GetMethodID( CookieCls, "getDomain", "()Ljava/lang/String;");
		if (!getDomainMID) { return; };
		param = (jstring)env->CallObjectMethod(cookie, getDomainMID);
		szDomain = param ? env->GetStringUTFChars(param, &isCopy) : NULL;
		if (szDomain) {
			lenCookie += strlen(fmtStrParam) + strlen("Domain") + strlen(szDomain);
		};

		jmethodID getMaxAgeMID = env->GetMethodID( CookieCls, "getMaxAge", "()I");
		if (!getMaxAgeMID) { return; };
		maxAge = env->CallIntMethod(cookie, getMaxAgeMID);

		if (maxAge) {
			time_t tT;
			PIPlatform_beforeUnsafeBlock();
			time( &tT );
			tT += maxAge;
			struct tm *pTms = gmtime( &tT );
			HTTPUtil_rFC822Time( pTms, (int *)&sExpires );
			PIPlatform_afterUnsafeBlock();
			lenCookie += strlen(sExpires) + strlen("Max-Age");
		};

		jmethodID getPathMID = env->GetMethodID( CookieCls, "getPath", "()Ljava/lang/String;");
		if (!getPathMID) { return; };
		param = (jstring)env->CallObjectMethod(cookie, getPathMID);
		szPath = param ? env->GetStringUTFChars(param, &isCopy) : NULL;
		if (szPath) {
			lenCookie += strlen(fmtStrParam) + strlen("Path") + strlen(szPath);
		};

		jmethodID getSecureMID = env->GetMethodID( CookieCls, "getSecure", "()Z");
		if (!getSecureMID) { return; };
		secure = env->CallBooleanMethod(cookie, getSecureMID);
		if (secure) {
			lenCookie += strlen("; Secure");
		};

		jmethodID getVersionMID = env->GetMethodID( CookieCls, "getVersion", "()I");
		if (!getVersionMID) { return; };
		version = env->CallIntMethod(cookie, getVersionMID);
		if (version) {
			lenCookie += strlen(fmtIntParam) + strlen("Version") + 10;
		};

	} else {
		return;
	};

	/* --- cookie buffer --- */
	char *szBuf = (char *)PIUtil_malloc(lenCookie);
	sprintf( szBuf, fmtCookie, szName, szValue );
	sprintf( szBuf + strlen(szBuf), fmtIntParam, "Version", version );
	if (szComment) { sprintf( szBuf + strlen(szBuf), fmtStrParam, "Comment", szComment ); };
	if (szPath) { sprintf( szBuf + strlen(szBuf), fmtStrParam, "Path", szPath ); };
	if (szDomain) { sprintf( szBuf + strlen(szBuf), fmtStrParam, "Domain", szDomain ); };
	if (maxAge) { sprintf( szBuf + strlen(szBuf), fmtStrParam, "Expires", (const char *)sExpires ); };
	if (secure) { sprintf( szBuf + strlen(szBuf), "; %s", "Secure" ); };

	PIDB_add( pPIHTTP->pResponseDB, PIDBTYPE_RFC822, KEY_HTTP_SETCOOKIE, (void *)szBuf, 0 );
	PIUtil_free(szBuf);
};

/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    addResponseVariable
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_addResponseVariable
(JNIEnv *env, jobject sutils, jstring name, jstring value) {

	jclass cls = env->GetObjectClass(sutils);
	jmethodID getPiHttp;
	getPiHttp = env->GetMethodID( cls, "getPiHttp", "()I");
	// Errorhandling!
	if (!getPiHttp) { return; };
	env->ExceptionClear();

	int PiHttp = env->CallIntMethod(sutils, getPiHttp);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return;
		};

	jboolean isCopy;
	const char *pVariable = env->GetStringUTFChars(name, &isCopy);
	const char *pValue = env->GetStringUTFChars(value, &isCopy);
	PIHTTP *pPIHTTP = (PIHTTP *)PiHttp;

	PIDB_add( pPIHTTP->pResponseDB, PIDBTYPE_RFC822, pVariable, (void *)pValue, 0 );
	env->ReleaseStringUTFChars(name, pVariable);
	env->ReleaseStringUTFChars(value, pValue);
};

/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    getHttpDate
 * Signature: (J)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_getHttpDate__J
(JNIEnv *env, jobject sutils, jlong date) {

	time_t pDate = date;
	const char *szDate = ctime(&pDate);
	struct tm pT;
	if (HTTPUtil_readTime( szDate, &pT ) == PIAPI_COMPLETED) {
		Pi3String *pResult = Pi3String_new( 0 );
		jstring result;
		if (HTTPUtil_rFC822Time( &pT, pResult ) == PIAPI_COMPLETED) {
			result = env->NewStringUTF((char *)pResult);
		} else {
			result = env->NewStringUTF((char *)pResult);
		};
		Pi3String_delete( pResult );
		return result;
	};
	return env->NewStringUTF("");
};

/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    getHttpDate
 * Signature: (Ljava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_getHttpDate__Ljava_lang_String_2
(JNIEnv *env, jobject sutils, jstring date) {

	jboolean isCopy;
	const char *szDate = env->GetStringUTFChars(date, &isCopy);
	struct tm pT;
	if (HTTPUtil_readTime( szDate, &pT ) == PIAPI_COMPLETED) {
		return mktime(&pT);
	};
	return -1;
};

/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    getResponseVariable
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_getResponseVariable
(JNIEnv *env, jobject sutils, jstring name) {

	jclass cls = env->GetObjectClass(sutils);
	jmethodID getServletHandler;
	getServletHandler = env->GetMethodID( cls, "getServletHandler", "()I");
	// Errorhandling!
	if (!getServletHandler) { return env->NewStringUTF(""); };
	env->ExceptionClear();

	int SHandler = env->CallIntMethod(sutils, getServletHandler);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return env->NewStringUTF("");
		};

	jmethodID getPiHttp;
	getPiHttp = env->GetMethodID( cls, "getPiHttp", "()I");
	// Errorhandling!
	if (!getPiHttp) { return env->NewStringUTF(""); };
	env->ExceptionClear();

	int PiHttp = env->CallIntMethod(sutils, getPiHttp);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return env->NewStringUTF("");
		};

	jboolean isCopy;
	const char *pName = env->GetStringUTFChars(name, &isCopy);
	Servlet *sh = (Servlet *)SHandler;

	const char *pValue = sh->GetResponseVariable((PIHTTP *)PiHttp, (char *)pName);
	env->ReleaseStringUTFChars(name, pName);
	return pValue ? env->NewStringUTF( pValue ) : NULL;
};

/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    getRequestVariable
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_getRequestVariable
(JNIEnv *env, jobject sutils, jstring name) {

	jclass cls = env->GetObjectClass(sutils);
	jmethodID getServletHandler;
	getServletHandler = env->GetMethodID( cls, "getServletHandler", "()I");
	// Errorhandling!
	if (!getServletHandler) { return env->NewStringUTF(""); };
	env->ExceptionClear();

	int SHandler = env->CallIntMethod(sutils, getServletHandler);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return env->NewStringUTF("");
		};

	jmethodID getPiHttp;
	getPiHttp = env->GetMethodID( cls, "getPiHttp", "()I");
	// Errorhandling!
	if (!getPiHttp) { return env->NewStringUTF(""); };
	env->ExceptionClear();

	int PiHttp = env->CallIntMethod(sutils, getPiHttp);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return env->NewStringUTF("");
		};

	jboolean isCopy;
	const char *pName = env->GetStringUTFChars(name, &isCopy);
	Servlet *sh = (Servlet *)SHandler;

	const char *pValue = sh->GetRequestVariable((PIHTTP *)PiHttp, (char *)pName);
	env->ReleaseStringUTFChars(name, pName);
	return pValue ? env->NewStringUTF( pValue ) : NULL;
};


/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    getRequestVariables
 * Signature: (Ljava/util/Hashtable;)V
 */
JNIEXPORT void JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_getRequestVariables
(JNIEnv *env, jobject sutils, jobject hreq) {

	jclass cls = env->GetObjectClass(sutils);
	jmethodID getPiHttp;
	getPiHttp = env->GetMethodID( cls, "getPiHttp", "()I");
	// Errorhandling!
	if (!getPiHttp) { return; };
	env->ExceptionClear();

	int PiHttp = env->CallIntMethod(sutils, getPiHttp);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return;
		};

	cls = env->GetObjectClass(hreq);
	jmethodID putMID;
	putMID = env->GetMethodID( cls, "put",
		"(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;");
	// Errorhandling!
	if (!putMID) { return; };
	env->ExceptionClear();

	/* ---
	iterate through the request DB	
	--- */
	PIHTTP *pPIHTTP = (PIHTTP *)PiHttp;
	const char *pKey;
	const char *pValue;
	PIDBIterator *pIter = PIDB_getIterator( pPIHTTP->pRequestDB,
		PIDBTYPE_RFC822, 0, 0 );
	if ( pIter ) {
		for( ; PIDBIterator_atValidElement( pIter ); PIDBIterator_next( pIter ) ) {

			pValue = (const char *)PIDBIterator_current( pIter, &pKey );
			
			env->CallObjectMethod(hreq, putMID, env->NewStringUTF( pKey ),
				env->NewStringUTF( pValue ));
			if ( env->ExceptionOccurred() )
				{
				env->ExceptionDescribe();
				env->ExceptionClear();
				// Errorhandling!
				PIDBIterator_delete( pIter );
				return;
				};
		};
		PIDBIterator_delete( pIter );
	};
};

/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    sendError
 * Signature: (I)V
 */
JNIEXPORT jint JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_sendError
(JNIEnv *env, jobject sutils, jint sc) {

	jclass cls = env->GetObjectClass(sutils);
	jmethodID getPiHttp;
	getPiHttp = env->GetMethodID( cls, "getPiHttp", "()I");
	// Errorhandling!
	if (!getPiHttp) { return PIAPI_ERROR; };
	env->ExceptionClear();

	int PiHttp = env->CallIntMethod(sutils, getPiHttp);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return PIAPI_ERROR;
		};
	PIHTTP *pPIHTTP = (PIHTTP *)PiHttp;
	char *szBuf;
	PIIOBuffer_getOutputBuffer( pPIHTTP->pBuffer, &szBuf );
	PIIOBuffer_advanceBufferPointer( pPIHTTP->pBuffer, 0);

	// returns INT_REDIRECT
	return HTTPUtil_doHTTPError(pPIHTTP, sc);
};


/*
 * Class:     org_pi3_servlet_util_Pi3ServletUtils
 * Method:    setResponseVariable
 * Signature: (Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT void JNICALL Java_org_pi3_servlet_util_Pi3ServletUtils_setResponseVariable
(JNIEnv *env, jobject sutils, jstring name, jstring value) {

	jclass cls = env->GetObjectClass(sutils);
	jmethodID getPiHttp;
	getPiHttp = env->GetMethodID( cls, "getPiHttp", "()I");
	// Errorhandling!
	if (!getPiHttp) { return; };
	env->ExceptionClear();

	int PiHttp = env->CallIntMethod(sutils, getPiHttp);
	if ( env->ExceptionOccurred() )
		{
		env->ExceptionDescribe();
		env->ExceptionClear();
		// Errorhandling!
		return;
		};

	jboolean isCopy;
	const char *pName = env->GetStringUTFChars(name, &isCopy);
	const char *pValue = env->GetStringUTFChars(value, &isCopy);
	PIHTTP *pPIHTTP = (PIHTTP *)PiHttp;

	PIDB_replace( pPIHTTP->pResponseDB, PIDBTYPE_RFC822, pName, (void *)pValue, 0 );
	env->ReleaseStringUTFChars(name, pName);
	env->ReleaseStringUTFChars(value, pValue);
};

JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void *reserved) { return JNI_VERSION_1_2; };

JNIEXPORT void JNICALL JNI_OnUnload(JavaVM *vm, void *reserved) {}; 
