/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission. 

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Servlet/org/pi3/servlet/core/Pi3Reader.java,v $
 * $Date: 2003/05/13 18:42:21 $
 *
 Description:
	Implementation of Reader interface.

\*____________________________________________________________________________*/

package org.pi3.servlet.core;

import java.io.IOException;
import java.io.Reader;


/**
 *
 * Overrides the {@link java.io.Reader} class to get the client data
 * from the native side. You can access an object of this class by
 * using the {@link org.pi3.servlet.core.Pi3HttpServletRequest#getReader}
 * method.
 *
 * <p>This class implements the abstract method
 * <code>java.io.Reader.read(char[], int, int)</code>.
 *
 *
 * @see		org.pi3.servlet.core.Pi3HttpServletRequest#getReader
 *
 *
 * @author 	Holger Zimmermann [zimpel@pi3.org]
 * @version	$Revision: 1.5 $
 *
 *
 */

final public class Pi3Reader extends Reader {

    private int hobj;
    private int hsrv;
	private boolean closed = false;




	/** This constructor is used by the native side */	    
	protected Pi3Reader(int pihttp, int servlethandler) {
        hobj = pihttp;
        hsrv = servlethandler;
	}




    /** The method to get the PiHttp object from the native side */
    protected int getPiHttp() {
        return hobj;
    }




    /** The method to get the Servlet handler object from the native side */
    protected int getServletHandler() {
        return hsrv;
    }




    native private int internalRead(char[] cbuf, int off, int len) throws IOException;

    public int read(char[] cbuf, int off, int len) throws IOException {

        if (closed) throw new IOException();

	    /* shortcut */
	    if (len == 0) return 0;

	    /* array to small, sorry! */
        if (cbuf.length < off + len) throw new IOException();

        try {
            int iRes = internalRead(cbuf, off, len);
			if (iRes == -1) closed = true;
			return iRes;
        }
        catch (IOException e) {
            throw e;
        };
    };

	public void close() {
	    closed = true;
	};

    static {
        System.loadLibrary("Pi3ServletJNI");
    }
}
