/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission. 

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/SiteRoot/SiteRoot.cpp,v $
 * $Date: 2003/05/13 18:42:22 $
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <stdio.h>

#include "HTTPCore.h"
#include "HTTPUtil.h"
#include "HandBase.h"
#include "PIStrStr.h"
#include "Admin.h"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_CONFIGFILE         "ConfigFile"
#define KEY_CONF_LOGICOBJECT        "LogicObject" 
#define KEY_CONF_ROOTDIRECTORY      "RootDirectory"
#define KEY_CONF_REALM				"Realm"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    SiteRoot

Description:
    Dispatches between web and remote administration requests and maintains
	a local configuration file by processing remote administration requests.

Options:

<TABLE BORDER=1>
<TH>Option
<TH>Default
<TH>Values
<TH>Short Description
<TH>Example(s)

<TR>
<TD>ConfigFile
<TD>+
<TD>&lt;file name&gt;
<TD>The remotely administered configuration file
<TD>ConfigFile "./Config.pi3"

<TR>
<TD>LogicObject
<TD>+
<TD>&lt;object name&gt;
<TD>The top logic handler object to handle web requests
<TD>LogicObject "TopSiteLogic"

<TR>
<TD>Realm
<TD>-
<TD>&lt;string&gt;
<TD>The authentication realm for remote administration
<TD>Realm "RemoteAdmin"

<TR>
<TD>RootDirectory
<TD>-
<TD>&lt;path name&gt;
<TD>A root directory for this object
<TD>RootDirectory "../"

</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>
<H5>ConfigFile</H5>
The configuration file, which is maintained by this object using remote
administration. It can be the current configuration file as well as another
file containing the remotely administered sections. The path to the file can
be absolute or is relative to the server root.

<H5>LogicObject</H5>
This object is the top of the handler logic, which is called by the SiteRoot
object for non-administrative request.

<H5>Realm</H5>
The authentication realm for remote administration. And authentication object
for that realm must exist elsewhere in the configuration. For remote administration
it is strongly recommended to use SSL for channel encryption or at least Digest
Access Authentication for secure remote authentication.

<H5>RootDirectory</H5>
This sets the root directory for everything under this object, i.e. the path
root for the host object, which processes remote administration requests.

Example:
	<PRE>
	&lt;Object&gt;
		Name SiteRoot
		Class SiteRootClass
		LogicObject "TopSiteLogic"
		Realm "Administration"
		ConfigFile "./Conf/Config.pi3"
	&lt;/Object&gt;

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SiteRoot : public HandlerBaseSR
{
private:
    /* ---
    Fastkeys
    --- */
    const char *pFKAdminCommand;

	/* ---
	Configuration data
	--- */
    PIString sConfigFile;           /* the configuration file name for site */
    PIString sConfigFileTmp;        /* temp. config file for replace ops. */
    PISync *pMutex;                 /* mutex to synchronize access to data */
    PIObject *pLogicObject;         /* the logic object being used */
    int iInSiteCount;               /* current number of threads using site */
    PIObject *pNewLogicObject;      /* new site ready to be swapped in */
    PIDB *pDB;                      /* DB for object */
    PIString sLogicObjectName;      /* Name of logic object */
    PIString sRootDirectory;        /* The root directory */
    PIString sRealm;				/* The authentication realm */

protected:
	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );
		PIOStrStream os;
		os << pWhere << "SiteRoot: ";
		if ( 0 )
			{
			}
        else if ( !PIUtil_stricmp( KEY_CONF_LOGICOBJECT, pVariable ) )
            {
            sLogicObjectName = pValue;
            }
        else if ( !PIUtil_stricmp( KEY_CONF_ROOTDIRECTORY, pVariable ) )
            {
            Pi3String *pTmp = Pi3String_new( pValue );
            HTTPCore_relativeToAbsolutePath( PIObject_getDB( Object() ),
                pValue, pTmp );
            sRootDirectory = Pi3String_getPtr( pTmp );
            Pi3String_delete( pTmp );

            /*
            ** Set this as the root directory for everything under this
            ** object
            */
            PIDB_replace( PIObject_getDB( Object() ), PIDBTYPE_STRING,
                KEY_INT_HOSTROOT, (void *)(const char *)sRootDirectory, 0 );
            }
        else if ( !PIUtil_stricmp( KEY_CONF_CONFIGFILE, pVariable ) )
            {
            sConfigFile = pValue;
            }
        else if ( !PIUtil_stricmp( KEY_CONF_REALM, pVariable ) )
            {
            sRealm = pValue;
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	SiteRoot( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	HandlerBaseSR( pObject ),
        pFKAdminCommand( PIDB_getFastKey( KEY_HTTP_X_ADMINCOMMAND,
            PIDBTYPE_RFC822 ) ),
        pMutex( PIPlatform_allocLocalMutex() ),
        pLogicObject( 0 ),
        iInSiteCount( 0 ),
        pNewLogicObject( 0 ),
        pDB( 0 )
		{
        /*
        ** Reset the DB for future object loads/unloads
        */
        pDB = PIDB_new( PIObject_getDB(pObject), "SiteRoot" );
        PIObject_setDB( pObject, pDB );

		ReadParameters( iArgc, ppArgv );

        if ( !IsOK() )
            { return; };

        if ( sConfigFile.Len()==0 )
            {
            CONFIG_ERR( Object(), "SiteRoot: 'ConfigFile' not defined" );
            SetOK( 0 );
            return;
            };

        /*
        ** Convert configuration file to absoluate format.
        */
        Pi3String *pTmp = Pi3String_new( sConfigFile );
        HTTPCore_relativeToAbsolutePath( PIObject_getDB( Object() ),
            sConfigFile, pTmp );
        sConfigFile = Pi3String_getPtr( pTmp );
        Pi3String_delete( pTmp );
        sConfigFileTmp = sConfigFile;
        sConfigFileTmp.Concatenate( ".new" );

        pLogicObject = LoadConfigurationFile( sConfigFile, 0 );
        if ( !pLogicObject ) 
            {
            SetOK( 0 );
            return;
            };
        };

	~SiteRoot()
		{
        assert( pNewLogicObject==0 );
        assert( iInSiteCount==0 );
        PISync_delete( pMutex );
        PIObject_delete( pLogicObject, 0, 0 );
        if ( pDB )
            {
            PIObject_setDB( Object(), PIDB_getParent( pDB ) );
            PIDB_delete( pDB );
            pDB = 0;
            };
		};

    /*
    ** Load up the logic object for this site from the specified
    ** configuration file
    */
    PIObject *LoadConfigurationFile( const char *pFile, int iNewConfig )
        {
        PIDB *pDB = PIObject_getDB( Object() );
        PIConfig *pConfigDB = 0;
        PIObject *pObject = 0;
        if ( PIConfig_loadConfigurationFile( pDB, pFile, &pConfigDB ) )
            {
            /*
            ** Write the error message into the same file that the 
            ** configuration was written into, after deleting it if this
            ** is a new configuration.
            */
            if ( iNewConfig )
                {
                ::remove( pFile );
                PILog_writeDiagnostics( pDB, PILOG_ALL, pFile );
                };
            return 0;
            };

        pObject = PIObject_load( pDB, pConfigDB, sLogicObjectName, 0, 0 );
        if ( !pObject )
            {
            /*
            ** Write the error message into the same file that the 
            ** configuration was written into, after deleting it if
            ** this is a new configuration
            */
            if ( iNewConfig )
                {
                ::remove( pFile );
                PILog_writeDiagnostics( pDB, PILOG_ALL, pFile );
                };
            };

        /*
        ** Delete the configuration DB 
        */
        PIDB_delete( (PIDB *)pConfigDB );

        /* ---
        Delete the configuration DB
        --- */
        PIDBIterator *pIter = PIDB_getIterator( pDB, PIDBTYPE_TREE,
            PIDBKEY_CONFIGURATION, 0 );
        if ( pIter )
            {
            for( ;
                PIDBIterator_atValidElement( pIter );
                PIDBIterator_next( pIter )
                )
                {
                PIDBIterator_removeCurrent( pIter );
                };
            PIDBIterator_delete( pIter );
            };

        /*
        ** The new object has been loaded
        */
        return pObject;
        };

    /*
    ** Send the configuration file to the client
    */
    int SendConfigFile( PIHTTP &tPIHTTP )
        {
        enum { BUF_SIZE=63 };
        char szBuf[BUF_SIZE+1];
        unsigned long ulLen;

		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pQ = tPIHTTP.pRequestDB;
        int iRet = PIAPI_ABORT;
        PIFInfo *pFInfo = PIFInfo_new( sConfigFile );

        if ( !pFInfo || !PIFInfo_exists( pFInfo ) ||
            !PIFInfo_isRegular( pFInfo ) )
            {
            HTTPCore_logError( &tPIHTTP, "SiteRoot: Configuration file \
'%s' does not exist or is not a regular file (internal error).",
                (const char *)sConfigFile );
            iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
            goto done;
            };

        /*
        ** Set the content type and content length
        */
        /* --- content type --- */
        PIDB_add( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE, (void *)ADMIN_CONTENTTYPE,
            0 );

        /* --- content-length --- */
        ulLen =  PIFInfo_getSize( pFInfo );
        sprintf( szBuf, "%lu", ulLen );
        PIDB_add( pR, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, szBuf, 0 );

        /* --- status --- */
        tPIHTTP.iStatus = ST_OK;

        /* --- start response --- */
        if (    HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
                HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) )
            {
            HTTPCore_logError( &tPIHTTP, "%s", "SiteRoot: Failed to send \
response headers for admin command." );
            iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
            goto done;
            };

        /* --- send the file --- */
        iRet = HTTPUtil_sendFile( tPIHTTP.pBuffer, pFInfo, PIIOBUF_NONE, 0 );

done:
        if ( pFInfo ) { PIFInfo_delete( pFInfo ); };

        return iRet;
        };

    /*
    ** Swallow up extranious input from client, typically in the
    ** event of an error.
    */
    void SwallowInput( PIIOBuffer *pBuffer, int iLen )
        {
        int iRead;
        for( ; iLen>0 && PIIOBuffer_read( pBuffer, &iRead ); iLen-=iRead );
        assert( iLen==0 );
        };

    /*
    ** Get the configuration file from the remote client
    */
    int GetConfigFile( PIHTTP &tPIHTTP )
        {
        enum { FILE_BUF_SIZE=4096 };
        char szFileBuf[FILE_BUF_SIZE];
        int iRet = PIAPI_ABORT;
        int iTmp;

		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pQ = tPIHTTP.pRequestDB;

        /*
        ** Verify the content length and content type for
        ** sanity
        */
        const char *pContentLength = (const char *)
            PIDB_lookup( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH, 0 );
        const char *pContentType = (const char *)
            PIDB_lookup( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE, 0 );
        int iContentLength = pContentLength ? atoi( pContentLength ) : 0;
        if ( iContentLength<=0 || !pContentType ||
            PIUtil_stricmp( pContentType, ADMIN_CONTENTTYPE ) )
            {
            HTTPCore_logError( &tPIHTTP, "%s", "SiteRoot: \
Bad body content headers or bad length or configuration file." );
            return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
            };
        /*
        ** Attempt to open the file
        */
        PIPLATFORM_FD tFd = PIFile_open( sConfigFileTmp, "wn" );
        if ( tFd == PIPLATFORM_FD_INVALID )
            {
            HTTPCore_logError( &tPIHTTP, "SiteRoot: \
Failed to open configuration temporary file '%s' for writing. Perhaps the \
configuration is being updated by two administrators at the same time. \
Configuration file upload failed.", 
                (const char *)sConfigFileTmp );
            SwallowInput( tPIHTTP.pBuffer, iContentLength );
            return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
            };

        /*
        ** Now read from the input buffer and write to the file
        */
        for(;;)
            {
            iTmp = (iContentLength<FILE_BUF_SIZE) ? iContentLength :
                FILE_BUF_SIZE;
            iTmp = PIIOBuffer_readToBuffer( tPIHTTP.pBuffer, szFileBuf, iTmp );
            switch( iTmp )
                {
                case 0:
                case -1:
                    /*
                    ** We didn't read sufficient data
                    */
                    HTTPCore_logError( &tPIHTTP, "SiteRoot: \
Failed to read the specificied 'Content-Length' of data from remote \
administration client. Temporary configuration file '%s' will be removed. \
Configuration file upload failed.",  (const char *)sConfigFileTmp );
                    iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
                    goto close_file;

                default:
                    /* good */
                    break;
                };

            assert( iTmp>0 );
            iContentLength -= iTmp;
            assert( iContentLength >= 0 );

            /*
            ** Write to the file
            */
            iTmp = PIFile_write( tFd, iTmp, szFileBuf );
            if ( iTmp )
                {
                /*
                ** Error
                */
                HTTPCore_logError( &tPIHTTP, "SiteRoot: \
Failed to write to temporary configuration file '%s'. File upload failed.",
                    (const char *)sConfigFileTmp );
                SwallowInput( tPIHTTP.pBuffer, iContentLength );
                iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
                goto close_file;
                };

            if ( !iContentLength )
                {
                iRet = PIAPI_COMPLETED;
                break;
                };
            };
        
close_file:
        /*
        ** The temporary file has been read - close it and consider
        ** making it the main configuration file
        */
        iTmp = PIFile_close( tFd );
        if ( iTmp!=PIAPI_COMPLETED )
            {
            HTTPCore_logError( &tPIHTTP, "SiteRoot: \
Failed to close temporary configuration file '%s'. Configuration file upload \
failed.", (const char *)sConfigFileTmp );
            iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
            };

        /*
        ** Attempt to load the new configuration
        */
        PIObject *pTmpObject = 0;
        if ( iRet!=INT_REDIRECT )
            {
            pTmpObject = LoadConfigurationFile( sConfigFileTmp, 1 );
  
            if ( !pTmpObject )
                {
                /*
                ** An error occurred, attempt to map the configuration
                ** file where the error will have been written
                */
                PIFInfo *pFInfo = 0;
                PIFMap *pMap = 0;
                const char *pFileMap = 0;
                int iLen = 0;

                pFInfo = PIFInfo_new( sConfigFileTmp );
                pMap = pFInfo ? PIFMap_new( pFInfo ) : 0 ;
                pFileMap = pMap ? (const char *)PIFMap_lock( pMap, &iLen ) : 0;
                if ( pFileMap )
                    {
                    HTTPCore_logError( &tPIHTTP, "SiteRoot: \
Failed to load new site configuration file '%s'. Detailed error information \
follows:\n%s", (const char *)sConfigFileTmp, pFileMap );
                    }
                else
                    {
                    HTTPCore_logError( &tPIHTTP, "SiteRoot: \
Failed to load new site configuration file '%s'.",
                        (const char *)sConfigFileTmp );
                    };

                if ( pMap ) { PIFMap_delete( pMap ); };
                if ( pFInfo ) { PIFInfo_delete( pFInfo ); };
                iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
                };
            };

        /*
        ** Send response to client?
        */
        if ( iRet!=INT_REDIRECT )
            {
            /*
            ** Send HTTP response to confirm that configuration has been
            ** accepted
            */
            /* --- start response --- */
            if (    HTTPCore_sendGeneralHeaders( &tPIHTTP ) ||
                    HTTPCore_sendEntityHeaders( &tPIHTTP, pR ) )
                {
                HTTPCore_logError( &tPIHTTP, "%s", "SiteRoot: Failed to send \
response headers for admin command." );
                iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
                };
            };

        /*
        ** Do we want to make this the configuration file?
        */
        if ( !tPIHTTP.iStatus || (tPIHTTP.iStatus>199 && tPIHTTP.iStatus<300))
            {
            /*
            ** Make this the new configuration.
            */
            iRet = remove( sConfigFile );
            assert( !iRet );

            iRet = rename( sConfigFileTmp, sConfigFile );
            assert( !iRet );

            pNewLogicObject = pTmpObject;
            if ( iInSiteCount )
                {
                /*
                ** There are threads currently executing the old logic
                ** object: let last thread out pick up the change.
                */
                assert( iInSiteCount>0 );
                pTmpObject = 0;
                }
            else
                {
                /*
                ** There are no threads executing the current logic
                ** object
                */
                pTmpObject = pLogicObject;
                pLogicObject = pNewLogicObject;
                pNewLogicObject = 0;
                };

            iRet = PIAPI_COMPLETED;
            }
        else
            {
            /*
            ** Nuke the temporary file
            */
            remove( sConfigFileTmp );
            };

        /*
        ** Delete any logic objects 
        */ 
        if ( pTmpObject )
            {
            PIObject_delete( pTmpObject, 0, 0 );
            };

        /*
        ** Done
        */
        return iRet;
        };

    /*
    ** Handle admin requests
    */
    int HandleAdminRequest( int iPhase, PIHTTP &tPIHTTP,
        const char *pAdminCommand )
        {
        /*
        ** Only handle the request is the current status is a
        ** non-error status
        */
        if (tPIHTTP.iStatus && (tPIHTTP.iStatus <200 || tPIHTTP.iStatus>299))
            {
            /* --- this is an error let something else handle it --- */
            return PIAPI_CONTINUE;
            };

        switch( iPhase )
            {
            case PH_INIT:
            case PH_HEADERS:
                assert( 0 );
                return PIAPI_COMPLETED;

            case PH_MAPPING:
                /*
                ** Set the physical path
                */
                PIDB_replace( tPIHTTP.pResponseDB, PIDBTYPE_STRING, 
                    KEY_INT_PATH, (void *)(const char *)sConfigFile, 0 );
				if (sRealm.Len())
					{
					PIDB_replace( tPIHTTP.pResponseDB, PIDBTYPE_STRING,
						KEY_INT_AUTHENTICATIONREALM, (void *)(const char *)sRealm, 0 );
					}
                return PIAPI_COMPLETED;

            case PH_CHECKPATH:
                return PIAPI_COMPLETED;

            case PH_CHECKAUTH:
		        /*
				** Do site request for this phase
				*/
				{
		        PIHTTP *pPIHTTP = &tPIHTTP;
				return PILogic_execute( pLogicObject, 1, (const char **)&pPIHTTP );
				}
            case PH_CHECKACCESS:
            case PH_CHECKTYPE:
                return PIAPI_COMPLETED;

            case PH_HANDLE:
                break;

            case PH_HOSTMAP:
            case PH_LOG:
            case PH_DESTROY:
                return PIAPI_CONTINUE;

            default:
                assert( 0 );
                return PIAPI_CONTINUE;
            };

        int iRet;
        if ( !PIUtil_stricmp( pAdminCommand, KEY_ADMIN_GETCONFIG ) )
            {
            PISync_lock( pMutex );
            iRet = SendConfigFile( tPIHTTP );
            PISync_unlock( pMutex );
            }
        else if ( !PIUtil_stricmp( pAdminCommand, KEY_ADMIN_SAVECONFIG ) )
            {
            PISync_lock( pMutex );
            iRet = GetConfigFile( tPIHTTP );
            PISync_unlock( pMutex );
            }
        else
            {
            HTTPCore_logError( &tPIHTTP, "SiteRoot: Unknown admin command \
'%s'.", pAdminCommand );
            iRet = HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
            };

        return iRet;
        };

    /*
    ** Handle requests for this site
    */
    int HandleSiteRequest( int iPhase, PIHTTP &tPIHTTP )
        {
again:
        PISync_lock( pMutex );
        if ( iPhase==PH_INIT && pNewLogicObject )
            {
            /*
            ** A new configuration is ready to be used, waiting for
            ** threads to stop executing objects within this 
            ** configuration.
            */
            assert( iInSiteCount>0 );
            PISync_unlock( pMutex );
            PIPlatform_sleep( 2000 );     /* high tech. solution */
            goto again;
            };
        assert( pLogicObject );
        iInSiteCount++;
        PISync_unlock( pMutex );

        /*
        ** Do site request for this phase
        */
        PIHTTP *pPIHTTP = &tPIHTTP;
        int iRet = PILogic_execute( pLogicObject, 1, (const char **)&pPIHTTP );

        PIObject *pTmpObject = 0;
        PISync_lock( pMutex );
        iInSiteCount--;
        if ( iPhase==PH_DESTROY && !iInSiteCount && pNewLogicObject )
            {
            /*
            ** No other threads are using the objects for this site.
            ** the new site may be used and the old one destroyed
            */
            pTmpObject = pLogicObject;
            pLogicObject = pNewLogicObject;
            pNewLogicObject = 0;
            };
        PISync_unlock( pMutex );

        /*
        ** Now we are outside the mutex we can delete the old logic
        ** object if a new one was copied over
        */
        if ( pTmpObject )
            {
            PIObject_delete( pTmpObject, 0, 0 );
            };

        return iRet;
        };

    /*
    ** The main function
    */
	int Handle( int iPhase, PIHTTP &tPIHTTP, PIIOBuffer &/* tBuffer */ )
		{
		int iRet;
		PIDB *pQ = tPIHTTP.pRequestDB;

		const char *pAdminCommand = (const char *)PIDB_lookup( pQ,
			PIDBTYPE_RFC822, pFKAdminCommand, PIDBFLAG_FASTKEY );

       	if ( pAdminCommand && strlen(pAdminCommand) )
           	{
           	/*
           	** This is an admin command
           	*/
           	iRet = HandleAdminRequest( iPhase, tPIHTTP, pAdminCommand );
			if (iRet == INT_REDIRECT)
				{
				PIDB_replace( pQ, PIDBTYPE_RFC822, pFKAdminCommand,
					NULL, PIDBFLAG_FASTKEY );
				}
#if 0
		cout << "Admin request: " << HTTPUtil_phaseNumberToName(iPhase)
			<< ", " << HTTPUtil_rcNumberToName(iRet)<< ", " << tPIHTTP.iStatus << endl;
#endif
            }
		else
			{
			iRet = HandleSiteRequest( iPhase, tPIHTTP );
#if 0
		cout << "Site request: " << HTTPUtil_phaseNumberToName(iPhase)
			<< ", " << HTTPUtil_rcNumberToName(iRet) << ", " << tPIHTTP.iStatus << endl;
#endif
			}

		return iRet;
        }
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int SiteRoot_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return HandlerBaseSR_constructor( pObj, PI_NEW( SiteRoot( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name SiteRootClass
		Type LogicExtension
		Library SiteRoot
		OnClassLoad HandlerBaseSR_onClassLoad
		Constructor SiteRoot_constructor
		CopyConstructor HandlerBaseSR_copyConstructor
		Destructor HandlerBaseSR_destructor
		Execute HandlerBaseSR_execute
	</Class>

	<Object>
		Name SiteRoot
		Class SiteRootClass
	</Object>

/*___+++CNF_END+++___*/
#endif

