/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2003 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Util/Base64.cpp,v $
 * $Date: 2003/05/13 18:42:22 $
 *
 Description:
\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "Base64.h"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define assert(x)

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
    Base64 encode
\*____________________________________________________________________________*/
const char __pBase64[65]="\
ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

void Base64Encode(	const char *pPlain,
					char *pBuffer,
					int iBufferLen )
{
    int iPos;
	int iLen;
	int i;
	enum { START, TWO, FOUR, SIX } eState;

    pBuffer[iBufferLen-1] = '\0';
    iBufferLen -= 2;
    iPos = 0;

#define CONCATENATE(c) \
    { \
    if ( iPos < iBufferLen ) \
        { pBuffer[iPos++] = (c); }; \
    }

	assert( pPlain );
	iLen = strlen( pPlain );
	eState = START;
	i=0;
	for( ;; )
		{
		int iPlain;
		int iBase64;
		char c = (i<iLen) ? pPlain[i] : 0 ;
		switch( eState )
			{
			case START:
				if ( i>=iLen )
                    {
                    CONCATENATE('\0');
                    return;
                    };
				iBase64 = (c >> 2);
				iPlain = (c << 4 ) & 0x30;
				i++;
				eState = TWO;
				break;

			case TWO:
				iBase64 = iPlain | ( c >> 4 );
				iPlain = (c << 2 ) & 0x3C;
				i++;
				eState = FOUR;
				break;
				
			case FOUR:
				iBase64 = iPlain | ( c >> 6 );
				iPlain = c & 0x3F;
				i++;
				eState = SIX;
				break;
				
			case SIX:
				iBase64 = iPlain;
				eState = START; 
				break;

			default:;
			};
		
		CONCATENATE( __pBase64[iBase64 & 0x3F] ); 

		if ( i>iLen )
			{
			switch( eState )
				{
				case START:
				case TWO:
					CONCATENATE( '=' );
				case FOUR:
					CONCATENATE( '=' );
				case SIX:
					CONCATENATE( '=' );
					CONCATENATE( '\0' );
					return;
				default:;
					assert( 0 );
				};
			};
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
    Base64 Decode
\*____________________________________________________________________________*/
const char __pPlain[128]={
/*   0.. 15 */ -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1, -1, -1, -1, -1, -1,
/*  16.. 31 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
/*  32.. 47 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
/*  48.. 63 */ 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1,  0, -1, -1,
/*  64.. 79 */ -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
/*  80.. 95 */ 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
/*  96..111 */ -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
/* 112..127 */ 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1
};
void Base64Decode(	const char *pBase64,
					char *pBuffer,
					int iBufferLen )
{
    int iPos;
	int iLen;
	enum { START, TWO, FOUR, SIX } eState;
	int iDone;
	int i;

    iPos = 0;
    pBuffer[iBufferLen-1] = '\0';
    iBufferLen--;
#define CONCATENATE(c) \
    { \
    if ( iPos < iBufferLen ) \
        { pBuffer[iPos++] = (c); }; \
    }

	assert( pBase64 );
	iLen = strlen( pBase64 );
	eState = START;
	iDone = 0;
	for( i=0; i<iLen; i++)
		{
		char c = pBase64[i];
		int iLast;
		int iPlain;
		if ( !isalnum(c) && (c!='+') && (c!='/') ) { break; };
		iPlain = __pPlain[c];
		assert( iPlain!= -1 );
		switch( eState )
			{
			case START:
				iLast = ( iPlain << 2 ) & 0xFC;
				eState = TWO;
				break;

			case TWO:
				CONCATENATE( iLast | ( (iPlain >> 4 ) & 0x03) );
				iLast = ( iPlain << 4 ) & 0xF0;
				eState = FOUR;
				break;
				
			case FOUR:
				CONCATENATE( iLast | ( (iPlain >> 2 ) & 0x0F) );
				iLast = ( iPlain << 6 ) & 0xC0;
				eState = SIX;
				break;
				
			case SIX:
				CONCATENATE( iLast | ( iPlain & 0x3F) );
				eState = START; 
				break;

			default:;
			};

		if ( c=='=')
			{ break; };
		};
		pBuffer[iPos] = '\0';
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Not very efficient, but it doesn't really need to be.
\*____________________________________________________________________________*/
void Base64Encode( const char *pPlain, PIString &sResult )
{
	assert( pPlain );
	int iLen = strlen( pPlain );
	enum { START, TWO, FOUR, SIX } eState = START;
	int i=0;
	for( ;; )
		{
		int iPlain;
		int iBase64;
		char c = (i<iLen) ? pPlain[i] : 0 ;
		switch( eState )
			{
			case START:
				if ( i>=iLen ) { return; };
				iBase64 = (c >> 2);
				iPlain = (c << 4 ) & 0x30;
				i++;
				eState = TWO;
				break;

			case TWO:
				iBase64 = iPlain | ( c >> 4 );
				iPlain = (c << 2 ) & 0x3C;
				i++;
				eState = FOUR;
				break;
				
			case FOUR:
				iBase64 = iPlain | ( c >> 6 );
				iPlain = c & 0x3F;
				i++;
				eState = SIX;
				break;
				
			case SIX:
				iBase64 = iPlain;
				eState = START; 
				break;

			default:;
			};
		
		sResult.Concatenate( __pBase64[iBase64 & 0x3F] );

		if ( i>iLen )
			{
			switch( eState )
				{
				case START:
				case TWO:
					sResult.Concatenate( '=' );
				case FOUR:
					sResult.Concatenate( '=' );
				case SIX:
					sResult.Concatenate( '=' );
					return;
				default:;
					assert( 0 );
				};
			};
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Not very efficient, but it doesn't really need to be.
\*____________________________________________________________________________*/
void Base64Decode( const char *pBase64, PIString &sResult )
{
	assert( pBase64 );
	int iLen = strlen( pBase64 );
	enum { START, TWO, FOUR, SIX } eState = START;
	int iDone = 0;
	for( int i=0; i<iLen; i++)
		{
		char c = pBase64[i];
		if ( !isalnum(c) && (c!='+') && (c!='/') ) { break; };
		int iLast;
		int iPlain = __pPlain[c];
		assert( iPlain!= -1 );
		switch( eState )
			{
			case START:
				iLast = ( iPlain << 2 ) & 0xFC;
				eState = TWO;
				break;

			case TWO:
				sResult.Concatenate( iLast | ( (iPlain >> 4 ) & 0x03) );
				iLast = ( iPlain << 4 ) & 0xF0;
				eState = FOUR;
				break;
				
			case FOUR:
				sResult.Concatenate( iLast | ( (iPlain >> 2 ) & 0x0F) );
				iLast = ( iPlain << 6 ) & 0xC0;
				eState = SIX;
				break;
				
			case SIX:
				sResult.Concatenate( iLast | ( iPlain & 0x3F) );
				eState = START; 
				break;

			default:;
			};

		if ( c=='=')
			{ break; };
		};
}
